Package: HARplus
Title: Enhanced R Package for 'GEMPACK' .har and .sl4 Files
Version: 1.2.0
Authors@R: 
    person("Pattawee", "Puangchit", email = "ppuangch@purdue.edu", role = c("aut", "cre"))
Description: Provides tools for processing and analyzing .har and .sl4 files, making it easier 
    for 'GEMPACK' users and 'GTAP' researchers to handle large economic datasets. 
    It simplifies the management of multiple experiment results, enabling faster 
    and more efficient comparisons without complexity. Users can extract, restructure, 
    and merge data seamlessly, ensuring compatibility across different tools. 
    The processed data can be exported and used in 'R', 'Stata', 'Python', 'Julia', or 
    any software that supports Text, CSV, or 'Excel' formats.
License: MIT + file LICENSE
Encoding: UTF-8
BugReports: https://github.com/bodysbobb/HARplus/issues/
URL: https://github.com/bodysbobb/HARplus/,
        https://www.pattawee-pp.com/HARplus/
Imports: openxlsx, haven, stats, utils, tidyr, tools, tidyselect
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-03 02:16:49 UTC; b_pat
Author: Pattawee Puangchit [aut, cre]
Maintainer: Pattawee Puangchit <ppuangch@purdue.edu>
Repository: CRAN
Date/Publication: 2026-05-03 05:30:02 UTC
Built: R 4.5.2; ; 2026-05-03 05:54:21 UTC; unix

Package: HARplus
Title: Enhanced R Package for 'GEMPACK' .har and .sl4 Files
Version: 1.2.0
Authors@R: 
    person("Pattawee", "Puangchit", email = "ppuangch@purdue.edu", role = c("aut", "cre"))
Description: Provides tools for processing and analyzing .har and .sl4 files, making it easier 
    for 'GEMPACK' users and 'GTAP' researchers to handle large economic datasets. 
    It simplifies the management of multiple experiment results, enabling faster 
    and more efficient comparisons without complexity. Users can extract, restructure, 
    and merge data seamlessly, ensuring compatibility across different tools. 
    The processed data can be exported and used in 'R', 'Stata', 'Python', 'Julia', or 
    any software that supports Text, CSV, or 'Excel' formats.
License: MIT + file LICENSE
Encoding: UTF-8
BugReports: https://github.com/bodysbobb/HARplus/issues/
URL: https://github.com/bodysbobb/HARplus/,
        https://www.pattawee-pp.com/HARplus/
Imports: openxlsx, haven, stats, utils, tidyr, tools, tidyselect
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-03 02:16:49 UTC; b_pat
Author: Pattawee Puangchit [aut, cre]
Maintainer: Pattawee Puangchit <ppuangch@purdue.edu>
Repository: CRAN
Date/Publication: 2026-05-03 05:30:02 UTC
Built: R 4.6.0; ; 2026-05-03 06:23:46 UTC; unix

# Generated by roxygen2: do not edit by hand

export(compare_var_structure)
export(create_calc_config)
export(create_initial_config)
export(create_target_config)
export(export_data)
export(get_data_by_dims)
export(get_data_by_var)
export(get_dim_elements)
export(get_dim_patterns)
export(get_var_structure)
export(group_data_by_dims)
export(load_harx)
export(load_sl4x)
export(pivot_data)
export(pivot_data_hierarchy)
export(rename_dims)
export(save_har)
export(shock_calculate)
export(shock_calculate_uniform)
importFrom(haven,write_dta)
importFrom(openxlsx,addStyle)
importFrom(openxlsx,addWorksheet)
importFrom(openxlsx,createStyle)
importFrom(openxlsx,createWorkbook)
importFrom(openxlsx,saveWorkbook)
importFrom(openxlsx,writeData)
importFrom(stats,complete.cases)
importFrom(stats,setNames)
importFrom(tidyr,pivot_wider)
importFrom(tidyselect,all_of)
importFrom(tools,file_ext)
importFrom(tools,file_path_sans_ext)
importFrom(utils,read.csv)
importFrom(utils,tail)
importFrom(utils,write.csv)
importFrom(utils,write.table)

# HARplus: Enhanced R Package for GEMPACK .har and .sl4 Files
[![Author](https://img.shields.io/badge/Pattawee.P-blue?label=Author)](https://www.pattawee-pp.com/)
![Last Updated](https://img.shields.io/github/last-commit/Bodysbobb/HARplus?label=Last%20Updated&color=blue) 
[![Star](https://img.shields.io/github/stars/Bodysbobb/HARplus?style=social)](https://github.com/Bodysbobb/HARplus/stargazers)

[![CRAN](https://www.r-pkg.org/badges/version/HARplus)](https://CRAN.R-project.org/package=HARplus)
[![GitHub Version](https://img.shields.io/github/v/tag/Bodysbobb/HARplus?label=GitHub%20Version&color=3CB371&sort=semver)](https://github.com/Bodysbobb/HARplus/releases/latest)
[![License](https://img.shields.io/github/license/Bodysbobb/HARplus?color=3CB371)](https://github.com/Bodysbobb/HARplus/blob/main/LICENSE.md/)

**Extensions:**  
[![GTAPViz](https://img.shields.io/badge/GTAPViz-276DC3?style=flat-square&logo=r&logoColor=white)](https://www.pattawee-pp.com/GTAPViz/)

---

## Version 1.2.0 - Update (02-May-2026)

### Bug fixes
- Fixed an issue where `load_sl4x()` and `load_harx()` returned a `subscript out of bounds` error for macro variables.

### New features
- `save_har()` now supports writing `1C` headers for string-format data.
- `save_har()` now supports writing ViewHAR-recognized mapping headers through the new `mappings` argument.
- Added support for writing mixed HAR files containing string sets, mapping vectors, numeric data frames, sparse numeric data, and integer matrices in a single `save_har()` call.
- Added optional `header_type` control for explicitly defining header roles when automatic detection is not sufficient.
- Added stricter header-name validation to prevent silent duplication after GEMPACK's four-character header truncation.
- Improved mapping validation: missing or invalid mapping targets are written as `"0"` and reported with warnings.
- Clarified the `save_har()` interface: `value_cols` is used only for numeric value columns in data-frame inputs, while mapping behavior is defined separately through `mappings`.
- Updated `save_har()` documentation and examples for CRAN-style usage, including mixed-header exports and ViewHAR-compatible mapping exports.

---

## Overview

HARplus is an R package designed to process and analyze .HAR and .SL4 files, making it easier for GEMPACK users and GTAP model researchers to handle large economic datasets. It simplifies the management of multiple experiment results, enabling faster and more efficient comparisons without complexity.

With HARplus, users can extract, restructure, and merge data seamlessly, ensuring compatibility across different tools. The processed data can be exported and used in R, Stata, Python, Julia, or any software that supports .txt, CSV, or Excel formats.

---

## Key Features

- **Efficient Data Extraction** – Supports selective header loading and optimized memory usage for handling large `.HAR` and `.SL4` files.  
- **Flexible Data Structuring** – Extract variables by name or dimension patterns while ensuring consistency across multiple inputs.  
- **Customizable Aggregation & Merging** – Manage subtotals, merge datasets, and structure data dynamically.  
- **Multiple Export Options** – Output extracted data in CSV, Stata, RDS, and Excel formats with structured formatting.  
- **Powerful HAR Writing** – Includes `save_har()` for exporting datasets to GEMPACK `.HAR` format with full binary compliance.  
- **Designed for GEMPACK** – Ensures smooth integration with `.HAR` and `.SL4` files while offering additional flexibility.  
- **Ideal for GTAP Model Users** – Built specifically to process and analyze GTAP model results efficiently.  

---

## How It Works

HARplus simplifies `.HAR` and `.SL4` file processing. You can:
- Load files and selectively extract headers.
- Extract data by variable name or dimension patterns.
- Group, merge, and restructure data with ease.
- Pivot and export data into structured formats.
- Filter subtotals and rename dimensions for clarity.

---

## Installation

HARplus (version 1.1.2) can be installed directly in R using:
```r
install.packages("HARplus")
```

While the latest HARplus (version 1.2.0) can be installed from my GitHub using:
```r
devtools::install_github("Bodysbobb/HARplus")
```

## Quick Guide to HARplus

All commands in this package have several options that allow users to play around with the data more freely and efficiently, not just import and get the data. For a complete guide on HARplus functions, check out the **[Vignette](https://rpubs.com/Bodysbob/1273998/)** or **[GitHub Vignette](https://www.pattawee-pp.com/HARplus/)**

Below is a categorized reference of the main functions in HARplus:

### Data Importing
- **`load_harx()`** – Loads `.HAR` files with selective header extraction and structured metadata.  
- **`load_sl4x()`** – Loads `.SL4` files, extracting variable names and dimension structures.

### Data Extraction
- **`get_data_by_var()`** – Extracts specific variables from `.HAR` or `.SL4` datasets, supporting subtotal filtering and merging.  
- **`get_data_by_dims()`** – Extracts data based on dimension patterns, with options for merging and subtotal filtering.

### Data Structure Summary & Comparison
- **`get_dim_elements()`** – Lists unique dimension elements (e.g., `REG`, `COMM`).  
- **`get_dim_patterns()`** – Extracts unique dimension structures (e.g., `REG*COMM*ACTS`).  
- **`get_var_structure()`** – Summarizes variable names, dimensions, and data structure.  
- **`compare_var_structure()`** – Compares variable structures across multiple datasets for compatibility.

### Data Grouping & Processing
- **`group_data_by_dims()`** – Groups extracted data by dimension priority, with support for automatic renaming and subtotal handling.  
- **`rename_dims()`** – Renames dimension names for consistency.

### Data Transformation
- **`pivot_data()`** – Converts long-format data into wide format.  
- **`pivot_data_hierarchy()`** – Creates hierarchical pivot tables for structured reporting.

### Data Export
- **`export_data()`** – Exports extracted data to CSV, Stata, TXT, RDS, or XLSX, with support for multi-sheet exports.

### HAR Exporting (New in v1.1.1)
- **`save_har()`** – Saves processed data frames or arrays into GEMPACK-compatible `.HAR` files, automatically generating 1C set headers and supporting up to seven dimensions.  

#### Technical Specifications
- Supports both **1C (string)** and **RE (real)** headers  
- Automatically generates 1C dimension sets (e.g., `REG`, `COMM`, `ENDW`)  
- Accepts data frames or arrays with flexible column naming  
- Writes associated set headers when `export_sets = TRUE`  
- Maintains full GEMPACK binary structure with no size limitation  
- Supports up to seven dimensions and approximately 2 million elements per chunk  
- Allows **dimension renaming** and supports **duplicate dimension names** (e.g., `COMMxREGxREG`) during export (New in v1.1.2)  

### Shock Calculation Framework (New in v1.1.2)
These functions provide a complete workflow to **calculate, structure, and export GEMPACK-compatible shock files** directly from `.HAR`, `.SL4`, `.CSV`, or `.XLSX` datasets—eliminating the need for manual conversion when preparing dynamic simulation shocks.

- **`shock_calculate_uniform()`** – Calculates uniform percentage shocks across all base rates and **exports directly to GEMPACK `.HAR` format**. Supports additive (`+`, `-`) and multiplicative (`*`, `/`) adjustments.  
- **`shock_calculate()`** – Computes target-based shocks by comparing initial and target datasets, automatically **exporting the resulting shocks to `.HAR` files** with dynamic timeline headers (e.g., `ONEY`, `TWOY`, `THRY`, etc.).  
- **`create_initial_config()`**, **`create_target_config()`**, and **`create_calc_config()`** – Define input sources, column mappings, and timeline periods for use in both uniform and target-based shock calculations.  

---

## License & Author  

HARplus is released under the **MIT License**. See the full **[license](https://github.com/Bodysbobb/HARplus/blob/main/LICENSE.md)**.  

**Author:**  
**Pattawee Puangchit**  
Ph.D. Candidate, Agricultural Economics  
Purdue University  
Research Assistant at GTAP  

---

## Acknowledgements

Acknowledgement is due to **Maros Ivanic** for his work on the [`HARr`](https://github.com/USDA-REE-ERS/MTED-HARr/) package, which served as the foundation for HARplus. This package would not have been possible without his contributions.

---

## GTAPViz: An Extension of HARplus for Visualization

I have developed another package specifically for visualization, particularly for GTAP users: **[GTAPViz](https://www.pattawee-pp.com/GTAPViz/)**

---

## GTAP Database

Sample data used in this [vignette](https://rpubs.com/Bodysbob/1273998/) is obtained from the GTAPv7 model and utilizes publicly available data from the [GTAP 9 database](https://www.gtap.agecon.purdue.edu/databases/archives.asp). For more details about the GTAP database and model, refer to the **[GTAP Database](https://www.gtap.agecon.purdue.edu/)**.

---

# Older Version

## Version 1.1.3 - Update (30-Oct-2025)

- **Writing HAR files without limitations** – The new `save_har()` function fully supports writing `.HAR` files with no size restrictions, allowing up to seven dimensions and approximately two million elements per chunk.  
- **Shock calculation and HAR export** – Introduced `shock_calculate_uniform()` and `shock_calculate()` to compute and **export shock results directly into GEMPACK-compatible `.HAR` files**, supporting dynamic multi-period calculations (e.g., `ONEY`, `TWOY`, `THRY`, etc.) for recursive-dynamic simulations.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message = FALSE,
  eval = requireNamespace("HARplus", quietly = TRUE)
)

## ----load package-------------------------------------------------------------
library(HARplus)

## ----load data----------------------------------------------------------------
# Paths to the .har files
har_path1 <- system.file("extdata", "TAR10-WEL.har", package = "HARplus")
har_path2 <- system.file("extdata", "SUBT10-WEL.har", package = "HARplus")

# Paths to the .sl4 files
sl4_path1 <- system.file("extdata", "TAR10.sl4", package = "HARplus")
sl4_path2 <- system.file("extdata", "SUBT10.sl4", package = "HARplus")

# Load the .har files using load_harx()
har_data1 <- load_harx(har_path1)
har_data2 <- load_harx(har_path2)

# Load the .sl4 files using load_sl4x()
sl4_data1 <- load_sl4x(sl4_path1)
sl4_data2 <- load_sl4x(sl4_path2)

## ----get_data_by_var----------------------------------------------------------
# Extract data for a single variable
data_qo <- get_data_by_var("qo", sl4_data1)
print(head(data_qo[["sl4_data1"]][["qo"]], 4))

# Extract multiple variables from multiple datasets
data_multiple <- get_data_by_var(c("qo", "qgdp"), sl4_data1, sl4_data2)
print(head(data_multiple[["sl4_data1"]][["qo"]], 4))
print(head(data_multiple[["sl4_data2"]][["qo"]], 4))

# Extract all variables separately from multiple datasets
data_list <- get_data_by_var(NULL, sl4_data1, sl4_data2)
print(names(data_list))

# Extract all variables and merge the same variable from multiple datasets
data_all <- get_data_by_var(NULL, sl4_data1, sl4_data2, merge_data = TRUE)
print(names(data_all))

# Return all value levels
data_all <- get_data_by_var("qo", sl4_data1, sl4_data2, subtotal_level = TRUE)
print(head(data_all[["sl4_data1"]][["qo"]], 4))

# Return only TOTAL, drop subtotal
data_total <- get_data_by_var("qo", sl4_data1, sl4_data2, subtotal_level = FALSE)
print(head(data_total[["sl4_data1"]][["qo"]], 4))

# Return only subtotal, drop TOTAL (result is empty if there in subtotal)
data_decomp <- get_data_by_var("qo", sl4_data1, sl4_data2, subtotal_level = "decomposed")
print(head(data_decomp[["sl4_data2"]][["qo"]], 4))

# Rename specific columns
data_col_renamed <- get_data_by_var("qo", sl4_data1, 
                             rename_cols = c(REG = "Region", COMM = "Commodity"))
str(data_col_renamed)

# Rename experiment names
data_exp_renamed <- get_data_by_var("qo", sl4_data1, sl4_data2, 
                             experiment_names = c("EXP1", "EXP2"))
print(names(data_exp_renamed))

# Merge variable data across multiple datasets with custom experiment names
data_merged <- get_data_by_var(, sl4_data1, sl4_data2,
                            experiment_names = c("EXP1", "EXP2"), 
                            merge_data = TRUE,
                            rename_cols = c(REG = "Region", COMM = "Commodity"))
print(head(data_merged$merged[[1]], 4))


## ----get_data_by_dims---------------------------------------------------------
# Merge data by dimensions (e.g., REG*COMM != COMM*REG)
data_no_mix <- get_data_by_dims(NULL, sl4_data1, sl4_data2, 
                                merge_data = TRUE, 
                                pattern_mix = FALSE)

# Merge data while allowing interchangeable dimensions (e.g., REG*COMM = COMM*REG)
data_pattern_mixed <- get_data_by_dims(NULL, sl4_data1, sl4_data2, 
                                       merge_data = TRUE, 
                                       pattern_mix = TRUE)

## ----group_data_single_priority-----------------------------------------------
# Define single priority (Only Region-based grouping)
priority_list <- list("Region" = c("REG"))

# Grouping data with a single priority
grouped_data_single <- group_data_by_dims("ALL", sl4_data1, sl4_data2,
                                          priority = priority_list, auto_rename = TRUE)

# Print structure
print(names(grouped_data_single))
print(names(grouped_data_single[["1D"]]))
print(names(grouped_data_single[["2D"]]))

## ----group_data_multi_priority------------------------------------------------
# Define multiple priority levels: Sector first, then Region
priority_list <- list(
  "Sector" = c("COMM", "ACTS"),
  "Region" = c("REG")
)

# Grouping data with multiple priorities
grouped_data_multiple <- group_data_by_dims("ALL", sl4_data1, 
                                            priority = priority_list, 
                                            auto_rename = TRUE)

# Print structure
print(names(grouped_data_multiple))
print(names(grouped_data_multiple[["1D"]]))
print(names(grouped_data_multiple[["2D"]]))

## ----group_data_no_auto_rename------------------------------------------------
# Define priority: First by Sector (COMM, ACTS), then by Region (REG)
priority_list <- list(
  "Sector" = c("COMM", "ACTS"),
  "Region" = c("REG")
)

# Grouping data without auto_rename
grouped_data_no_rename <- group_data_by_dims("ALL", sl4_data1, 
                                             priority = priority_list, 
                                             auto_rename = FALSE)

# Print structure
print(names(grouped_data_no_rename))
print(names(grouped_data_no_rename[["1D"]]))
print(names(grouped_data_no_rename[["2D"]]))

## ----pivot--------------------------------------------------------------------
# Extract data to pivot
data_multiple <- get_data_by_var(c("qo", "pca"), sl4_data1)

# Pivot a single column
pivoted_single <- pivot_data(data_multiple, 
                             pivot_cols = "REG")

# Pivot multiple columns
pivoted_multi <- pivot_data(data_multiple, 
                            pivot_cols = c("COMM", "REG"))


## ----pivot_data_hierarchy, eval=FALSE-----------------------------------------
# # Create hierarchical pivot without export
# pivot_hier <- pivot_data_hierarchy(data_multiple,
#                                   pivot_cols = c("REG", "COMM"))
# 
# # Create and export to Excel in one step
# pivot_export <- pivot_data_hierarchy(data_multiple,
#                                    pivot_cols = c("REG", "COMM"),
#                                    export = TRUE,
#                                    file_path = file.path(tempdir(), "pivot_output.xlsx"))
# 

## ----rename-------------------------------------------------------------------
# Define a renaming map
mapping_df <- data.frame(
 old = c("REG", "COMM"),
 new = c("Region", "Commodity")
)

# Rename dimensions only
renamed_dims <- rename_dims(sl4_data1, mapping_df)

# Rename both dimensions and list names
renamed_both <- rename_dims(sl4_data1, mapping_df, rename_list_names = TRUE)

## ----save_har_single, eval=FALSE----------------------------------------------
# # Load example HAR data
# har_path <- system.file("extdata", "TAR10-WEL.har", package = "HARplus")
# har_data <- load_harx(har_path)
# 
# # Extract one matrix for demonstration
# welfare_data <- get_data_by_var("C17", har_data)
# welfare_matrix <- welfare_data[["har_data"]][["C17"]]
# 
# # Save as a single HAR file
# save_har(
#   data_list   = list(WELF = welfare_matrix),
#   file_path   = file.path(tempdir(), "output_single.har"),
#   dimensions  = list(WELF = c("REG", "COLUMN")),
#   value_cols  = c(WELF = "Value"),
#   long_desc   = c(WELF = "Welfare Decomposition"),
#   dim_rename  = list(RTMS = c(COMM = "COMM", SREG = "REG", DREG = "REG")),
#   export_sets = TRUE,
#   dim_order   = NULL,
#   lowercase   = FALSE
# )

## ----export data, eval=FALSE--------------------------------------------------
# # Extract data
# data_multiple <- get_data_by_var(c("qo", "pca"), sl4_data1)
# 
# # Export
# export_data(data_multiple, file.path(tempdir(), "output_directory"),
#            format = c("csv", "xlsx", "stata", "txt", "rds"),
#            create_subfolder = TRUE,
#            multi_sheet_xlsx = TRUE)

## ----get_var_structure--------------------------------------------------------
# (1) Getting all variables from the input file
vars_har_sum <- get_var_structure("ALL", har_data1)
vars_sl4_sum <- get_var_structure(, har_data1)

# (2) Getting selected variables
var_sl4_sum <- get_var_structure(c("pds","pfd","pms"), sl4_data1)
print(head(var_sl4_sum[["sl4_data1"]], 4))


# (3) Including column size and number of observation in the summary
var_sl4_sum <- get_var_structure(c("pds","pfd","pms"), sl4_data1, sl4_data2, 
                                 include_col_size = TRUE)
print(head(var_sl4_sum[["sl4_data1"]], 4))

## ----compare_var_structure----------------------------------------------------
# (1) Comparing all variable structures across experiments 
vars_comparison <- compare_var_structure(
  variables = "ALL", sl4_data1, sl4_data2
)
print(vars_comparison$match[1:2, ])

# (2) Comparing selected variable structures across experiments 
var_comparison <- compare_var_structure(
  variables = c("pds", "pms"), sl4_data1, sl4_data2
)
print(var_comparison$match[1:2, ])

## ----unique_compare_var_structure---------------------------------------------
# (3) Extracting unique variable structures
unique_vars <- compare_var_structure(, 
                                     sl4_data1, sl4_data2,
  keep_unique = TRUE
)
print(unique_vars$match[1:10, ])

## ----get_dim_patterns---------------------------------------------------------
# (1) Extracting dimension patterns (e.g., REG*COMM*ACTS)
dims_strg_har <- get_dim_patterns(har_data1, har_data2)
print(dims_strg_har[1:4, ])

# (2) Extracting unique dimension patterns (e.g., REG*COMM*ACTS)
dims_strg_har <- get_dim_patterns(har_data1, har_data2,
                                  keep_unique =TRUE)
print(dims_strg_har[1:4, ])

# (2) Extracting dimension elements e.g., REG, COMM, ACTS
dims_strg_har_uniq <- get_dim_elements(har_data1, har_data2, 
                                       keep_unique =TRUE)
print(dims_strg_har_uniq[1:4, ])

#' @title Load and Process SL4 Files with Enhanced Options
#' 
#' @description Reads an SL4 file and processes its structured data into an enhanced SL4 object. 
#' Extracts structured variable information, dimensions, and handles subtotal columns.
#'
#' @details
#' - Uses `load_harplus()` internally for optimized SL4 file reading.
#' - Extracts variable names, dimension structures, and metadata.
#' - Converts variable names to lowercase if `lowercase = TRUE`.
#' - Allows the selection of specific headers using `select_header`.
#' - Returns structured data with explicit dimension names and sizes.
#'
#' @param file_path Character. The full path to the SL4 file to be read.
#' @param lowercase Logical. If `TRUE`, converts all variable names to lowercase. Default is `FALSE`.
#' @param select_header Character vector. Specific headers to extract; if `NULL`, all headers are read.
#'
#' @return A structured list containing:
#' - `data`: Extracted SL4 variable data, stored as arrays or matrices.
#' - `dimension_info`: A list with:
#'     - `dimension_string`: A textual representation of dimensions (e.g., "REG*COMM*YEAR").
#'     - `dimension_names`: The names of each dimension.
#'     - `dimension_sizes`: The size of each dimension.
#'
#' @author Pattawee Puangchit
#' 
#' @seealso \code{\link{load_harx}}, \code{\link{get_data_by_var}}, \code{\link{get_data_by_dims}}
#'
#' @export
#'
#' @examples
#' # Path to example files
#' sl4_path <- system.file("extdata", "TAR10.sl4", package = "HARplus")
#' 
#' # Basic loading
#' sl4_data <- load_sl4x(sl4_path)
#' 
#' # Load with lowercase names
#' sl4_data_lower <- load_sl4x(sl4_path, lowercase = TRUE)
#' 
#' # Load specific headers
#' sl4_selected <- load_sl4x(sl4_path, select_header = c("qo", "qgdp"))
#' 
load_sl4x <- function(file_path, lowercase = FALSE, select_header = NULL) {
  if (!file.exists(file_path)) {
    stop(sprintf("File '%s' does not exist", file_path))
  }
  
  solution <- load_harplus(file_path, lowercase = lowercase)
  subtotals <- c('TOTAL', solution$STDS)
  dimension_info <- list()
  
  var_indices <- seq_along(solution$VCNM)
  
  for (v in var_indices) {
    var_name <- solution$VCNM[v]
    if (solution$VCNI[v] > 0) {
      dim_indices <- solution$VCSP[v]:(solution$VCSP[v] + solution$VCNI[v] - 1)
      dimensions <- solution$VCSN[dim_indices]
      dim_names <- solution$STNM[dimensions]
      dim_sizes <- solution$SSZ[dimensions]
      dim_string <- paste(dim_names, collapse = "*")
      
      dimension_info[[var_name]] <- list(
        dimension_string = dim_string,
        dimension_names = dim_names,
        dimension_sizes = dim_sizes
      )
    }
  }
  
  results <- lapply(var_indices, function(f) {
    if (solution$VCNI[f] > 0) {
      dimensions <- solution$VCSN[solution$VCSP[f]:(solution$VCSP[f] + solution$VCNI[f] - 1)]
      sizes <- c(solution$SSZ[dimensions], length(subtotals))
      labels <- c(
        lapply(dimensions, function(g) {
          if (solution$SSZ[g] == 0) {
            character(0)
          } else {
            solution$STEL[solution$ELAD[g]:(solution$ELAD[g] + solution$SSZ[g] - 1)]
          }
        }),
        list(subtotals)
      )
      names(labels) <- c(solution$STNM[dimensions], 'subtotal')
    } else {
      sizes <- c(length(subtotals))
      labels <- list(subtotals)
      names(labels) <- c('subtotal')
    }
    array(NA, dim = sizes, dimnames = labels)
  })
  names(results) <- solution$VCNM
  
  partials <- solution$OREX > 0 & solution$OREX != solution$VNCP
  stHeaders <- c('CUMS', sprintf('%03dS', seq_along(solution$STDS)))
  
  complete_indices <- which(partials == FALSE & solution$PCUM > 0)
  
  for (v in complete_indices) {
    range <- solution$PCUM[v]:(solution$PCUM[v] + solution$ORND[v] - 1)
    results[[solution$VARS[v]]][] <- unlist(lapply(stHeaders, function(f) solution[[f]][range]))
  }
  
  zero_indices <- which(partials == FALSE & solution$PCUM == 0)
  
  if (length(zero_indices) > 0) {
    has_valid_shoc <- !is.null(solution$PSHK) && !is.null(solution$SHOC) && 
      is.matrix(solution$SHOC) && nrow(solution$SHOC) > 0
    
    shoc_row <- 1
    for (v in zero_indices) {
      # Guard: ensure PSHK index is valid before accessing
      if (has_valid_shoc && 
          v <= length(solution$PSHK) && 
          !is.na(solution$PSHK[v]) &&
          solution$PSHK[v] == 1) {
        
        # Guard: ensure OREX index is valid and value is positive
        if (v > length(solution$OREX) || is.na(solution$OREX[v]) || solution$OREX[v] <= 0) {
          results[[solution$VARS[v]]][] <- 0
          next
        }
        
        end_row <- shoc_row + solution$OREX[v] - 1
        
        if (shoc_row >= 1 && end_row <= nrow(solution$SHOC)) {
          shock_values <- solution$SHOC[shoc_row:end_row, 1]
          
          var_name <- solution$VARS[v]
          if (!is.null(results[[var_name]])) {
            if (length(dim(results[[var_name]])) == 2) {
              # Multi-dimensional: fill shock values into each subtotal column
              n_elements <- solution$OREX[v]
              for (st_idx in seq_along(stHeaders)) {
                if (n_elements == nrow(results[[var_name]])) {
                  results[[var_name]][, st_idx] <- shock_values
                } else {
                  # Size mismatch - fill what we can
                  fill_len <- min(n_elements, nrow(results[[var_name]]))
                  results[[var_name]][1:fill_len, st_idx] <- shock_values[1:fill_len]
                }
              }
            } else if (length(dim(results[[var_name]])) > 2) {
              # Higher-dimensional arrays: broadcast shock across subtotal dimension
              n_elements <- solution$OREX[v]
              arr_dims <- dim(results[[var_name]])
              subtotal_dim <- length(arr_dims)  # last dimension is subtotal
              
              for (st_idx in seq_along(stHeaders)) {
                idx_list <- vector("list", length(arr_dims))
                for (d in seq_along(arr_dims)) {
                  if (d == subtotal_dim) {
                    idx_list[[d]] <- st_idx
                  } else {
                    idx_list[[d]] <- seq_len(arr_dims[d])
                  }
                }
                sub_array <- do.call(`[`, c(list(results[[var_name]]), idx_list, list(drop = FALSE)))
                if (length(sub_array) == n_elements) {
                  do.call(`[<-`, c(list(results[[var_name]]), idx_list, list(value = shock_values)))
                  results[[var_name]] <- do.call(`[<-`, c(list(results[[var_name]]), idx_list, list(value = shock_values)))
                } else {
                  fill_len <- min(n_elements, length(sub_array))
                  fill_vals <- rep(0, length(sub_array))
                  fill_vals[1:fill_len] <- shock_values[1:fill_len]
                  results[[var_name]] <- do.call(`[<-`, c(list(results[[var_name]]), idx_list, list(value = fill_vals)))
                }
              }
            } else {
              # Scalar or 1D
              results[[var_name]][seq_along(stHeaders)] <- shock_values[1]
            }
          }
          
          shoc_row <- shoc_row + solution$OREX[v]
        } else {
          # SHOC matrix exhausted or row out of bounds
          results[[solution$VARS[v]]][] <- 0
        }
      } else {
        results[[solution$VARS[v]]][] <- 0
      }
    }
  }
  
  partial_indices <- which(partials == TRUE & solution$PCUM > 0)
  
  start <- 1
  for (v in partial_indices) {
    range <- solution$PCUM[v]:(solution$PCUM[v] + solution$ORND[v] - 1)
    positions <- solution$ORNL[start - 1 + (1:solution$ORND[v])]
    start <- solution$ORND[v] + start
    toFill <- logical(solution$VNCP[v])
    toFill[positions] <- TRUE
    results[[solution$VARS[v]]][toFill] <- unlist(lapply(stHeaders, function(f) solution[[f]][range]))
  }
  
  if (!is.null(select_header)) {
    results <- results[select_header]
    dimension_info <- dimension_info[select_header]
  }
  
  enhanced_results <- list(
    data = results,
    dimension_info = dimension_info
  )
  
  class(enhanced_results) <- c("sl4")
  return(enhanced_results)
}

#' @title Load and Process GEMPACK HAR Files (Internal)
#'
#' @description Reads a GEMPACK HAR file and efficiently extracts structured data while maintaining 
#' compatibility with standard HAR formats. This implementation builds upon the foundational 
#' work of the **HARr** package, reorganizing the process for improved execution speed, 
#' memory management, and handling of sparse data structures.
#'
#' @details
#' - **Efficient File Reading:** Reads large HAR files in chunks for better performance.
#' - **Optimized Memory Usage:** Reduces unnecessary allocations and improves cleanup.
#' - **Streamlined Header Processing:** Ensures accurate extraction of dimension metadata.
#' - **Supports Sparse Data Structures:** Handles `RESPSE` and `REFULL` headers efficiently.
#'
#' **Supported HAR Header Types:**
#' - `1CFULL`: Character headers
#' - `2IFULL`: Integer headers
#' - `2RFULL`: Real headers
#' - `REFULL`: Real headers with extended metadata
#' - `RESPSE`: Sparse real headers
#'
#' @param con Character or connection. The file path to the HAR file or an open binary connection.
#' @param coefAsname Logical. If `TRUE`, replaces four-letter headers with coefficient names when available. Default is `FALSE`.
#' @param lowercase Logical. If `TRUE`, converts all string values to lowercase. Default is `TRUE`.
#' @param select_header Character vector. Specific headers to extract; if `NULL`, reads all headers.
#'
#' @return A structured list where:
#' - Each element corresponds to a header in the HAR file.
#' - Names are either header names or coefficient names (if `coefAsname = TRUE`).
#' - Data maintains its original dimensions and attributes.
#'
#' @author Pattawee Puangchit
#' 
#' @seealso \code{\link{load_sl4x}}, \code{\link{load_harx}}
#'
#' @keywords internal
#'
load_harplus <- function(con, coefAsname = FALSE, lowercase = TRUE, select_header = NULL) {
  if(is.character(con)) {
    con <- file(con, 'rb')
    on.exit(close(con))
  }
  
  cf <- raw()
  while (length(chunk <- readBin(con, raw(), n=1e6)) > 0) {
    cf <- c(cf, chunk)
  }
  
  headers <- list()
  
  if (cf[1] == 0xfd) {
    currentHeader <- ""
    i <- 2
    while (i < length(cf)) {
      fb <- cf[i]
      i <- i + 1
      
      bitsLength <- as.integer(rawToBits(fb))[3:8]
      toRead <- as.integer(rawToBits(fb))[1:2]
      toReadBytes <- sum(2^((1:2)-1) * toRead)
      
      if (toReadBytes > 0) {
        bitsLength <- c(bitsLength, rawToBits(cf[i:(i + toReadBytes - 1)]))
        i <- i + toReadBytes
        i <- i + 1
      }
      
      recordLength <- sum(2^((1:length(bitsLength))-1) * bitsLength)
      
      if (recordLength == 4) {
        currentHeader <- trimws(rawToChar(cf[i:(i + recordLength - 1)]))
      }
      if (is.null(headers[[currentHeader]])) {
        headers[[currentHeader]] <- list()
      }
      if (is.null(headers[[currentHeader]]$records)) {
        headers[[currentHeader]]$records <- list()
      }
      
      headers[[currentHeader]]$records[[length(headers[[currentHeader]]$records) + 1]] <- 
        cf[i:(i + recordLength - 1)]
      
      i <- i + recordLength
      
      totalLength <- recordLength + 1 + toReadBytes
      endingBits <- intToBits(totalLength)
      maxPosition <- max(which(endingBits == 1))
      needEnd <- if(maxPosition <= 6) 0 else ceiling((maxPosition - 6) / 8)
      
      expectedEnd <- packBits(c(intToBits(needEnd)[1:2], 
                                intToBits(totalLength))[1:(8 * (needEnd + 1))], 'raw')
      expectedEnd <- rev(expectedEnd)
      
      if (!identical(cf[i:(i + length(expectedEnd) - 1)], expectedEnd)) {
        stop("Unexpected end of record")
      }
      i <- i + length(expectedEnd)
    }
  } else {
    i <- 1
    while (i < length(cf)) {
      toRead <- readBin(cf[i:(i + 3)], 'integer', size = 4)
      if (toRead == 4) {
        if (!all(cf[(i + 4):(i + 3 + toRead)] == 0x20)) {
          headerName <- trimws(rawToChar(cf[(i + 4):(i + 3 + toRead)]))
          headers[[headerName]] <- list(start = i)
        }
      }
      i <- i + 3 + toRead + 1
      hasRead <- readBin(cf[i:(i + 3)], 'integer', size = 4)
      i <- i + 4
    }
    
    for (h in names(headers)) {
      start <- headers[[h]]$start
      end <- if(h == tail(names(headers), 1)) length(cf) else headers[[names(headers)[which(names(headers) == h) + 1]]]$start - 1
      headers[[h]]$binary <- cf[start:end]
      
      headers[[h]]$records <- list()
      i <- 1
      while (i < length(headers[[h]]$binary)) {
        toRead <- readBin(headers[[h]]$binary[i:(i + 3)], 'integer', size = 4)
        i <- i + 4
        headers[[h]]$records[[length(headers[[h]]$records) + 1]] <- headers[[h]]$binary[i:(i + toRead - 1)]
        i <- i + toRead
        i <- i + 4
      }
    }
  }
  
  if(length(select_header) > 0) {
    headers <- headers[select_header]
  }
  
  for (h in names(headers)) {
    if (!is.null(headers[[h]])) {
      headers[[h]]$type <- rawToChar(headers[[h]]$records[[2]][5:10])
      headers[[h]]$description <- rawToChar(headers[[h]]$records[[2]][11:80])
      headers[[h]]$numberOfDimensions <- readBin(headers[[h]]$records[[2]][81:84], 'integer', size = 4)
      headers[[h]]$dimensions <- readBin(headers[[h]]$records[[2]][85:(85 + headers[[h]]$numberOfDimensions * 4 - 1)], 
                                         'integer', size = 4, n = headers[[h]]$numberOfDimensions)
    }
  }
  
  for (h in names(headers)) {
    if (headers[[h]]$type == '1CFULL') {
      contents <- Reduce(function(a, f) c(a, f[17:length(f)]), 
                         headers[[h]]$records[3:length(headers[[h]]$records)], c())
      contents[contents == 0x00] <- as.raw(0x20)
      
      expected_length <- headers[[h]]$dimensions[1] * headers[[h]]$dimensions[2]
      actual_length <- length(contents)
      
      if (actual_length < expected_length) {
        contents <- c(contents, rep(as.raw(0x20), expected_length - actual_length))
      } else if (actual_length > expected_length) {
        contents <- contents[1:expected_length]
      }
      
      char_vector <- strsplit(rawToChar(contents), '')[[1]]
      char_length <- length(char_vector)
      
      if (char_length < expected_length) {
        char_vector <- c(char_vector, rep(' ', expected_length - char_length))
      } else if (char_length > expected_length) {
        char_vector <- char_vector[1:expected_length]
      }
      
      m <- matrix(char_vector, 
                  nrow = headers[[h]]$dimensions[2], 
                  ncol = headers[[h]]$dimensions[1])
      
      headers[[h]]$data <- if(tolower(h) == 'xxhs') {
        apply(m, 2, paste, collapse = '')
      } else {
        trimws(apply(m, 2, paste, collapse = ''))
      }
      
      if(lowercase) {
        headers[[h]]$data <- tolower(headers[[h]]$data)
      }
    }
    
    if (headers[[h]]$type == '2IFULL') {
      contents <- Reduce(function(a, f) c(a, f[33:length(f)]), 
                         headers[[h]]$records[3:length(headers[[h]]$records)], c())
      headers[[h]]$data <- matrix(readBin(contents, 'integer', size = 4, 
                                          n = prod(headers[[h]]$dimensions)),
                                  nrow = headers[[h]]$dimensions[1],
                                  ncol = headers[[h]]$dimensions[2])
    }
    
    if (headers[[h]]$type == '2RFULL') {
      contents <- Reduce(function(a, f) c(a, f[33:length(f)]), 
                         headers[[h]]$records[3:length(headers[[h]]$records)], c())
      headers[[h]]$data <- array(readBin(contents, 'double', size = 4,
                                         n = prod(headers[[h]]$dimensions)),
                                 dim = headers[[h]]$dimensions)
    }
    
    if (headers[[h]]$type %in% c('REFULL', 'RESPSE')) {
      headers[[h]]$definedDimensions <- readBin(headers[[h]]$records[[3]][5:8], 'integer', size = 4)
      headers[[h]]$usedDimensions <- readBin(headers[[h]]$records[[3]][13:16], 'integer', size = 4)
      headers[[h]]$coefficient <- rawToChar(headers[[h]]$records[[3]][17:28])
      
      if (headers[[h]]$usedDimensions > 0) {
        raw_bytes <- headers[[h]]$records[[3]][33:(33 + headers[[h]]$usedDimensions * 12 - 1)]
        char_vector <- strsplit(rawToChar(raw_bytes), '')[[1]]
        expected_len <- headers[[h]]$usedDimensions * 12
        
        if (length(char_vector) < expected_len) {
          char_vector <- c(char_vector, rep(' ', expected_len - length(char_vector)))
        } else if (length(char_vector) > expected_len) {
          char_vector <- char_vector[1:expected_len]
        }
        
        dnames <- matrix(char_vector, nrow = 12)
        dnames <- apply(dnames, 2, paste, collapse = '')
        
        actualDimsNames <- headers[[h]]$records[[3]][(33 + headers[[h]]$usedDimensions * 12) + 0:6] == 0x6b
        uniqueDimNames <- unique(dnames[actualDimsNames])
        dimNames <- vector("list", headers[[h]]$usedDimensions)
        
        if (length(uniqueDimNames) > 0) {
          for (d in seq_along(uniqueDimNames)) {
            nele <- readBin(headers[[h]]$records[[3 + d]][13:16], 'integer', size = 4)
            raw_bytes <- headers[[h]]$records[[3 + d]][17:(17 + nele * 12 - 1)]
            char_vector <- strsplit(rawToChar(raw_bytes), '')[[1]]
            expected_len <- nele * 12
            
            if (length(char_vector) < expected_len) {
              char_vector <- c(char_vector, rep(' ', expected_len - length(char_vector)))
            } else if (length(char_vector) > expected_len) {
              char_vector <- char_vector[1:expected_len]
            }
            
            m <- matrix(char_vector, nrow = 12)
            
            for (dd in which(dnames == uniqueDimNames[d])) {
              dimNames[[dd]] <- trimws(apply(m, 2, paste, collapse = ''))
              names(dimNames)[dd] <- trimws(uniqueDimNames[d])
            }
          }
        }
        
        dataStart <- 3 + length(uniqueDimNames) + 1
        
        if (headers[[h]]$type == 'REFULL') {
          numberOfFrames <- readBin(headers[[h]]$records[[dataStart]][5:8], 'integer')
          numberOfDataFrames <- (numberOfFrames - 1) / 2
          dataFrames <- (dataStart) + 1:numberOfDataFrames * 2
          
          dataBytes <- Reduce(function(a, f) c(a, headers[[h]]$records[[f]][9:length(headers[[h]]$records[[f]])]), 
                              dataFrames, c())
          
          headers[[h]]$data <- array(readBin(dataBytes, 'double', size = 4,
                                             n = prod(headers[[h]]$dimensions)),
                                     dim = headers[[h]]$dimensions[1:headers[[h]]$usedDimensions],
                                     dimnames = dimNames)
        } else {
          elements <- readBin(headers[[h]]$records[[dataStart]][5:8], 'integer', size = 4)
          dataVector <- numeric(prod(headers[[h]]$dimensions))
          
          for (rr in (dataStart + 1):length(headers[[h]]$records)) {
            dataBytes <- headers[[h]]$records[[rr]][17:length(headers[[h]]$records[[rr]])]
            currentPoints <- length(dataBytes) / 8
            
            locations <- readBin(dataBytes[1:(4 * currentPoints)], 'integer', size = 4,
                                 n = currentPoints)
            values <- readBin(dataBytes[(4 * currentPoints + 1):(8 * currentPoints)],
                              'double', size = 4, n = currentPoints)
            
            dataVector[locations] <- values
          }
          
          headers[[h]]$data <- array(dataVector,
                                     dim = headers[[h]]$dimensions[1:headers[[h]]$usedDimensions],
                                     dimnames = dimNames)
        }
      } else {
        lastRecord <- headers[[h]]$records[[length(headers[[h]]$records)]]
        headers[[h]]$data <- array(readBin(lastRecord[9:length(headers[[h]]$records[[3]])],
                                           'double', size = 4,
                                           n = prod(headers[[h]]$dimensions)),
                                   dim = headers[[h]]$dimensions[1:headers[[h]]$usedDimensions])
      }
    }
  }
  
  result <- lapply(headers, function(f) {
    data <- f$data
    if (lowercase && !is.null(dimnames(data))) {
      names(dimnames(data)) <- tolower(names(dimnames(data)))
      dimnames(data) <- lapply(dimnames(data), tolower)
    }
    data
  })
  
  if (coefAsname) {
    for (h in names(headers)) {
      if (!is.null(headers[[h]]$coefficient)) {
        names(result)[which(names(result) == h)] <- trimws(headers[[h]]$coefficient)
      }
    }
  }
  
  if (lowercase) {
    names(result) <- tolower(names(result))
  }
  
  result
}

#' @title Rename Columns in a Data Frame (Internal)
#'
#' @description A helper function that renames columns in a data frame based on a specified mapping.
#' Used internally in `get_var_structure()`, `get_data_by_dims()`, and `rename_dims()`.
#'
#' @details
#' - Replaces column names according to the provided `rename_cols` mapping.
#' - Ensures no duplicate column names by appending numerical suffixes when necessary.
#' - Helps standardize column names across SL4 and HAR datasets.
#'
#' @param df A data frame containing columns to be renamed.
#' @param rename_cols A named vector where names are existing column names, 
#' and values are the corresponding new names.
#'
#' @return A modified data frame with renamed columns.
#'
#' @author Pattawee Puangchit
#' 
#' @seealso \code{\link{get_var_structure}}, \code{\link{get_data_by_dims}}, \code{\link{rename_dims}}
#'
#' @keywords internal
#' 
rename_col <- function(df, rename_cols) {
  if (!is.null(rename_cols)) {
    for (old_name in names(rename_cols)) {
      matching_cols <- which(names(df) == old_name)
      if (length(matching_cols) > 0) {
        for (i in seq_along(matching_cols)) {
          new_name <- if (i == 1) rename_cols[old_name] else paste0(rename_cols[old_name], i - 1)
          names(df)[matching_cols[i]] <- new_name
        }
      }
    }
  }
  return(df)
}


#' @title Extract and Organize Dimension Metadata (Internal)
#'
#' @description A helper function that extracts and structures dimension-related metadata 
#' from a given dimension structure. Used internally in `get_var_structure()` 
#' and `compare_var_structure()`.
#'
#' @details
#' - Retrieves structured metadata for variables in SL4 and HAR datasets.
#' - Computes data shape and ensures consistency in dimension structures.
#' - Helps determine observation counts and column sizes for variable summaries.
#'
#' @param dim_info A list containing dimension metadata, including:
#'   - `dimension_string`: A textual representation of dimensions (e.g., `"REG*COMM*YEAR"`).
#'   - `dimension_names`: A character vector of dimension names.
#'   - `dimension_sizes`: A numeric vector indicating the size of each dimension.
#'
#' @return A structured list containing:
#' - `dimension_string`: The original dimension string.
#' - `dim_size`: The number of dimensions.
#' - `data_shape`: A formatted string representing the data shape (e.g., `"10x20x30"`).
#' - `col_size`: The product of all dimension sizes except the first, representing column count.
#' - `n_obs`: The first dimension size, typically representing the number of observations.
#'
#' @author Pattawee Puangchit
#' 
#' @seealso \code{\link{get_var_structure}}, \code{\link{compare_var_structure}}
#'
#' @keywords internal
#' 
get_dim_info <- function(dim_info) {
  list(
    dimension_string = dim_info$dimension_string,
    dim_size = length(dim_info$dimension_names),
    data_shape = paste(dim_info$dimension_sizes, collapse = "x"),
    col_size = prod(dim_info$dimension_sizes[-1], 1),
    n_obs = dim_info$dimension_sizes[1]
  )
}


#' @title Match Patterns with Optional Mixing (Internal)
#'
#' @description Compares two patterns to determine if they match, with an option to allow flexible dimension order.
#'
#' @details
#' - Performs case-insensitive pattern matching.
#' - If `mix_patterns = TRUE`, allows patterns to match even if dimensions are in different order.
#'
#' @param pattern1 A character string representing the first pattern.
#' @param pattern2 A character string representing the second pattern.
#' @param mix_patterns Logical; if `TRUE`, allows dimension order to be ignored during comparison.
#'
#' @return Logical; `TRUE` if the patterns match, `FALSE` otherwise.
#'
#' @author Pattawee Puangchit
#' 
#' @seealso \code{\link{get_original_pattern}}, \code{\link{process_pattern}}, \code{\link{get_data_by_dims}}
#'
#' @keywords internal
#' 
pattern_match <- function(pattern1, pattern2, mix_patterns = FALSE) {
  if (mix_patterns) {
    split1 <- unlist(strsplit(tolower(pattern1), "\\*"))
    split2 <- unlist(strsplit(tolower(pattern2), "\\*"))
    return(length(split1) == length(split2) && 
             all(sort(split1) == sort(split2)))
  } else {
    return(tolower(pattern1) == tolower(pattern2))
  }
}


#' @title Retrieve the Original Dimension Pattern (Internal)
#'
#' @description A helper function that finds the original dimension pattern name in an SL4 or HAR dataset 
#' that matches a given pattern. Used internally in `get_data_by_dims()`.
#'
#' @details
#' - Performs a case-insensitive comparison to identify matching dimension patterns.
#' - Supports flexible pattern matching when `mix_patterns = TRUE`, allowing dimension order to vary.
#' - Returns the standardized dimension pattern name as stored in the dataset.
#'
#' @param pattern Character. The pattern to search for in dimension structures.
#' @param data_obj An SL4 or HAR object containing dimension information.
#' @param mix_patterns Logical. If `TRUE`, allows dimension order to vary when matching patterns.
#'
#' @return The original dimension pattern name as a character string, or `NULL` if no match is found.
#'
#' @author Pattawee Puangchit
#' 
#' @seealso \code{\link{pattern_match}}, \code{\link{process_pattern}}, \code{\link{get_data_by_dims}}
#'
#' @keywords internal
#' 
get_original_pattern <- function(pattern, data_obj, mix_patterns = FALSE) {
  all_vars <- names(data_obj$dimension_info)
  
  matching_vars <- character(0)
  for (var_name in all_vars) {
    dim_info <- data_obj$dimension_info[[var_name]]
    if (!is.null(dim_info$dimension_string) && 
        pattern_match(dim_info$dimension_string, pattern, mix_patterns)) {
      matching_vars <- c(matching_vars, var_name)
      break  
    }
  }
  
  if (length(matching_vars) > 0) {
    dim_info <- data_obj$dimension_info[[matching_vars[1]]]
    return(paste(dim_info$dimension_names, collapse="*"))
  }
  
  return(NULL)
}


#' @title Extract and Process Pattern-Matched Variables (Internal)
#'
#' @description A helper function that extracts and processes variables matching a specified pattern 
#' within an SL4 or HAR data object. Used internally in `get_data_by_dims()`.
#'
#' @details
#' - Searches for variables whose dimension structures match the given pattern.
#' - Supports flexible pattern matching when `pattern_mix = TRUE`, allowing dimension order to vary.
#' - Extracted data is converted into a tidy format, preserving dimension structures.
#' - Standardizes `"Subtotal"` column naming for consistency across datasets.
#' - Ensures only non-empty extracted variables are retained.
#'
#' @param pattern Character. The pattern to match against dimension structures.
#' @param data_obj An SL4 or HAR object containing dimension information and data.
#' @param exp_name Character. The experiment name assigned to the extracted data.
#' @param pattern_mix Logical. If `TRUE`, allows pattern matching to ignore dimension order.
#'
#' @return A data frame containing processed data for the matching pattern, or `NULL` if no matches are found.
#'
#' @author Pattawee Puangchit
#' 
#' @seealso \code{\link{get_original_pattern}}, \code{\link{pattern_match}}, \code{\link{get_data_by_dims}}
#'
#' @keywords internal
#' 
process_pattern <- function(pattern, data_obj, exp_name, pattern_mix = FALSE) {
  matching_vars <- names(data_obj$dimension_info)[
    sapply(data_obj$dimension_info, function(x) 
      pattern_match(x$dimension_string, pattern, pattern_mix)
    )
  ]
  
  if (length(matching_vars) == 0) {
    warning(sprintf("No variables found with pattern '%s' in experiment '%s'. Please check the dimension name or try pattern_mix = TRUE", 
                    pattern, exp_name))
    return(NULL)
  }
  
  var_data_list <- list()
  for (var_name in matching_vars) {
    var_data <- data_obj$data[[var_name]]
    dim_info <- data_obj$dimension_info[[var_name]]
    
    if (length(dim(var_data)) == 0) {
      next
    }
    
    df <- as.data.frame.table(var_data, stringsAsFactors = FALSE, responseName = "Value")
    
    setNames(df, c(dim_info$dimension_names))
    
    if ("type" %in% tolower(names(df))) {
      names(df)[tolower(names(df)) == "type"] <- "Subtotal"
    } else if ("subtotal" %in% tolower(names(df))) {
      names(df)[tolower(names(df)) == "subtotal"] <- "Subtotal"
    }
    
    df$Variable <- var_name
    df$Dimension <- dim_info$dimension_string
    df$Experiment <- exp_name
    
    df <- df[!is.na(df$Value), ]
    
    if (nrow(df) > 0) {
      var_data_list[[var_name]] <- df
    }
  }
  
  if (length(var_data_list) > 0) {
    result <- do.call(rbind, var_data_list)
    rownames(result) <- NULL 
    return(result)
  }
  return(NULL)
}


#' @title Process Decomposition Levels in Data Frames (Internal)
#' 
#' @description A helper function that filters data based on decomposition levels in the `"Subtotal"` column.
#' Used internally in `get_data_by_var()`, `get_data_by_dims()`, and `group_data_by_dims()`.
#'
#' @details
#' - If `subtotal_level = "total"`, **keeps only `"TOTAL"` values**, removing all decomposed components.
#' - If `subtotal_level = "decomposed"`, **keeps only decomposed components**, removing `"TOTAL"`.
#' - If `subtotal_level = "all"`, **keeps both `"TOTAL"` and decomposed values** (no filtering).
#' - If `subtotal_level = TRUE` (logical), it is **equivalent to `"all"`**, meaning all values are kept.
#' - If `subtotal_level = FALSE` (logical), it is **equivalent to `"total"`**, meaning only `"TOTAL"` values are kept, and decomposed components are removed.
#' - Filtering is applied **only when both `"TOTAL"` and decomposed values exist**.
#'
#' @param df A data frame containing a `"Subtotal"` column.
#' @param subtotal_level  Character or logical. Determines which values to retain:
#'   - `"total"`: Keeps only `"TOTAL"` values.
#'   - `"decomposed"`: Keeps only decomposed values (excludes `"TOTAL"`).
#'   - `"all"`: Keeps all rows.
#'   - `TRUE`: Equivalent to `"all"` (keeps both `"TOTAL"` and decomposed values).
#'   - `FALSE`: Equivalent to `"total"` (keeps only `"TOTAL"` values, removing decomposed components).
#'
#' @return A filtered data frame based on the specified decomposition level.
#'
#' @author Pattawee Puangchit
#' 
#' @seealso \code{\link{get_data_by_var}}, \code{\link{get_data_by_dims}}, \code{\link{group_data_by_dims}}
#'
#' @keywords internal
#' 
process_decomp_level <- function(df, subtotal_level) {
  if (!("Subtotal" %in% names(df))) return(df)
  
  level <- if (is.logical(subtotal_level)) {
    if (subtotal_level) "all" else "total"
  } else {
    subtotal_level
  }
  
  if ("Subtotal" %in% names(df)) {
    df <- switch(level,
                 "all" = df,
                 "total" = df[df$Subtotal == "TOTAL", ],
                 "decomposed" = df[df$Subtotal != "TOTAL", ],
                 df)
  }
  return(df)
}



#' @title Process and Export Report (Internal)
#'
#' @description Generates a summary report of variables and their corresponding output files
#' from structured SL4 or HAR data. The report is saved as an Excel file.
#'
#' @details
#' - Extracts unique variable names and their associated export filenames.
#' - Handles nested list structures by recursively traversing them.
#' - The output report is saved as an Excel file named `"Report_<data_name>.xlsx"`.
#'
#' @param data A structured SL4 or HAR object or a nested list of data frames.
#' @param output_path A character string specifying the output directory or file path.
#' @param prefix A character string to prepend to the exported filenames.
#'
#' @importFrom openxlsx createWorkbook addWorksheet writeData createStyle addStyle saveWorkbook
#' @importFrom tools file_ext file_path_sans_ext
#'
#' @author Pattawee Puangchit
#' 
#' @seealso \code{\link{export_data}}
#' 
#' @keywords internal
#'
process_export_report <- function(data, output_path, prefix = "", data_name = "data") {
  report_data <- data.frame(
    Variable = character(),
    OutputFile = character(),
    stringsAsFactors = FALSE
  )
  
  stack <- list(list(data = data, name = ""))
  
  while (length(stack) > 0) {
    current <- stack[[length(stack)]]
    stack <- stack[-length(stack)]
    
    if (is.data.frame(current$data) && "Variable" %in% names(current$data)) {
      output_file <- if (current$name == "") "data" else gsub("\\*", "_", current$name)
      output_file <- paste0(prefix, output_file)
      
      new_rows <- data.frame(
        Variable = unique(current$data$Variable),
        OutputFile = output_file,
        stringsAsFactors = FALSE
      )
      report_data <- rbind(report_data, new_rows)
      
    } else if (is.list(current$data)) {
      for (name in rev(names(current$data))) {
        if (name != "report") {
          new_name <- if (current$name == "") name else paste(current$name, name, sep = "_")
          stack[[length(stack) + 1]] <- list(data = current$data[[name]], name = new_name)
        }
      }
    }
  }
  
  if (nrow(report_data) > 0) {
    report_data <- unique(report_data[order(report_data$Variable), ])
    
    wb <- openxlsx::createWorkbook()
    openxlsx::addWorksheet(wb, "Variables")
    openxlsx::writeData(wb, "Variables", report_data, startRow = 1, colNames = TRUE)
    
    header_style <- openxlsx::createStyle(
      textDecoration = "bold",
      border = "Bottom",
      borderStyle = "medium"
    )
    openxlsx::addStyle(wb, "Variables", style = header_style, 
                       rows = 1, cols = 1:ncol(report_data))
    
    output_dir <- if (tools::file_ext(output_path) == "") output_path else dirname(output_path)
    report_filename <- paste0("Report_", data_name, ".xlsx")
    report_path <- file.path(output_dir, report_filename)
    openxlsx::saveWorkbook(wb, report_path, overwrite = TRUE)
    
    message(sprintf("Created %s in %s", report_filename, normalizePath(output_dir)))
  }
}

#' @title Extract Variable Data from SL4 or HAR Objects
#'
#' @description Extracts structured data for one or more variables from SL4 or HAR objects,
#' transforming array-like data into a tidy format.
#'
#' @details
#' - Retrieves specific variables, multiple variables, or all available variables from SL4 or HAR datasets.
#' - Supports merging data from multiple experiments (`merge_data = TRUE`).
#' - Allows renaming of column names (`rename_cols`).
#' - Handles subtotal filtering (`subtotal_level`), controlling whether `"TOTAL"` or decomposed values are retained.
#'
#' @param var_names Character vector. Variable names to extract. Use `"ALL"` or `NULL` to extract all available variables.
#' @param ... One or more SL4 or HAR data objects loaded using `load_sl4x()` or `load_harx()`.
#' @param experiment_names Character vector. Names assigned to each dataset. If `NULL`, names are inferred.
#' @param subtotal_level Character or logical. Determines which decomposition levels to retain:
#'   - `"total"`: Keeps only `"TOTAL"` values.
#'   - `"decomposed"`: Keeps only decomposed values (excludes `"TOTAL"`).
#'   - `"all"`: Keeps all rows.
#'   - `TRUE`: Equivalent to `"all"`, retaining both `"TOTAL"` and decomposed values.
#'   - `FALSE`: Equivalent to `"total"`, keeping only `"TOTAL"` values.
#' @param rename_cols Named vector. Column name replacements (`c("old_name" = "new_name")`).
#' @param merge_data Logical. If `TRUE`, attempts to merge data across multiple experiments. Default is `FALSE`.
#'
#' @return A list of structured data:
#' - If `merge_data = FALSE`, returns a named list where each element corresponds to an experiment.
#' - If `merge_data = TRUE`, returns a named list of all merged data
#'
#' @importFrom stats setNames
#' @importFrom utils tail
#'
#' @author Pattawee Puangchit
#'
#' @seealso \code{\link{get_data_by_dims}}, , \code{\link{group_data_by_dims}}, \code{\link{load_sl4x}}, \code{\link{load_harx}}
#'
#' @export
#'
#' @examples
#' # Import sample data:
#' sl4_data <- load_sl4x(system.file("extdata", "TAR10.sl4", package = "HARplus"))
#' sl4_data1 <- load_sl4x(system.file("extdata", "SUBT10.sl4", package = "HARplus"))
#'
#' # Extract a single variable
#' data_qo <- get_data_by_var("qo", sl4_data)
#'
#' # Extract multiple variables
#' data_multiple <- get_data_by_var(c("qo", "qgdp"), sl4_data)
#'
#' # Extract all variables separately from multiple datasets
#' data_all <- get_data_by_var(NULL, sl4_data, sl4_data1, merge_data = FALSE)
#'
#' # Merge variable data across multiple datasets
#' data_merged <- get_data_by_var(NULL, sl4_data, sl4_data1, merge_data = TRUE)
#'
#' # Retain only "TOTAL" values, removing decomposed components (subtotal_level = "total" or FALSE)
#' data_total_only <- get_data_by_var("qo", sl4_data, subtotal_level = "total")
#' data_total_only_alt <- get_data_by_var("qo", sl4_data, subtotal_level = FALSE)
#'
#' # Retain only decomposed components, removing "TOTAL" (subtotal_level = "decomposed")
#' data_decomposed_only <- get_data_by_var("qo", sl4_data, subtotal_level = "decomposed")
#'
#' # Retain all value levels (subtotal_level = "all" or TRUE)
#' data_all_decomp <- get_data_by_var("qo", sl4_data, subtotal_level = "all")
#' data_all_decomp_alt <- get_data_by_var("qo", sl4_data, subtotal_level = TRUE)
#'
#' # Rename specific columns
#' data_renamed <- get_data_by_var("qo", sl4_data, rename_cols = c(REG = "Region", COMM = "Commodity"))
#'
#' # Merge data across multiple datasets with custom experiment names
#' data_merged_experiments <- get_data_by_var("qo", sl4_data, sl4_data1,
#' experiment_names = c("EXP1", "EXP2"),
#' merge_data = TRUE)
#'
get_data_by_var <- function(var_names = NULL, ..., experiment_names = NULL,
                            subtotal_level = FALSE, rename_cols = NULL, merge_data = FALSE) {
  if (!is.logical(subtotal_level) && !subtotal_level %in% c("all", "total", "decomposed")) {
    stop("subtotal_level must be either logical (TRUE/FALSE) or one of: 'all', 'total', 'decomposed'")
  }

  data_list <- list(...)
  if (length(data_list) == 0) {
    stop("At least one data object is required.")
  }

  if (is.null(experiment_names)) {
    dots <- match.call(expand.dots = FALSE)$...
    experiment_names <- if (length(dots) == 1) {
      deparse(dots[[1]])
    } else {
      vapply(dots, deparse, character(1))
    }
  }

  if (length(experiment_names) != length(data_list)) {
    stop("The number of experiment names must match the number of data objects.")
  }

  if (length(data_list) == 1) {
    merge_data <- FALSE
  }

  if (is.null(var_names) || (length(var_names) == 1 && var_names == "ALL")) {
    var_names <- unique(unlist(lapply(data_list, function(x) names(x$data))))
  }

  process_variable <- function(var_name, data_obj, experiment_name) {
    if (!var_name %in% names(data_obj$data)) return(NULL)

    var_data <- data_obj$data[[var_name]]
    if (length(dim(var_data)) == 0) return(NULL)

    dim_info <- data_obj$dimension_info[[var_name]]
    df <- as.data.frame.table(var_data, stringsAsFactors = FALSE, responseName = "Value")
    setNames(df, c(dim_info$dimension_names))

    if ("subtotal" %in% tolower(names(df))) {
      names(df)[tolower(names(df)) == "subtotal"] <- "Subtotal"
    }

    df$Variable <- var_name
    df$Dimension <- dim_info$dimension_string
    df$Experiment <- experiment_name

    df <- process_decomp_level(df, subtotal_level)
    df <- rename_col(df, rename_cols)
    return(df)
  }

  if (merge_data) {
    result <- lapply(var_names, function(var_name) {
      df_list <- lapply(seq_along(data_list), function(i) {
        process_variable(var_name, data_list[[i]], experiment_names[i])
      })
      df_list <- Filter(Negate(is.null), df_list)
      if (length(df_list) > 0) {
        result <- do.call(rbind, df_list)
        rownames(result) <- NULL
        return(result)
      }
      return(NULL)
    })
    names(result) <- var_names
    result <- Filter(Negate(is.null), result)

    return(result)

  } else {
    result_by_experiment <- lapply(seq_along(data_list), function(i) {
      obj <- data_list[[i]]
      experiment_name <- experiment_names[i]

      var_results <- lapply(var_names, function(var_name) {
        process_variable(var_name, obj, experiment_name)
      })

      names(var_results) <- var_names
      var_results <- Filter(Negate(is.null), var_results)

      if (length(var_results) == 0) return(NULL)
      return(var_results)
    })

    names(result_by_experiment) <- experiment_names
    return(Filter(Negate(is.null), result_by_experiment))
  }
}



#' @title Extract Data by Dimension Patterns from SL4 or HAR Objects
#'
#' @description Retrieves structured data from SL4 or HAR objects based on specified dimension patterns.
#' Supports multiple experiments and merging datasets while maintaining structured dimension metadata.
#'
#' @details
#' - Extracts variables matching specified dimension patterns.
#' - Allows for flexible pattern matching (`pattern_mix = TRUE`).
#' - Supports merging data across multiple experiments (`merge_data = TRUE`).
#' - Provides column renaming functionality (`rename_cols`).
#' - Handles subtotal filtering (`subtotal_level`), controlling whether `"TOTAL"` or decomposed values are retained.
#'
#' @param patterns Character vector. Dimension patterns to extract. Use `"ALL"` or `NULL` to extract all available patterns.
#' @param ... One or more SL4 or HAR data objects loaded using `load_sl4x()` or `load_harx()`.
#' @param experiment_names Character vector. Names assigned to each dataset. If `NULL`, names are inferred.
#' @param subtotal_level Character or logical. Determines which decomposition levels to retain:
#'   - `"total"`: Keeps only `"TOTAL"` values.
#'   - `"decomposed"`: Keeps only decomposed values (excludes `"TOTAL"`).
#'   - `"all"`: Keeps all rows.
#'   - `TRUE`: Equivalent to `"all"`, retaining both `"TOTAL"` and decomposed values.
#'   - `FALSE`: Equivalent to `"total"`, keeping only `"TOTAL"` values.
#' @param rename_cols Named vector. Column name replacements (`c("old_name" = "new_name")`).
#' @param merge_data Logical. If `TRUE`, attempts to merge data across multiple experiments. Default is `FALSE`.
#' @param pattern_mix Logical. If `TRUE`, allows flexible pattern matching, ignoring dimension order. Default is `FALSE`.
#'
#' @return A structured list of extracted data:
#' - If `merge_data = FALSE`, returns a named list where each element corresponds to an experiment.
#' - If `merge_data = TRUE`, returns a named list of all merged data
#'
#' @importFrom stats setNames
#' @importFrom utils tail
#'
#' @author Pattawee Puangchit
#'
#' @seealso \code{\link{get_data_by_var}}, \code{\link{group_data_by_dims}}
#'
#' @export
#'
#' @examples
#' # Import sample data:
#' sl4_data <- load_sl4x(
#'   system.file("extdata", "TAR10.sl4", package = "HARplus")
#' )
#' sl4_data1 <- load_sl4x(
#'   system.file("extdata", "SUBT10.sl4", package = "HARplus")
#' )
#'
#' # Extract data for a single dimension pattern
#' data_single_pattern <- get_data_by_dims(
#'   "comm*reg",
#'   sl4_data
#' )
#'
#' # Extract multiple dimension patterns
#' data_multiple_patterns <- get_data_by_dims(
#'   c("comm*reg", "REG*ACTS"),
#'   sl4_data
#' )
#'
#' # Extract all dimension patterns separately from multiple datasets
#' data_all_patterns <- get_data_by_dims(
#'   NULL,
#'   sl4_data, sl4_data1,
#'   merge_data = FALSE
#' )
#'
#' # Merge data for identical patterns across multiple datasets
#' data_merged_patterns <- get_data_by_dims(
#'   NULL,
#'   sl4_data, sl4_data1,
#'   merge_data = TRUE
#' )
#'
#' # Merge data while allowing interchangeable dimensions (e.g., A*B = B*A)
#' data_pattern_mixed <- get_data_by_dims(
#'   NULL,
#'   sl4_data, sl4_data1,
#'   merge_data = TRUE,
#'   pattern_mix = TRUE
#' )
#'
#' # Retain only "TOTAL" values
#' data_total_only <- get_data_by_dims(
#'   "comm*reg",
#'   sl4_data,
#'   subtotal_level = "total"
#' )
#' data_total_only_alt <- get_data_by_dims(
#'   "comm*reg",
#'   sl4_data,
#'   subtotal_level = FALSE
#' )
#'
#' # Retain only decomposed components
#' data_decomposed_only <- get_data_by_dims(
#'   "comm*reg",
#'   sl4_data,
#'   subtotal_level = "decomposed"
#' )
#'
#' # Retain all value levels
#' data_all_decomp <- get_data_by_dims(
#'   "comm*reg",
#'   sl4_data,
#'   subtotal_level = "all"
#' )
#' data_all_decomp_alt <- get_data_by_dims(
#'   "comm*reg",
#'   sl4_data,
#'   subtotal_level = TRUE
#' )
#'
#' # Rename specific columns
#' data_renamed <- get_data_by_dims(
#'   "comm*reg",
#'   sl4_data,
#'   rename_cols = c(REG = "Region", COMM = "Commodity")
#' )
#'
#' # Merge data with custom experiment names
#' data_merged_experiments <- get_data_by_dims(
#'   "comm*reg",
#'   sl4_data, sl4_data1,
#'   experiment_names = c("EXP1", "EXP2"),
#'   merge_data = TRUE
#' )
#'
get_data_by_dims <- function(patterns = NULL, ..., experiment_names = NULL,
                             subtotal_level = FALSE, rename_cols = NULL,
                             merge_data = FALSE, pattern_mix = FALSE) {
  data_list <- list(...)

  if (length(data_list) == 0) {
    stop("At least one data object is required.")
  }

  if (is.null(experiment_names) && !is.null(names(data_list))) {
    experiment_names <- names(data_list)
  } else if (is.null(experiment_names)) {
    dots <- match.call(expand.dots = FALSE)$...
    experiment_names <- if (length(dots) == 1) {
      deparse(dots[[1]])
    } else {
      vapply(dots, deparse, character(1))
    }
  }

  if (length(experiment_names) != length(data_list)) {
    stop("The number of experiment names must match the number of data objects.")
  }

  if (length(data_list) == 1) {
    merge_data <- FALSE
  }

  if (is.null(patterns) || identical(patterns, "ALL")) {
    patterns <- unique(unlist(lapply(data_list, function(x) {
      sapply(x$dimension_info, function(y) y$dimension_string)
    })))
  }

  extract_data <- function(data_obj, pattern, exp_name) {
    matching_vars <- character(0)
    for (var_name in names(data_obj$dimension_info)) {
      x <- data_obj$dimension_info[[var_name]]
      if (!is.null(x$dimension_string)) {
        if (pattern_mix) {
          p1 <- sort(strsplit(tolower(x$dimension_string), "\\*")[[1]])
          p2 <- sort(strsplit(tolower(pattern), "\\*")[[1]])
          if (identical(p1, p2)) matching_vars <- c(matching_vars, var_name)
        } else {
          if (tolower(x$dimension_string) == tolower(pattern)) {
            matching_vars <- c(matching_vars, var_name)
          }
        }
      }
    }

    if (length(matching_vars) == 0) return(NULL)

    df_list <- lapply(matching_vars, function(var_name) {
      var_data <- data_obj$data[[var_name]]
      if (length(dim(var_data)) == 0) return(NULL)

      dim_info <- data_obj$dimension_info[[var_name]]
      df <- as.data.frame.table(var_data, stringsAsFactors = FALSE, responseName = "Value")

      dim_names <- dim_info$dimension_names
      if (length(dim_names) > 0) {
        names(df)[1:length(dim_names)] <- dim_names
      }

      if ("subtotal" %in% tolower(names(df))) {
        names(df)[tolower(names(df)) == "subtotal"] <- "Subtotal"
      }

      df$Variable <- var_name
      df$Dimension <- dim_info$dimension_string
      df$Experiment <- exp_name

      if (!is.logical(subtotal_level) && !subtotal_level %in% c("all", "total", "decomposed")) {
        stop("subtotal_level must be either logical (TRUE/FALSE) or one of: 'all', 'total', 'decomposed'")
      }

      if ("Subtotal" %in% names(df)) {
        df <- switch(
          if(is.logical(subtotal_level)) if(subtotal_level) "all" else "total" else subtotal_level,
          "total" = df[df$Subtotal == "TOTAL", ],
          "decomposed" = df[df$Subtotal != "TOTAL", ],
          df
        )
      }

      if (!is.null(rename_cols)) {
        for (old_name in names(rename_cols)) {
          matches <- which(names(df) == old_name)
          if (length(matches) > 0) {
            for (i in seq_along(matches)) {
              names(df)[matches[i]] <- if(i == 1) rename_cols[old_name] else paste0(rename_cols[old_name], i-1)
            }
          }
        }
      }

      df[!is.na(df$Value), ]
    })

    df_list <- Filter(Negate(is.null), df_list)
    if (length(df_list) == 0) return(NULL)

    result <- do.call(rbind, df_list)
    rownames(result) <- NULL
    result
  }

  if (merge_data) {
    result <- lapply(patterns, function(pattern) {
      pattern_data <- lapply(seq_along(data_list), function(i) {
        extract_data(data_list[[i]], pattern, experiment_names[i])
      })
      pattern_data <- Filter(Negate(is.null), pattern_data)
      if (length(pattern_data) == 0) return(NULL)
      do.call(rbind, pattern_data)
    })
    names(result) <- patterns
    result <- Filter(Negate(is.null), result)

    if (length(result) == 0) {
      stop("No patterns could be processed")
    }

    attr(result, "n_patterns") <- length(result)
    attr(result, "patterns") <- names(result)
    if (length(data_list) > 1) {
      attr(result, "experiments") <- experiment_names
    }
    class(result) <- c("grouped_multi_data", class(result))

  } else {
    result <- lapply(seq_along(data_list), function(i) {
      pattern_data <- lapply(patterns, function(pattern) {
        extract_data(data_list[[i]], pattern, experiment_names[i])
      })
      pattern_data <- Filter(Negate(is.null), pattern_data)
      if (length(pattern_data) == 0) return(NULL)
      names(pattern_data) <- vapply(pattern_data, function(x) unique(x$Dimension)[1], character(1))
      pattern_data
    })
    names(result) <- experiment_names
    result <- Filter(Negate(is.null), result)

    if (length(data_list) == 1) {
      return(result[[1]])
    }

    if (length(result) == 0) {
      stop("No patterns could be processed for any experiment")
    }
  }

  return(result)
}


#' @title Group Data by Dimension Patterns in SL4 or HAR Objects
#'
#' @description Groups extracted SL4 or HAR data based on specified dimension structures and priority rules.
#' Supports automatic renaming, merging, subtotal filtering, and structured metadata handling.
#'
#' @details
#' - Groups extracted variables based on dimension elements.
#' - Applies predefined priority rules to structure the data.
#' - Allows automatic renaming of dimensions (`auto_rename = TRUE`).
#' - Supports merging of grouped data across multiple experiments.
#' - Handles subtotal filtering (`subtotal_level`), controlling whether `"TOTAL"` or decomposed values are retained.
#'
#' @param patterns Character vector. Dimension patterns to extract. Use `"ALL"` or `NULL` to extract all available patterns.
#' @param ... One or more SL4 or HAR objects loaded using `load_sl4x()` or `load_harx()`.
#' @param priority Named list. Specifies priority dimension elements (`c("group_name" = c("dim1", "dim2"))`).
#' @param experiment_names Character vector. Names assigned to each dataset. If `NULL`, names are inferred.
#' @param subtotal_level Character or logical. Determines which decomposition levels to retain:
#'   - `"total"`: Keeps only `"TOTAL"` values.
#'   - `"decomposed"`: Keeps only decomposed values (excludes `"TOTAL"`).
#'   - `"all"`: Keeps all rows.
#'   - `TRUE`: Equivalent to `"all"`, retaining both `"TOTAL"` and decomposed values.
#'   - `FALSE`: Equivalent to `"total"`, keeping only `"TOTAL"` values.
#' @param rename_cols Named vector. Column name replacements (`c("old_name" = "new_name")`).
#' @param auto_rename Logical. If `TRUE`, automatically renames dimensions for consistency. Default is `FALSE`.
#'
#' @return A structured list of grouped data:
#' - A named list where each element corresponds to a dimension size group (e.g., "2D", "3D").
#' - Each group contains dimension-grouped data based on priority rules.
#' - If unmerged data exists, includes a report attribute detailing merge issues.
#'
#' @importFrom stats setNames
#' @importFrom utils tail
#'
#' @author Pattawee Puangchit
#'
#' @seealso \code{\link{get_data_by_dims}}, \code{\link{get_data_by_var}}, \code{\link{load_sl4x}}, \code{\link{load_harx}}
#'
#' @export
#'
#' @examples
#' # Import sample data
#' sl4_data1 <- load_sl4x(system.file("extdata", "TAR10.sl4", package = "HARplus"))
#' sl4_data2 <- load_sl4x(system.file("extdata", "SUBT10.sl4", package = "HARplus"))
#'
#' # Case 1: Multiple priority levels (Sector then Region) with auto_rename
#' priority_list <- list(
#'   "Sector" = c("COMM", "ACTS"),
#'   "Region" = c("REG")
#' )
#' grouped_data_multiple <- group_data_by_dims(
#'   patterns = "ALL",
#'   sl4_data1,
#'   priority = priority_list,
#'   auto_rename = TRUE
#' )
#'
#' # Case 2: Single priority (Region only) with auto_rename
#' priority_list <- list("Region" = c("REG"))
#' grouped_data_single <- group_data_by_dims(
#'   patterns = "ALL",
#'   sl4_data1, sl4_data2,
#'   priority = priority_list,
#'   auto_rename = TRUE
#' )
#'
#' # Case 3: Multiple priorities without auto_rename
#' priority_list <- list(
#'   "Sector" = c("COMM", "ACTS"),
#'   "Region" = c("REG")
#' )
#' grouped_data_no_rename <- group_data_by_dims(
#'   patterns = "ALL",
#'   sl4_data1,
#'   priority = priority_list,
#'   auto_rename = FALSE
#' )
group_data_by_dims <- function(patterns = NULL, ..., priority,
                               rename_cols = NULL, experiment_names = NULL,
                               subtotal_level = FALSE, auto_rename = FALSE) {
  if (is.null(experiment_names)) {
    dots <- match.call(expand.dots = FALSE)$...
    experiment_names <- if (length(dots) == 1) {
      deparse(dots[[1]])
    } else {
      vapply(dots, deparse, character(1))
    }
  }

  data_list <- list(...)

  all_dims <- get_dim_elements(...)$DimName
  priority_elements <- unlist(priority)
  auto_rename_elements <- setdiff(all_dims, priority_elements)

  extracted_data <- get_data_by_dims(
    patterns = patterns, ...,
    pattern_mix = TRUE,
    merge_data = TRUE,
    subtotal_level = subtotal_level,
    experiment_names = experiment_names
  )

  dim_sizes <- vapply(names(extracted_data), function(x) {
    length(strsplit(x, "\\*")[[1]])
  }, numeric(1))

  result <- setNames(
    vector("list", length(unique(dim_sizes))),
    paste0(sort(unique(dim_sizes)), "D")
  )

  report <- data.frame(
    Pattern = character(),
    DimSize = character(),
    Reason = character(),
    stringsAsFactors = FALSE
  )

  get_highest_priority <- function(pattern, priority_list) {
    pattern_elements <- strsplit(pattern, "\\*")[[1]]
    for (prio_name in names(priority_list)) {
      if (any(tolower(priority_list[[prio_name]]) %in% tolower(pattern_elements))) {
        return(prio_name)
      }
    }
    return(NULL)
  }

  check_mergeable <- function(pattern) {
    elements <- strsplit(pattern, "\\*")[[1]]

    if (length(elements) != length(unique(elements))) {
      return(list(mergeable = FALSE, reason = "Duplicate elements detected"))
    }

    if (!any(tolower(elements) %in% tolower(priority_elements))) {
      return(list(mergeable = FALSE, reason = "No priority elements present"))
    }

    return(list(mergeable = TRUE, reason = NULL))
  }

  handle_duplicate_cols <- function(df) {
    col_counts <- table(colnames(df))
    duplicate_cols <- names(col_counts[col_counts > 1])

    if (length(duplicate_cols) > 0) {
      for (col in duplicate_cols) {
        col_indices <- which(colnames(df) == col)
        for (i in seq_along(col_indices)[-1]) {
          colnames(df)[col_indices[i]] <- paste0(col, ".", i-1)
        }
      }
    }
    return(df)
  }

  process_dimension <- function(dim_size, patterns, priority_list) {
    current_patterns <- patterns[dim_sizes == dim_size]
    if (length(current_patterns) == 0) return(list())

    if (auto_rename) {
      dimension_groups <- list()

      if (dim_size <= 2) {
        pattern_groups <- list()
        other_patterns <- character(0)

        for (pattern in current_patterns) {
          highest_prio <- get_highest_priority(pattern, priority_list)
          if (!is.null(highest_prio)) {
            pattern_groups[[highest_prio]] <- c(pattern_groups[[highest_prio]], pattern)
          } else {
            other_patterns <- c(other_patterns, pattern)
          }
        }

        for (prio_name in names(priority_list)) {
          matching_patterns <- pattern_groups[[prio_name]]

          if (length(matching_patterns) > 0) {
            df_list <- lapply(matching_patterns, function(pattern) {
              df <- extracted_data[[pattern]]

              col_mapping <- list()
              for (col in colnames(df)) {
                for (p_name in names(priority_list)) {
                  prio_elements <- priority_list[[p_name]]
                  if (any(tolower(col) == tolower(prio_elements))) {
                    col_mapping[[col]] <- p_name
                  }
                }
              }

              for (old_col in names(col_mapping)) {
                colnames(df)[colnames(df) == old_col] <- col_mapping[[old_col]]
              }

              if (auto_rename) {
                for (col in colnames(df)) {
                  if (col %in% auto_rename_elements) {
                    colnames(df)[colnames(df) == col] <- "Dim1"
                  }
                }
              }

              df <- handle_duplicate_cols(df)
              return(df)
            })

            common_cols <- Reduce(intersect, lapply(df_list, colnames))
            merged_df <- do.call(rbind, lapply(df_list, function(x) {
              x[, common_cols, drop = FALSE]
            }))

            dimension_groups[[prio_name]] <- merged_df
          }
        }

        if (length(other_patterns) > 0) {
          col_counts <- lapply(other_patterns, function(pattern) {
            ncol(extracted_data[[pattern]])
          })
          unique_counts <- unique(unlist(col_counts))

          for (count in unique_counts) {
            matching_others <- other_patterns[vapply(other_patterns,
                                                     function(p) ncol(extracted_data[[p]]) == count,
                                                     logical(1))]

            if (length(matching_others) > 0) {
              df_list <- lapply(matching_others, function(pattern) {
                df <- extracted_data[[pattern]]
                if (auto_rename) {
                  for (col in colnames(df)) {
                    if (col %in% auto_rename_elements) {
                      colnames(df)[colnames(df) == col] <- "Dim1"
                    }
                  }
                }
                df <- handle_duplicate_cols(df)
                return(df)
              })

              common_cols <- Reduce(intersect, lapply(df_list, colnames))
              merged_df <- do.call(rbind, lapply(df_list, function(x) {
                x[, common_cols, drop = FALSE]
              }))

              group_name <- paste0("Other", if(length(unique_counts) > 1) count else "")
              dimension_groups[[group_name]] <- merged_df
            }
          }
        }
      } else {
        for (pattern in current_patterns) {
          df <- extracted_data[[pattern]]
          df <- handle_duplicate_cols(df)
          dimension_groups[[pattern]] <- df
        }
      }
      return(dimension_groups)

    } else {
      dimension_groups <- list()

      if (dim_size <= 2) {
        pattern_groups <- list()
        unmerged_patterns <- character(0)

        for (pattern in current_patterns) {
          merge_check <- check_mergeable(pattern)
          if (!merge_check$mergeable) {
            report <<- rbind(report, data.frame(
              Pattern = pattern,
              DimSize = paste0(dim_size, "D"),
              Reason = merge_check$reason,
              stringsAsFactors = FALSE
            ))
            unmerged_patterns <- c(unmerged_patterns, pattern)
            next
          }

          highest_prio <- get_highest_priority(pattern, priority_list)
          if (!is.null(highest_prio)) {
            pattern_groups[[highest_prio]] <- c(pattern_groups[[highest_prio]], pattern)
          } else {
            unmerged_patterns <- c(unmerged_patterns, pattern)
          }
        }

        for (prio_name in names(priority_list)) {
          matching_patterns <- pattern_groups[[prio_name]]
          if (length(matching_patterns) > 0) {
            df_list <- lapply(matching_patterns, function(pattern) {
              df <- extracted_data[[pattern]]
              for (p_name in names(priority_list)) {
                prio_elements <- priority_list[[p_name]]
                for (elem in prio_elements) {
                  if (tolower(elem) %in% tolower(colnames(df))) {
                    colnames(df)[tolower(colnames(df)) == tolower(elem)] <- p_name
                  }
                }
              }
              # Handle duplicates in column names
              df <- handle_duplicate_cols(df)
              return(df)
            })

            col_names_list <- lapply(df_list, colnames)
            if (length(unique(lapply(col_names_list, paste, collapse = "|"))) == 1) {
              merged_df <- do.call(rbind, df_list)
              dimension_groups[[prio_name]] <- merged_df
            } else {
              report <<- rbind(report, data.frame(
                Pattern = matching_patterns,
                DimSize = paste0(dim_size, "D"),
                Reason = "Different column names (fixed by auto_rename = TRUE)",
                stringsAsFactors = FALSE
              ))
              unmerged_patterns <- c(unmerged_patterns, matching_patterns)
            }
          }
        }

        if (length(unmerged_patterns) > 0) {
          dimension_groups$unmerged <- lapply(extracted_data[unmerged_patterns], handle_duplicate_cols)
        }

      } else {
        for (pattern in current_patterns) {
          df <- extracted_data[[pattern]]
          df <- handle_duplicate_cols(df)
          dimension_groups[[pattern]] <- df
        }
      }

      return(dimension_groups)
    }
  }

  for (dim_key in names(result)) {
    dim_size <- as.numeric(sub("D$", "", dim_key))
    result[[dim_key]] <- process_dimension(dim_size, names(extracted_data), priority)
  }

  if (!is.null(rename_cols)) {
    result <- lapply(result, function(dim_group) {
      lapply(dim_group, function(df) {
        rename_col(df, rename_cols)
      })
    })
  }

  has_unmerged <- any(sapply(result, function(dim_group) {
    "unmerged" %in% names(dim_group)
  }))

  if (has_unmerged) {
    sorted_report <- report[order(report$DimSize), ]
    attr(result, "report") <- sorted_report
    result$report <- sorted_report
  }

  class(result) <- c("grouped_dims_element", class(result))
  return(result)
}

#' @title Rename Dimensions in SL4 or HAR Objects
#' @description Renames dimension and list names in structured SL4 or HAR objects.
#'
#' @details
#' - Replaces old dimension names with new ones as specified in `mapping_df`.
#' - If `rename_list_names = TRUE`, renames list element names as well.
#' - Ensures consistency across SL4 and HAR datasets.
#'
#' @param data_obj A structured SL4 or HAR object.
#' @param mapping_df A two-column data frame where the first column (`old`) contains the current names, 
#'        and the second column (`new`) contains the new names.
#' @param rename_list_names Logical. If `TRUE`, renames list element names. Default is `FALSE`.
#'
#' @return The modified SL4 or HAR object with updated dimension names and, optionally, updated list names.
#'
#' @author Pattawee Puangchit
#' 
#' @seealso \code{\link{get_data_by_var}}, \code{\link{get_data_by_dims}}
#'
#' @export
#'
#' @examples
#' # Import sample data:
#' sl4_data <- load_sl4x(system.file("extdata", "TAR10.sl4", package = "HARplus"))
#' 
#' # Define a renaming map
#' mapping_df <- data.frame(
#'   old = c("REG", "COMM"),
#'   new = c("Region", "Commodity")
#' )
#'
#' # Rename columns in the dataset
#' rename_dims(sl4_data, mapping_df)
#' 
#' # Rename both columns and list names
#' rename_dims(sl4_data, mapping_df, rename_list_names = TRUE)
#' 
rename_dims <- function(data_obj, mapping_df, rename_list_names = FALSE) {
  if (!is.list(data_obj)) {
    stop("Invalid input: Expected a list object.")
  }
  if (!is.data.frame(mapping_df) || ncol(mapping_df) != 2) {
    stop("Invalid mapping_df: Expected a two-column data frame with old and new names.")
  }
  
  old_names <- mapping_df[[1]]
  new_names <- mapping_df[[2]]
  rename_cols <- setNames(new_names, old_names)
  
  rename_parts <- function(str, mapping) {
    parts <- strsplit(str, "\\*")[[1]]
    new_parts <- sapply(parts, function(part) {
      if (part %in% names(mapping)) {
        mapping[[part]]
      } else {
        part
      }
    })
    paste(new_parts, collapse = "*")
  }
  
  if (rename_list_names) {
    old_names <- names(data_obj)
    new_names <- sapply(old_names, function(name) {
      rename_parts(name, rename_cols)
    })
    names(data_obj) <- new_names
  }
  
  for (var_name in names(data_obj)) {
    df <- data_obj[[var_name]]
    
    if (!is.data.frame(df)) next  
    
    for (old_name in names(rename_cols)) {
      matching_cols <- which(names(df) == old_name)
      if (length(matching_cols) > 0) {
        for (i in seq_along(matching_cols)) {
          new_name <- if (i == 1) rename_cols[old_name] else paste0(rename_cols[old_name], i - 1)
          names(df)[matching_cols[i]] <- new_name
        }
      }
    }
    
    data_obj[[var_name]] <- df 
  }
  
  return(data_obj)  
}


#' @title Pivot Data from SL4 or HAR Objects
#' 
#' @description Transforms long-format SL4 or HAR data into wide format by pivoting selected columns.
#' Supports both single data frames and nested lists.
#'
#' @details
#' - Uses `tidyr::pivot_wider()` internally to reshape data.
#' - Allows multiple columns to be pivoted simultaneously.
#' - Recursively processes nested lists, ensuring all data frames are transformed.
#'
#' @param data_obj A list or data frame. The SL4 or HAR data to pivot.
#' @param pivot_cols Character vector. Column names to use as pivot keys.
#' @param name_repair Character. Method for handling duplicate column names (`"unique"`, `"minimal"`, `"universal"`). Default is `"unique"`.
#'
#' @return A transformed data object where the specified `pivot_cols` are pivoted into wide format.
#'
#' @importFrom tidyr pivot_wider
#'
#' @author Pattawee Puangchit
#' 
#' @seealso \code{\link{get_data_by_var}}, \code{\link{get_data_by_dims}}
#'
#' @export
#'
#' @examples
#' # Import sample data:
#' sl4_data <- load_sl4x(system.file("extdata", "TAR10.sl4", package = "HARplus"))
#' 
#' # Extract multiple variables
#' data_multiple <- get_data_by_var(c("qo", "qxs"), sl4_data)
#' 
#' # Pivot a single column
#' pivoted_data <- pivot_data(data_multiple, pivot_cols = "REG")
#'
#' # Pivot multiple columns
#' pivoted_data_multi <- pivot_data(data_multiple, pivot_cols = c("REG", "COMM"))
#' 
pivot_data <- function(data_obj, pivot_cols, name_repair = "unique") {
  pivot_df <- function(df, cols) {
    if (!inherits(df, "data.frame")) return(df)
    
    available_cols <- cols[cols %in% names(df)]
    if (length(available_cols) == 0) return(df)
    
    non_pivot_cols <- setdiff(names(df), c(available_cols, "Value"))
    
    names_prefix <- paste(available_cols, collapse = "_")
    
    tidyr::pivot_wider(df,
                       id_cols = tidyselect::all_of(non_pivot_cols),
                       names_from = tidyselect::all_of(available_cols),
                       values_from = "Value",
                       names_sep = "_",
                       names_repair = name_repair,
                       names_prefix = paste0(names_prefix, "_"))
  }
  
  process_nested <- function(obj, cols) {
    if (is.data.frame(obj)) {
      return(pivot_df(obj, cols))
    } else if (is.list(obj)) {
      return(lapply(obj, function(x) process_nested(x, cols)))
    }
    return(obj)
  }
  
  process_nested(data_obj, pivot_cols)
}


#' @title Export Data to Various Formats (CSV/STATA/TEXT/RDS/XLSX)
#' 
#' @description Exports structured SL4 or HAR data to multiple file formats, including CSV, Stata, TXT, RDS, and XLSX.
#' Supports nested lists, automatic subfolder creation, and multi-sheet Excel exports.
#'
#' @details
#' - Supports exporting data in `"csv"`, `"stata"`, `"txt"`, `"rds"`, and `"xlsx"` formats.
#' - Handles nested lists and exports each data frame individually.
#' - Optionally creates subfolders for each format (`create_subfolder = TRUE`).
#' - Customizes file names using `prefix`.
#' - When multi_sheet_xlsx = TRUE, all exported data is stored in a **single Excel workbook**, with each dataset as a separate sheet.
#'
#' @param data A list or data frame. The SL4 or HAR data to export.
#' @param output_path Character. The base output directory or file path.
#' @param format Character. The export format (`"csv"`, `"stata"`, `"txt"`, `"rds"`, `"xlsx"`). Default is `"csv"`.
#' @param prefix Character. An optional prefix added to exported file names. Default is `""` (empty).
#' @param create_subfolder Logical. If `TRUE`, creates a subfolder for each format. Default is `FALSE`.
#' @param multi_sheet_xlsx Logical. If TRUE, exports lists as multi-sheet XLSX files.
#' @param xlsx_filename An optional filename for the XLSX file (used when `multi_sheet_xlsx = TRUE`).
#' @param report_output Logical. If TRUE, generates an export report.
#' 
#' @details
#' - If exporting to Stata (`"stata"` format), column names containing `.` will be replaced with `_` to ensure compatibility.
#' - If `multi_sheet_xlsx = TRUE`, list elements are exported as separate sheets in a single XLSX file.
#' - The function creates necessary directories if they do not exist.
#' 
#' @return A list containing the file paths of the exported data.
#' 
#' @importFrom utils write.csv write.table
#' @importFrom haven write_dta
#' @importFrom openxlsx createWorkbook addWorksheet writeData saveWorkbook
#' @importFrom tools file_ext
#'
#' @author Pattawee Puangchit
#' 
#' @seealso \code{\link{pivot_data}}, \code{\link{get_data_by_var}}, \code{\link{get_data_by_dims}}
#'
#' @export
#' 
#' @examples
#' \donttest{
#' # Import sample data:
#' sl4_data <- load_sl4x(system.file("extdata", "TAR10.sl4", package = "HARplus"))
#' 
#' # Extract data
#' data_multiple <- get_data_by_var(c("qo", "pca"), sl4_data)
#' 
#' # Export
#' export_data(data_multiple, file.path(tempdir(), "output_directory"), 
#'            format = c("csv", "xlsx", "stata", "txt", "rds"),
#'            create_subfolder = TRUE,
#'            multi_sheet_xlsx = TRUE)
#' }
#' 
export_data <- function(data, output_path, format = "csv", prefix = "", 
                        create_subfolder = FALSE, multi_sheet_xlsx = FALSE,
                        xlsx_filename = NULL, report_output = FALSE) {
  
  data_name <- as.character(match.call()$data)
  
  default_filename <- if (is.null(xlsx_filename)) {
    basename(tools::file_path_sans_ext(output_path))
  } else {
    xlsx_filename
  }
  
  supported_formats <- c("csv", "stata", "txt", "rds", "xlsx")
  if (!all(format %in% supported_formats)) {
    invalid_formats <- format[!format %in% supported_formats]
    stop("Unsupported format(s): ", paste(invalid_formats, collapse = ", "))
  }
  
  output_dir <- if (tools::file_ext(output_path) == "") {
    output_path
  } else {
    dirname(output_path)
  }
  
  if (!dir.exists(output_dir)) {
    dir.create(output_dir, recursive = TRUE)
  }
  
  exported_files <- list()
  
  for (fmt in format) {
    format_dir <- if (create_subfolder) {
      dir_path <- file.path(output_dir, fmt)
      dir.create(dir_path, showWarnings = FALSE, recursive = TRUE)
      dir_path
    } else {
      output_dir
    }
    
    export_single <- function(df, name) {
      ext <- if(fmt == "stata") "dta" else fmt
      file_path <- file.path(format_dir, paste0(prefix, name, ".", ext))
      
      if (fmt == "stata") {
        df_stata <- df
        names(df_stata) <- gsub("\\.", "_", names(df_stata))
        haven::write_dta(df_stata, file_path)
      } else if (fmt == "xlsx") {
        wb <- openxlsx::createWorkbook()
        sheet_name <- make.names(substr(name, 1, 31))
        openxlsx::addWorksheet(wb, sheet_name)
        openxlsx::writeData(wb, sheet_name, df)
        openxlsx::saveWorkbook(wb, file_path, overwrite = TRUE)
      } else {
        switch(fmt,
               "csv" = utils::write.csv(df, file_path, row.names = FALSE),
               "txt" = utils::write.table(df, file_path, sep = "\t", 
                                          row.names = FALSE, quote = FALSE),
               "rds" = base::saveRDS(df, file_path))
      }
      return(file_path)
    }
    
    process_list <- function(lst, parent = "") {
      if (is.data.frame(lst)) {
        name <- if (parent == "") "data" else parent
        return(export_single(lst, name))
      }
      
      if (fmt == "xlsx" && multi_sheet_xlsx) {
        wb <- openxlsx::createWorkbook()
        sheets_added <- FALSE
        
        for (name in names(lst)) {
          if (name == "report") next
          
          item <- lst[[name]]
          current_name <- if (parent == "") name else paste(parent, name, sep = "_")
          current_name <- make.names(substr(gsub("\\*", "_", current_name), 1, 31))
          
          if (is.data.frame(item)) {
            openxlsx::addWorksheet(wb, current_name)
            openxlsx::writeData(wb, current_name, item)
            sheets_added <- TRUE
          } else if (is.list(item)) {
            for (subname in names(item)) {
              if (is.data.frame(item[[subname]])) {
                sheet_name <- make.names(substr(paste(current_name, subname, sep = "_"), 1, 31))
                openxlsx::addWorksheet(wb, sheet_name)
                openxlsx::writeData(wb, sheet_name, item[[subname]])
                sheets_added <- TRUE
              }
            }
          }
        }
        
        if (sheets_added) {
          file_path <- file.path(format_dir, paste0(prefix, default_filename, ".xlsx"))
          openxlsx::saveWorkbook(wb, file_path, overwrite = TRUE)
          return(file_path)
        }
        return(NULL)
      } else {
        exported <- lapply(names(lst), function(name) {
          if (name == "report") return(NULL)
          
          item <- lst[[name]]
          current_name <- if (parent == "") name else paste(parent, name, sep = "_")
          current_name <- gsub("\\*", "_", current_name)
          
          if (is.data.frame(item)) {
            export_single(item, current_name)
          } else if (is.list(item)) {
            process_list(item, current_name)
          }
        })
        return(unlist(exported))
      }
    }
    
    result <- process_list(data)
    if (!is.null(result)) {
      if (fmt == "xlsx" && multi_sheet_xlsx) {
        message(sprintf("Exported multi-sheet Excel file '%s.xlsx' to %s", 
                        default_filename, normalizePath(format_dir)))
      } else {
        exported_files[[fmt]] <- result
        message(sprintf("Exported %d file(s) to %s format in %s", 
                        length(result), fmt, normalizePath(format_dir)))
      }
    }
  }
  
  if (report_output) {
    process_export_report(data, output_path, prefix, data_name)
  }
  
  invisible(exported_files)
}



# Pivot Data Hirarchy -----------------------------------------------------

#' @title Create Hierarchical Pivot Table from SL4 or HAR Objects
#' 
#' @description Creates hierarchical pivot tables from structured SL4 or HAR data, with optional Excel export.
#' Supports both single data frames and nested lists, preserving dimension hierarchies.
#'
#' @details
#' - Transforms data into hierarchical pivot format with nested column headers.
#' - Supports multiple pivot columns in specified order (e.g., REG > COMM).
#' - Handles both single data frames and nested list structures.
#' - Optional direct export to Excel with formatted hierarchical headers.
#' - Uses efficient data processing with tidyr and openxlsx.
#'
#' @param data_obj A list or data frame. The SL4 or HAR data to pivot.
#' @param pivot_cols Character vector. Column names to use as pivot keys in order of hierarchy.
#' @param name_repair Character. Method for handling duplicate column names 
#'        (`"unique"`, `"minimal"`, `"universal"`). Default is `"unique"`.
#' @param export Logical. If `TRUE`, exports result to Excel. Default is `FALSE`.
#' @param file_path Character. Required if export = TRUE. The path for Excel export.
#' @param xlsx_filename Character. The name for the Excel file when using multi_sheet_xlsx. If NULL, uses the name of the dataset. Default is `NULL`.
#' 
#' @return A pivoted data object with hierarchical structure:
#' - If input is a data frame: Returns a hierarchical pivot table.
#' - If input is a list: Returns a nested list of hierarchical pivot tables.
#' - If export = TRUE: Invisibly returns the pivoted object after Excel export.
#'
#' @author Pattawee Puangchit
#' 
#' @seealso \code{\link{pivot_data}}
#'
#' @importFrom tidyr pivot_wider
#' @importFrom tidyselect all_of
#' @importFrom openxlsx createWorkbook addWorksheet writeData createStyle addStyle saveWorkbook
#'
#' @export
#'
#' @examples
#' \donttest{
#' # Import sample data:
#' sl4_data <- load_sl4x(system.file("extdata", "TAR10.sl4", package = "HARplus"))
#' 
#' # Extract data
#' data_multiple <- get_data_by_var(c("qo", "pca"), sl4_data)
#' 
#' # Create hierarchical pivot without export
#' pivot_hier <- pivot_data_hierarchy(data_multiple, 
#'                                    pivot_cols = c("REG", "COMM"))
#' 
#' # Create and export to Excel in one step
#' pivot_export <- pivot_data_hierarchy(data_multiple, 
#'                                      pivot_cols = c("REG", "COMM"),
#'                                      export = TRUE,
#'                                      file_path = file.path(tempdir(), "pivot_output.xlsx"))
#' }
#' 
pivot_data_hierarchy <- function(data_obj, pivot_cols, name_repair = "unique", 
                                 export = FALSE, file_path = NULL,
                                 xlsx_filename = NULL) {
  if (export && is.null(xlsx_filename)) {
    xlsx_filename <- basename(tools::file_path_sans_ext(file_path))
  }
  if (!is.list(data_obj) && !is.data.frame(data_obj)) {
    stop("data_obj must be a list or data frame")
  }
  if (!is.character(pivot_cols) || length(pivot_cols) == 0) {
    stop("pivot_cols must be a non-empty character vector")
  }
  if (export && (is.null(file_path) || !is.character(file_path))) {
    stop("file_path must be provided when export = TRUE")
  }
  
  pivot_df <- function(df, cols) {
    if (!inherits(df, "data.frame")) return(df)
    
    available_cols <- cols[cols %in% names(df)]
    if (length(available_cols) == 0) return(df)
    
    non_pivot_cols <- setdiff(names(df), c(available_cols, "Value"))
    
    # Create pivot using tidyr
    pivoted <- tidyr::pivot_wider(df,
                                  id_cols = tidyselect::all_of(non_pivot_cols),
                                  names_from = tidyselect::all_of(available_cols),
                                  values_from = "Value",
                                  names_sep = ".",
                                  names_repair = name_repair)
    
    # Create hierarchical header matrix
    col_names <- colnames(pivoted)
    pivot_cols <- col_names[!col_names %in% non_pivot_cols]
    
    if (length(pivot_cols) > 0) {
      split_names <- strsplit(pivot_cols, "\\.")
      max_levels <- max(sapply(split_names, length))
      
      header_matrix <- matrix("", nrow = max_levels, ncol = length(col_names))
      
      # Fill non-pivot columns
      for (i in seq_along(non_pivot_cols)) {
        header_matrix[1, i] <- non_pivot_cols[i]
      }
      
      # Fill pivot columns
      for (i in seq_along(pivot_cols)) {
        parts <- split_names[[i]]
        col_idx <- which(col_names == pivot_cols[i])
        for (j in seq_along(parts)) {
          header_matrix[j, col_idx] <- parts[j]
        }
      }
      
      attr(pivoted, "header_matrix") <- header_matrix
      class(pivoted) <- c("hierarchical_pivot", class(pivoted))
    }
    
    return(pivoted)
  }
  
  # Process data object
  result <- if (is.data.frame(data_obj)) {
    pivot_df(data_obj, pivot_cols)
  } else {
    nested_result <- lapply(data_obj, function(nested) {
      if (is.list(nested)) {
        lapply(nested, function(df) pivot_df(df, pivot_cols))
      } else {
        pivot_df(nested, pivot_cols)
      }
    })
    class(nested_result) <- c("hierarchical_pivot_list", class(nested_result))
    nested_result
  }
  
  # Handle export if requested
  if (export) {
    export_hierarchy_to_excel(result, file_path)
  }
  
  return(result)
}


#' @title Export Hierarchical Pivot Table to Excel (Internal)
#'
#' @description Internal function to export hierarchical pivot tables to Excel with formatted headers.
#'
#' @param pivot_df A hierarchical pivot object created by pivot_data_hierarchy().
#' @param file_path Character. The file path for Excel export.
#' @param xlsx_filename Character. The name for the Excel file when using multi_sheet_xlsx. If NULL, uses the name of the dataset. Default is `NULL`.
#' 
#' @return Invisibly returns NULL.
#'
#' @keywords internal
#'
export_hierarchy_to_excel <- function(pivot_df, file_path, xlsx_filename = NULL) {
  wb <- openxlsx::createWorkbook()
  
  write_sheet <- function(df, sheet_name) {
    sheet_name <- make.names(substr(gsub("[^[:alnum:]]", "", sheet_name), 1, 31))
    
    openxlsx::addWorksheet(wb, sheet_name)
    
    if (inherits(df, "hierarchical_pivot")) {
      header_matrix <- attr(df, "header_matrix")
      if (!is.null(header_matrix)) {
        openxlsx::writeData(wb, sheet_name, header_matrix, startRow = 1, colNames = FALSE)
        openxlsx::writeData(wb, sheet_name, df, 
                            startRow = nrow(header_matrix) + 1, 
                            colNames = FALSE)
        
        header_style <- openxlsx::createStyle(textDecoration = "bold")
        openxlsx::addStyle(wb, sheet_name, style = header_style,
                           rows = 1:nrow(header_matrix),
                           cols = 1:ncol(header_matrix),
                           gridExpand = TRUE)
      } else {
        openxlsx::writeData(wb, sheet_name, df)
      }
    } else {
      openxlsx::writeData(wb, sheet_name, df)
    }
  }
  
  process_list <- function(obj) {
    if (inherits(obj, "hierarchical_pivot")) {
      write_sheet(obj, "pivot_data")
    } else if (is.list(obj)) {
      for (name in names(obj)) {
        if (is.data.frame(obj[[name]])) {
          write_sheet(obj[[name]], name)
        } else if (is.list(obj[[name]])) {
          lapply(names(obj[[name]]), function(subname) {
            if (is.data.frame(obj[[name]][[subname]])) {
              write_sheet(obj[[name]][[subname]], paste(name, subname, sep = "_"))
            }
          })
        }
      }
    }
  }
  
  process_list(pivot_df)
  
  actual_filename <- paste0(xlsx_filename, ".xlsx")
  if (!grepl("\\.xlsx$", file_path)) {
    final_path <- file.path(file_path, actual_filename)
  } else {
    final_path <- file_path
  }
  
  openxlsx::saveWorkbook(wb, final_path, overwrite = TRUE)
  
  message(sprintf("Exported hierarchical pivot Excel file '%s' to %s", 
                  actual_filename, normalizePath(dirname(final_path))))
  
  invisible(NULL)
}

#' @title Save Data to GEMPACK HAR Format
#'
#' @description
#' Writes a named list of R objects to a GEMPACK-compatible HAR file, including
#' character sets, mapping vectors, integer matrices, numeric arrays, sparse
#' numeric arrays, and data frames reshaped to arrays.
#'
#' @param data_list Named list of objects to write. List names are used as HAR
#'   header names after conversion to uppercase and truncation to four characters.
#' @param file_path Character string giving the output HAR file path.
#' @param dimensions Optional named list. For data-frame inputs, each element
#'   gives the columns used as array dimensions.
#' @param value_cols Optional named list or named character vector. For
#'   data-frame inputs, each element gives the numeric value column. Defaults to
#'   \code{"Value"} when omitted.
#' @param header_type Optional named list or named character vector giving
#'   explicit header roles. Accepted values are \code{"auto"}, \code{"set"},
#'   \code{"mapping"}, \code{"real"}, \code{"sparse"}, and \code{"integer"}.
#' @param mappings Optional named list. Each element must be
#'   \code{c(source_set, destination_set)} for the corresponding mapping header.
#' @param long_desc Optional named list or named character vector of long header
#'   descriptions.
#' @param coefficients Optional named list or named character vector of
#'   coefficient names for numeric headers.
#' @param export_sets Logical. If \code{TRUE}, dimension sets from numeric
#'   arrays are written as character headers unless already supplied. Default is
#'   \code{TRUE}.
#' @param lowercase Logical. If \code{TRUE}, character elements and dimension
#'   values are converted to lowercase during data-frame and ordering
#'   processing. Default is \code{TRUE}.
#' @param dim_order Optional dimension-ordering specification. Can be
#'   \code{NULL}, a data frame, a named list, or a path to a CSV or Excel file.
#' @param dim_rename Optional named list for renaming array dimensions in the
#'   HAR output.
#' @param force_sparse Optional character vector of headers to write in sparse
#'   numeric format.
#' @param max_chunk Integer. Maximum number of elements per dense numeric data
#'   chunk. Default is \code{2e6}.
#'
#' @return Invisibly returns a list containing the output path, written headers,
#'   and counts of set, data, and mapping headers.
#'
#' @author Pattawee Puangchit
#'
#' @seealso \code{\link{load_harx}}, \code{\link{load_sl4x}}
#'
#' @export
#'
#' @examples
#' # Example 1: Save one numeric data frame
#' REG <- c("USA", "EU", "ROW")
#' COLUMN <- c("alloc_A1", "tot_E1")
#' WELF <- expand.grid(REG = REG, COLUMN = COLUMN, stringsAsFactors = FALSE)
#' WELF$Value <- seq_len(nrow(WELF))
#'
#' save_har(
#'   data_list = list(WELF = WELF),
#'   file_path = file.path(tempdir(), "output_single.har"),
#'   dimensions = list(WELF = c("REG", "COLUMN")),
#'   value_cols = list(WELF = "Value"),
#'   long_desc = list(WELF = "Welfare Decomposition"),
#'   coefficients = list(WELF = "WELF"),
#'   export_sets = TRUE,
#'   lowercase = FALSE
#' )
#'
#' # Example 2: Save multiple numeric data frames
#' DECOM <- expand.grid(REG = REG, ALLOCEFF = c("A1", "A2"), stringsAsFactors = FALSE)
#' DECOM$Value <- seq_len(nrow(DECOM))
#'
#' save_har(
#'   data_list = list(WELF = WELF, DECOM = DECOM),
#'   file_path = file.path(tempdir(), "output_multi.har"),
#'   dimensions = list(
#'     WELF = c("REG", "COLUMN"),
#'     DECOM = c("REG", "ALLOCEFF")
#'   ),
#'   value_cols = list(
#'     WELF = "Value",
#'     DECOM = "Value"
#'   ),
#'   long_desc = list(
#'     WELF = "Welfare Decomposition",
#'     DECOM = "Allocative efficiency effect"
#'   ),
#'   coefficients = list(
#'     WELF = "WELF",
#'     DECOM = "DECOM"
#'   ),
#'   export_sets = TRUE,
#'   lowercase = FALSE
#' )
#'
#' # Example 3: Save a mapping vector
#' SC <- c("AF", "AP", "BA")
#' GSEC <- c("OCR", "V_F", "GRO")
#' MASC <- c(AF = "OCR", AP = "V_F", BA = "GRO")
#'
#' save_har(
#'   data_list = list(SC = SC, GSEC = GSEC, MASC = MASC),
#'   file_path = file.path(tempdir(), "mapping.har"),
#'   mappings = list(MASC = c("SC", "GSEC")),
#'   long_desc = list(
#'     SC = "Set SC",
#'     GSEC = "Set GSEC",
#'     MASC = "Mapping SC to GSEC"
#'   ),
#'   export_sets = FALSE,
#'   lowercase = FALSE
#' )
#'
#' # Example 4: Save mixed headers
#' CODE <- matrix(as.integer(c(1, 2, 3, 4)), nrow = 2)
#' TAX <- expand.grid(SC = SC, REG = REG, stringsAsFactors = FALSE)
#' TAX$Value <- c(0, 0, 0, 1.2, 0, 0, 0, 0, 2.5)
#'
#' save_har(
#'   data_list = list(
#'     WELF = WELF,
#'     REG = REG,
#'     SC = SC,
#'     GSEC = GSEC,
#'     MASC = MASC,
#'     TAX = TAX,
#'     CODE = CODE
#'   ),
#'   file_path = file.path(tempdir(), "output_mixed.har"),
#'   dimensions = list(
#'     WELF = c("REG", "COLUMN"),
#'     TAX = c("SC", "REG")
#'   ),
#'   value_cols = list(
#'     WELF = "Value",
#'     TAX = "Value"
#'   ),
#'   mappings = list(
#'     MASC = c("SC", "GSEC")
#'   ),
#'   long_desc = list(
#'     WELF = "Welfare Decomposition",
#'     REG = "Set REG",
#'     SC = "Set SC",
#'     GSEC = "Set GSEC",
#'     MASC = "Mapping SC to GSEC",
#'     TAX = "Sparse tax example",
#'     CODE = "Integer matrix example"
#'   ),
#'   coefficients = list(
#'     WELF = "WELF",
#'     TAX = "TAX"
#'   ),
#'   force_sparse = "TAX",
#'   export_sets = FALSE,
#'   lowercase = FALSE
#' )
#'
#' # Example 5: Apply custom dimension ordering
#' dim_order <- list(
#'   REG = c("ROW", "USA", "EU"),
#'   COLUMN = c("tot_E1", "alloc_A1")
#' )
#'
#' save_har(
#'   data_list = list(WELF = WELF),
#'   file_path = file.path(tempdir(), "output_sorted.har"),
#'   dimensions = list(WELF = c("REG", "COLUMN")),
#'   value_cols = list(WELF = "Value"),
#'   long_desc = list(WELF = "Welfare Decomposition"),
#'   coefficients = list(WELF = "WELF"),
#'   export_sets = TRUE,
#'   dim_order = dim_order,
#'   lowercase = FALSE
#' )
save_har <- function(data_list,
                     file_path,
                     dimensions = NULL,
                     value_cols = NULL,
                     header_type = NULL,
                     mappings = NULL,
                     long_desc = NULL,
                     coefficients = NULL,
                     export_sets = TRUE,
                     lowercase = TRUE,
                     dim_order = NULL,
                     dim_rename = NULL,
                     force_sparse = NULL,
                     max_chunk = 2e6) {
  if (!is.list(data_list) || is.null(names(data_list)) || any(names(data_list) == "")) {
    stop("data_list must be a named list")
  }
  original_headers <- names(data_list)
  header_names <- normalize_header_names(original_headers)
  if (any(duplicated(header_names))) {
    dup <- unique(header_names[duplicated(header_names)])
    stop("Duplicated HAR header names after four-character truncation: ", paste(dup, collapse = ", "))
  }
  names(data_list) <- header_names
  dimensions <- normalize_named_spec(dimensions)
  value_cols <- normalize_named_spec(value_cols)
  header_type <- normalize_named_spec(header_type)
  mappings <- normalize_named_spec(mappings)
  long_desc <- normalize_named_spec(long_desc)
  coefficients <- normalize_named_spec(coefficients)
  dim_rename <- normalize_named_spec(dim_rename)
  force_sparse <- normalize_header_names(force_sparse)
  validate_mapping_specs(mappings, data_list)
  header_types <- vapply(header_names, function(hdr) {
    detect_header_type(hdr, data_list[[hdr]], header_type, mappings)
  }, character(1), USE.NAMES = TRUE)
  char_headers <- header_names[header_types %in% c("set", "mapping")]
  int_headers <- header_names[header_types == "integer"]
  re_headers <- header_names[header_types %in% c("real", "sparse")]
  unknown_headers <- header_names[header_types == "unknown"]
  if (length(unknown_headers) > 0) {
    warning(sprintf("Skipping unsupported data types: %s", paste(unknown_headers, collapse = ", ")))
  }
  for (hdr in re_headers) {
    if (is.data.frame(data_list[[hdr]]) && is.null(value_cols[[hdr]])) value_cols[[hdr]] <- "Value"
  }
  dim_order_map <- process_dim_order(dim_order, lowercase)
  all_arrays <- list()
  for (hdr in re_headers) {
    obj <- data_list[[hdr]]
    if (is.data.frame(obj)) {
      if (is.null(dimensions[[hdr]])) stop(sprintf("Header '%s' is a data frame but dimensions[['%s']] is missing", hdr, hdr))
      all_arrays[[hdr]] <- convert_df_to_array(obj, dimensions[[hdr]], value_cols[[hdr]], lowercase)
    } else {
      all_arrays[[hdr]] <- obj
    }
  }
  if (!is.null(dim_rename)) {
    for (hdr in names(dim_rename)) {
      if (hdr %in% names(all_arrays)) all_arrays[[hdr]] <- rename_array_dims(all_arrays[[hdr]], dim_rename[[hdr]])
    }
  }
  unique_sets <- if (export_sets && length(all_arrays) > 0) extract_unique_sets(all_arrays) else NULL
  explicit_sets <- char_headers[header_types[char_headers] == "set"]
  if (!is.null(unique_sets)) unique_sets <- unique_sets[!(names(unique_sets) %in% explicit_sets)]
  if (!is.null(unique_sets) && !is.null(dim_order_map)) {
    for (set_name in names(unique_sets)) {
      if (set_name %in% names(dim_order_map)) {
        current_vals <- unique_sets[[set_name]]
        desired_order <- dim_order_map[[set_name]]
        common_vals <- intersect(desired_order, current_vals)
        extra_vals <- setdiff(current_vals, desired_order)
        unique_sets[[set_name]] <- if (length(common_vals) > 0) c(common_vals, sort(extra_vals)) else sort(current_vals)
      } else {
        unique_sets[[set_name]] <- sort(unique_sets[[set_name]])
      }
    }
  }
  if (!is.null(unique_sets)) all_arrays <- lapply(all_arrays, reorder_array_by_sets, set_list = unique_sets)
  out_dir <- dirname(file_path)
  if (!dir.exists(out_dir)) dir.create(out_dir, recursive = TRUE, showWarnings = FALSE)
  con <- file(file_path, "wb")
  on.exit(close(con), add = TRUE)
  n_sets <- 0
  n_mappings <- 0
  n_data <- 0
  written_headers <- character(0)
  for (hdr in char_headers) {
    obj <- data_list[[hdr]]
    if (header_types[[hdr]] == "set") {
      if (!(hdr %in% written_headers)) {
        write_string(con, hdr, obj, description = get_long_desc(hdr, long_desc, paste0("Set ", hdr)))
        n_sets <- n_sets + 1
        written_headers <- c(written_headers, hdr)
      }
    } else {
      map_result <- prepare_mapping_values(hdr, obj, mappings[[hdr]], data_list)
      from_set <- map_result$from_set
      to_set <- map_result$to_set
      if (!(from_set %in% written_headers)) {
        write_string(con, from_set, map_result$from_elements, description = get_long_desc(from_set, long_desc, paste0("Set ", from_set)))
        n_sets <- n_sets + 1
        written_headers <- c(written_headers, from_set)
      }
      if (!(to_set %in% written_headers)) {
        write_string(con, to_set, map_result$to_elements, description = get_long_desc(to_set, long_desc, paste0("Set ", to_set)))
        n_sets <- n_sets + 1
        written_headers <- c(written_headers, to_set)
      }
      default_desc <- sprintf("Mapping %s2%s from %s(%d) to %s(%d)", from_set, to_set, from_set, length(map_result$from_elements), to_set, length(map_result$to_elements))
      write_string(con, hdr, map_result$values, description = get_long_desc(hdr, long_desc, default_desc))
      n_mappings <- n_mappings + 1
      written_headers <- c(written_headers, hdr)
    }
  }
  if (!is.null(unique_sets)) {
    for (nm in names(unique_sets)) {
      if (!(nm %in% written_headers)) {
        write_string(con, nm, unique_sets[[nm]], description = get_long_desc(nm, long_desc, paste0("Set ", nm)))
        n_sets <- n_sets + 1
        written_headers <- c(written_headers, nm)
      }
    }
  }
  for (hdr in int_headers) {
    write_2ifull(con, hdr, data_list[[hdr]], description = get_long_desc(hdr, long_desc, hdr))
    n_data <- n_data + 1
    written_headers <- c(written_headers, hdr)
  }
  for (hdr in re_headers) {
    arr <- all_arrays[[hdr]]
    desc <- get_long_desc(hdr, long_desc, hdr)
    coef <- get_long_desc(hdr, coefficients, hdr)
    use_sparse <- header_types[[hdr]] == "sparse" || (!is.null(force_sparse) && hdr %in% force_sparse)
    if (!use_sparse && is.numeric(arr) && length(arr) > 0 && sum(arr == 0) / length(arr) > 0.5) {
      use_sparse <- TRUE
      message(sprintf("Doing sparse: %s", hdr))
    }
    if (use_sparse) write_sparse(con, hdr, arr, description = desc, coefficient = coef, maxSize = max_chunk)
    else write_matrix(con, hdr, arr, description = desc, coefficient = coef, maxSize = max_chunk)
    n_data <- n_data + 1
    written_headers <- c(written_headers, hdr)
  }
  close(con)
  on.exit(NULL, add = FALSE)
  n_total <- n_sets + n_data + n_mappings
  cat(sprintf("\nSuccessfully wrote %d header(s) to HAR file\n", n_total))
  cat(sprintf("  Set headers (1C type): %d\n", n_sets))
  cat(sprintf("  Mapping headers (1C type): %d\n", n_mappings))
  cat(sprintf("  Data headers (RE/2I type): %d\n", n_data))
  if (!is.null(dim_order_map) && length(dim_order_map) > 0) {
    cat("\nDimension ordering applied:\n")
    for (dm in names(dim_order_map)) cat(sprintf("  %s: %d prioritized values, remaining A-Z\n", dm, length(dim_order_map[[dm]])))
  } else if (length(all_arrays) > 0) {
    cat("\nAll dimensions sorted A-Z (no custom mapping provided)\n")
  }
  cat(sprintf("\nOutput file: %s\n", normalizePath(file_path)))
  cat(sprintf("File size: %s bytes\n\n", format(file.info(file_path)$size, big.mark = ",")))
  invisible(list(file_path = normalizePath(file_path), headers_written = written_headers, n_sets = n_sets, n_data = n_data, n_mappings = n_mappings))
}

#' @keywords internal
#' @noRd
normalize_header_names <- function(x) {
  if (is.null(x)) return(NULL)
  toupper(substr(as.character(x), 1, 4))
}

#' @keywords internal
#' @noRd
normalize_named_spec <- function(x) {
  if (is.null(x)) return(NULL)
  if (is.atomic(x) && !is.null(names(x))) x <- as.list(x)
  if (!is.list(x)) stop("Named specification arguments must be named lists or named vectors")
  if (is.null(names(x)) || any(names(x) == "")) stop("Named specification arguments must have names")
  names(x) <- normalize_header_names(names(x))
  x
}

#' @keywords internal
#' @noRd
normalize_header_type_value <- function(x) {
  if (is.null(x)) return(NULL)
  z <- tolower(as.character(x)[1])
  if (z %in% c("auto", "")) return(NULL)
  if (z %in% c("set", "str", "string", "1c", "1cfull")) return("set")
  if (z %in% c("mapping", "map")) return("mapping")
  if (z %in% c("real", "re", "refull", "numeric", "dense")) return("real")
  if (z %in% c("sparse", "respse")) return("sparse")
  if (z %in% c("integer", "int", "2i", "2ifull")) return("integer")
  stop("Unknown header_type value: ", as.character(x)[1])
}

#' @keywords internal
#' @noRd
detect_header_type <- function(hdr, obj, header_type, mappings) {
  if (!is.null(mappings) && hdr %in% names(mappings)) return("mapping")
  if (!is.null(header_type) && hdr %in% names(header_type)) {
    explicit <- normalize_header_type_value(header_type[[hdr]])
    if (!is.null(explicit)) return(explicit)
  }
  if (is.character(obj)) return("set")
  if (is.matrix(obj) && is.integer(obj)) return("integer")
  if (is.data.frame(obj)) return("real")
  if (is.numeric(obj) || is.matrix(obj) || is.array(obj)) return("real")
  "unknown"
}

#' @keywords internal
#' @noRd
validate_mapping_specs <- function(mappings, data_list) {
  if (is.null(mappings)) return(invisible(TRUE))
  for (hdr in names(mappings)) {
    spec <- normalize_header_names(mappings[[hdr]])
    if (length(spec) != 2) stop(sprintf("mappings[['%s']] must be c(source_set, destination_set)", hdr))
    if (!(hdr %in% names(data_list))) stop(sprintf("Mapping header '%s' is not in data_list", hdr))
    if (!(spec[1] %in% names(data_list)) || !is.character(data_list[[spec[1]]])) stop(sprintf("Source set '%s' for mapping '%s' must be a character vector in data_list", spec[1], hdr))
    if (!(spec[2] %in% names(data_list)) || !is.character(data_list[[spec[2]]])) stop(sprintf("Destination set '%s' for mapping '%s' must be a character vector in data_list", spec[2], hdr))
    mappings[[hdr]] <- spec
  }
  invisible(TRUE)
}

#' @keywords internal
#' @noRd
get_long_desc <- function(hdr, desc_list, default) {
  if (!is.null(desc_list) && hdr %in% names(desc_list) && !is.null(desc_list[[hdr]])) return(as.character(desc_list[[hdr]])[1])
  default
}

#' @keywords internal
#' @noRd
prepare_mapping_values <- function(hdr, obj, spec, data_list) {
  spec <- normalize_header_names(spec)
  from_set <- spec[1]
  to_set <- spec[2]
  from_elements <- as.character(unname(data_list[[from_set]]))
  to_elements <- as.character(unname(data_list[[to_set]]))
  obj <- as.character(obj)
  if (!is.null(names(obj)) && any(names(obj) != "")) {
    obj_names <- as.character(names(obj))
    values <- setNames(rep("0", length(from_elements)), from_elements)
    common <- intersect(from_elements, obj_names)
    if (length(common) > 0) values[common] <- unname(obj[match(common, obj_names)])
    missing_source <- setdiff(from_elements, obj_names)
    extra_source <- setdiff(obj_names[obj_names != ""], from_elements)
    if (length(missing_source) > 0) warning(sprintf("Header '%s' has missing source elements written as '0': %s", hdr, paste(missing_source, collapse = ", ")))
    if (length(extra_source) > 0) warning(sprintf("Header '%s' has source elements not in '%s' and they are ignored: %s", hdr, from_set, paste(extra_source, collapse = ", ")))
    map_values <- unname(values)
  } else {
    if (length(obj) != length(from_elements)) stop(sprintf("Header '%s' has %d mapping values but source set '%s' has %d elements", hdr, length(obj), from_set, length(from_elements)))
    map_values <- obj
  }
  map_values[is.na(map_values) | trimws(map_values) == ""] <- "0"
  bad <- !(map_values %in% c(to_elements, "0"))
  if (any(bad)) {
    warning(sprintf("Header '%s' has invalid mapping targets written as '0': %s", hdr, paste(unique(map_values[bad]), collapse = ", ")))
    map_values[bad] <- "0"
  }
  list(from_set = from_set, to_set = to_set, from_elements = from_elements, to_elements = to_elements, values = map_values)
}

#' @keywords internal
#' @noRd
write_string <- function(con, hdr_name, elements, description = NULL) {
  hdr_name <- substr(paste0(hdr_name, "    "), 1, 4)
  elements <- as.character(elements)
  if (length(elements) == 0) stop(sprintf("Header '%s' has no character elements", trimws(hdr_name)))
  desc <- substr(paste0(if (is.null(description)) hdr_name else description, strrep(" ", 70)), 1, 70)
  max_len <- max(12, max(nchar(elements)))
  dimensions <- c(length(elements), max_len)
  rec1 <- charToRaw(hdr_name)
  writeBin(as.integer(4), con, size = 4)
  writeBin(rec1, con)
  writeBin(as.integer(4), con, size = 4)
  rec2 <- c(charToRaw("    "), charToRaw("1CFULL"), charToRaw(desc), writeBin(as.integer(2), raw(), size = 4), writeBin(as.integer(dimensions), raw(), size = 4))
  writeBin(as.integer(length(rec2)), con, size = 4)
  writeBin(rec2, con)
  writeBin(as.integer(length(rec2)), con, size = 4)
  padded_elements <- vapply(elements, function(e) {
    e <- substr(e, 1, max_len)
    paste0(e, strrep(" ", max_len - nchar(e)))
  }, character(1), USE.NAMES = FALSE)
  contents <- charToRaw(paste0(padded_elements, collapse = ""))
  rec3 <- c(charToRaw("    "), writeBin(as.integer(1), raw(), size = 4), writeBin(as.integer(length(elements)), raw(), size = 4), writeBin(as.integer(length(elements)), raw(), size = 4), contents)
  writeBin(as.integer(length(rec3)), con, size = 4)
  writeBin(rec3, con)
  writeBin(as.integer(length(rec3)), con, size = 4)
}

#' @keywords internal
#' @noRd
write_2ifull <- function(con, hdr_name, arr, description = NULL) {
  hdr_name <- substr(paste0(hdr_name, "    "), 1, 4)
  if (is.null(description)) description <- hdr_name
  desc <- substr(paste0(description, strrep(" ", 70)), 1, 70)
  dims <- dim(arr)
  if (is.null(dims) || length(dims) != 2) stop("2IFULL requires a two-dimensional integer matrix")
  rec1 <- charToRaw(hdr_name)
  writeBin(as.integer(4), con, size = 4)
  writeBin(rec1, con)
  writeBin(as.integer(4), con, size = 4)
  rec2 <- c(charToRaw("    "), charToRaw("2IFULL"), charToRaw(desc), writeBin(as.integer(2), raw(), size = 4), writeBin(as.integer(dims), raw(), size = 4))
  writeBin(as.integer(length(rec2)), con, size = 4)
  writeBin(rec2, con)
  writeBin(as.integer(length(rec2)), con, size = 4)
  rec3 <- c(charToRaw("    "), writeBin(1L, raw(), size = 4), writeBin(as.integer(dims[1]), raw(), size = 4), writeBin(as.integer(dims[2]), raw(), size = 4), writeBin(1L, raw(), size = 4), writeBin(as.integer(dims[1]), raw(), size = 4), writeBin(1L, raw(), size = 4), writeBin(as.integer(dims[2]), raw(), size = 4), writeBin(as.integer(arr), raw(), size = 4))
  writeBin(as.integer(length(rec3)), con, size = 4)
  writeBin(rec3, con)
  writeBin(as.integer(length(rec3)), con, size = 4)
}

#' @keywords internal
#' @noRd
write_matrix <- function(con, hdr_name, arr, description = NULL, coefficient = NULL, maxSize = 2e6) {
  hdr_name <- substr(paste0(hdr_name, "    "), 1, 4)
  if (is.null(description)) description <- hdr_name
  description <- substr(paste0(description, strrep(" ", 70)), 1, 70)
  if (is.null(coefficient)) coefficient <- trimws(hdr_name)
  coefficient <- substr(paste0(coefficient, strrep(" ", 12)), 1, 12)
  dimensions <- dim(arr)
  if (is.null(dimensions)) dimensions <- c(length(arr))
  used_dimensions <- length(dimensions)
  dimensions <- c(dimensions, rep(1, 7 - length(dimensions)))
  message(sprintf("%s with maxsize %s", trimws(hdr_name), maxSize))
  rec1 <- charToRaw(hdr_name)
  writeBin(as.integer(4), con, size = 4)
  writeBin(rec1, con)
  writeBin(as.integer(4), con, size = 4)
  rec2 <- c(charToRaw("    "), charToRaw("REFULL"), charToRaw(description), writeBin(as.integer(7), raw(), size = 4), writeBin(as.integer(dimensions), raw(), size = 4))
  writeBin(as.integer(length(rec2)), con, size = 4)
  writeBin(rec2, con)
  writeBin(as.integer(length(rec2)), con, size = 4)
  if (!is.null(dimnames(arr)) && length(names(dimnames(arr))) > 0) {
    dim_names <- names(dimnames(arr))
    set_names_raw <- unlist(lapply(dim_names, function(nm) charToRaw(substr(paste0(nm, strrep(" ", 12)), 1, 12))))
    set_names <- c(set_names_raw, as.raw(rep(0x6b, length(dim_names))), as.raw(rep(0x00, 4 + 4 * length(dim_names))))
    defined_dims <- length(unique(dim_names))
    used_dims <- length(dim_names)
  } else {
    set_names <- as.raw(c(0x00, 0x00, 0x00, 0x00))
    defined_dims <- 0
    used_dims <- 0
  }
  rec3 <- c(charToRaw("    "), writeBin(as.integer(defined_dims), raw(), size = 4), as.raw(c(0xff, 0xff, 0xff, 0xff)), writeBin(as.integer(used_dims), raw(), size = 4), charToRaw(coefficient), as.raw(c(0xff, 0xff, 0xff, 0xff)), set_names)
  writeBin(as.integer(length(rec3)), con, size = 4)
  writeBin(rec3, con)
  writeBin(as.integer(length(rec3)), con, size = 4)
  if (!is.null(dimnames(arr)) && length(dimnames(arr)) > 0) {
    for (ud in unique(names(dimnames(arr)))) {
      ele <- dimnames(arr)[[which(names(dimnames(arr)) == ud)[1]]]
      padded_ele <- vapply(ele, function(e) {
        e <- substr(e, 1, 12)
        paste0(e, strrep(" ", 12 - nchar(e)))
      }, character(1), USE.NAMES = FALSE)
      element_data <- unlist(lapply(padded_ele, charToRaw))
      rec_ele <- c(charToRaw("    "), writeBin(as.integer(1), raw(), size = 4), writeBin(as.integer(length(ele)), raw(), size = 4), writeBin(as.integer(length(ele)), raw(), size = 4), element_data)
      writeBin(as.integer(length(rec_ele)), con, size = 4)
      writeBin(rec_ele, con)
      writeBin(as.integer(length(rec_ele)), con, size = 4)
    }
  }
  total_elements <- prod(dim(arr)[1:used_dimensions])
  if (total_elements <= maxSize) {
    slice_size <- total_elements
    num_data_records <- 1
  } else {
    arr2 <- arr
    arr2[] <- FALSE
    arr2[maxSize] <- TRUE
    breaks <- which(arr2 == TRUE, arr.ind = TRUE)
    cut_dim <- 1
    for (cc in length(dim(arr)):1) if (breaks[cc] > 1) { cut_dim <- cc; break }
    cutPoint <- rep(1, length(dim(arr)))
    cutPoint[cut_dim] <- 2
    arr2[] <- FALSE
    arr2[matrix(cutPoint, nrow = 1)] <- TRUE
    slice_size <- which(arr2 == TRUE) - 1
    if (length(slice_size) == 0 || slice_size < 1) slice_size <- total_elements
    num_data_records <- ceiling(total_elements / slice_size)
  }
  rec_frame <- c(charToRaw("    "), writeBin(as.integer(1 + num_data_records * 2), raw(), size = 4), writeBin(as.integer(7), raw(), size = 4), writeBin(as.integer(dimensions), raw(), size = 4))
  writeBin(as.integer(length(rec_frame)), con, size = 4)
  writeBin(rec_frame, con)
  writeBin(as.integer(length(rec_frame)), con, size = 4)
  arr_vector <- as.numeric(as.vector(arr))
  for (dr in seq_len(num_data_records)) {
    from_element <- (dr - 1) * slice_size + 1
    to_element <- min(dr * slice_size, total_elements)
    arr2 <- arr
    arr2[] <- FALSE
    arr2[from_element] <- TRUE
    arr2[to_element] <- TRUE
    from_to_indices <- as.matrix(which(arr2 == TRUE, arr.ind = TRUE))
    if (nrow(from_to_indices) == 1) from_to_indices <- rbind(from_to_indices, from_to_indices)
    from_to_vector <- c(as.vector(from_to_indices), rep(c(1, 1), 7 - length(dim(arr))))
    rec_bounds <- c(charToRaw("    "), writeBin(as.integer(num_data_records * 2 - dr * 2 + 2), raw(), size = 4), writeBin(as.integer(from_to_vector), raw(), size = 4))
    writeBin(as.integer(length(rec_bounds)), con, size = 4)
    writeBin(rec_bounds, con)
    writeBin(as.integer(length(rec_bounds)), con, size = 4)
    data_chunk <- arr_vector[from_element:to_element]
    rec_data <- c(charToRaw("    "), writeBin(as.integer(num_data_records * 2 - dr * 2 + 1), raw(), size = 4), writeBin(as.numeric(data_chunk), raw(), size = 4))
    writeBin(as.integer(length(rec_data)), con, size = 4)
    writeBin(rec_data, con)
    writeBin(as.integer(length(rec_data)), con, size = 4)
  }
}

#' @keywords internal
#' @noRd
write_sparse <- function(con, hdr_name, arr, description = NULL, coefficient = NULL, maxSize = 2e6) {
  hdr_name <- substr(paste0(hdr_name, "    "), 1, 4)
  if (is.null(description)) description <- hdr_name
  description <- substr(paste0(description, strrep(" ", 70)), 1, 70)
  if (is.null(coefficient)) coefficient <- trimws(hdr_name)
  coefficient <- substr(paste0(coefficient, strrep(" ", 12)), 1, 12)
  dimensions <- dim(arr)
  if (is.null(dimensions)) dimensions <- c(length(arr))
  used_dimensions <- length(dimensions)
  dimensions <- c(dimensions, rep(1, 7 - length(dimensions)))
  message(sprintf("%s with maxsize %s", trimws(hdr_name), maxSize))
  rec1 <- charToRaw(hdr_name)
  writeBin(as.integer(4), con, size = 4)
  writeBin(rec1, con)
  writeBin(as.integer(4), con, size = 4)
  rec2 <- c(charToRaw("    "), charToRaw("RESPSE"), charToRaw(description), writeBin(as.integer(7), raw(), size = 4), writeBin(as.integer(dimensions), raw(), size = 4))
  writeBin(as.integer(length(rec2)), con, size = 4)
  writeBin(rec2, con)
  writeBin(as.integer(length(rec2)), con, size = 4)
  if (!is.null(dimnames(arr)) && length(names(dimnames(arr))) > 0) {
    dim_names <- names(dimnames(arr))
    set_names_raw <- unlist(lapply(dim_names, function(nm) charToRaw(substr(paste0(nm, strrep(" ", 12)), 1, 12))))
    set_names <- c(set_names_raw, as.raw(rep(0x6b, length(dim_names))), as.raw(rep(0x00, 4 + 4 * length(dim_names))))
    defined_dims <- length(unique(dim_names))
    used_dims <- length(dim_names)
  } else {
    set_names <- as.raw(c(0x00, 0x00, 0x00, 0x00))
    defined_dims <- 0
    used_dims <- 0
  }
  rec3 <- c(charToRaw("    "), writeBin(as.integer(defined_dims), raw(), size = 4), as.raw(c(0xff, 0xff, 0xff, 0xff)), writeBin(as.integer(used_dims), raw(), size = 4), charToRaw(coefficient), as.raw(c(0xff, 0xff, 0xff, 0xff)), set_names)
  writeBin(as.integer(length(rec3)), con, size = 4)
  writeBin(rec3, con)
  writeBin(as.integer(length(rec3)), con, size = 4)
  if (!is.null(dimnames(arr)) && length(dimnames(arr)) > 0) {
    for (ud in unique(names(dimnames(arr)))) {
      ele <- dimnames(arr)[[which(names(dimnames(arr)) == ud)[1]]]
      padded_ele <- vapply(ele, function(e) {
        e <- substr(e, 1, 12)
        paste0(e, strrep(" ", 12 - nchar(e)))
      }, character(1), USE.NAMES = FALSE)
      element_data <- unlist(lapply(padded_ele, charToRaw))
      rec_ele <- c(charToRaw("    "), writeBin(as.integer(1), raw(), size = 4), writeBin(as.integer(length(ele)), raw(), size = 4), writeBin(as.integer(length(ele)), raw(), size = 4), element_data)
      writeBin(as.integer(length(rec_ele)), con, size = 4)
      writeBin(rec_ele, con)
      writeBin(as.integer(length(rec_ele)), con, size = 4)
    }
  }
  val <- as.vector(arr)
  pos <- seq_along(val)
  nonzeros <- val != 0
  nzval <- val[nonzeros]
  nzpos <- pos[nonzeros]
  sparse_frame <- c(charToRaw("    "), writeBin(as.integer(length(nzval)), raw(), size = 4), writeBin(4L, raw(), size = 4), writeBin(4L, raw(), size = 4), charToRaw(substr(strrep(" ", 80), 1, 80)))
  writeBin(as.integer(length(sparse_frame)), con, size = 4)
  writeBin(sparse_frame, con)
  writeBin(as.integer(length(sparse_frame)), con, size = 4)
  if (length(nzval) == 0) {
    rec_sparse <- c(charToRaw("    "), writeBin(as.integer(1), raw(), size = 4), writeBin(as.integer(0), raw(), size = 4), writeBin(as.integer(0), raw(), size = 4))
    writeBin(as.integer(length(rec_sparse)), con, size = 4)
    writeBin(rec_sparse, con)
    writeBin(as.integer(length(rec_sparse)), con, size = 4)
    return(invisible(NULL))
  }
  numberDataRecords <- if (length(nzval) <= maxSize / 2) 1 else ceiling(2 * length(nzval) / maxSize)
  chunk_size <- ceiling(length(nzval) / numberDataRecords)
  for (dr in seq_len(numberDataRecords)) {
    fromElement <- (dr - 1) * chunk_size + 1
    toElement <- min(dr * chunk_size, length(nzval))
    positions <- nzpos[fromElement:toElement]
    values <- nzval[fromElement:toElement]
    rec_sparse <- c(charToRaw("    "), writeBin(as.integer(numberDataRecords + 1 - dr), raw(), size = 4), writeBin(as.integer(length(nzval)), raw(), size = 4), writeBin(as.integer(length(values)), raw(), size = 4), writeBin(as.integer(positions), raw(), size = 4), writeBin(as.double(values), raw(), size = 4))
    writeBin(as.integer(length(rec_sparse)), con, size = 4)
    writeBin(rec_sparse, con)
    writeBin(as.integer(length(rec_sparse)), con, size = 4)
  }
}

#' @keywords internal
#' @importFrom stats complete.cases
#' @noRd
convert_df_to_array <- function(df, dim_cols, val_col, lowercase) {
  dim_cols <- as.character(dim_cols)
  val_col <- as.character(val_col)[1]
  needed <- c(dim_cols, val_col)
  missing_cols <- setdiff(needed, names(df))
  if (length(missing_cols) > 0) stop("Missing column(s): ", paste(missing_cols, collapse = ", "))
  df <- df[complete.cases(df[, needed]), , drop = FALSE]
  dim_sets <- lapply(dim_cols, function(col) {
    vals <- sort(unique(as.character(df[[col]])))
    if (lowercase) tolower(vals) else vals
  })
  names(dim_sets) <- if (lowercase) tolower(dim_cols) else dim_cols
  arr <- array(0, dim = vapply(dim_sets, length, integer(1)), dimnames = dim_sets)
  for (i in seq_len(nrow(df))) {
    idx <- vapply(seq_along(dim_cols), function(j) {
      val <- as.character(df[[dim_cols[j]]][i])
      if (lowercase) val <- tolower(val)
      match(val, dim_sets[[j]])
    }, integer(1))
    arr[matrix(idx, nrow = 1)] <- as.numeric(df[[val_col]][i])
  }
  arr
}

#' @keywords internal
#' @noRd
rename_array_dims <- function(arr, rename_map) {
  if (is.null(dimnames(arr)) || is.null(names(dimnames(arr)))) return(arr)
  old_names <- names(dimnames(arr))
  new_names <- old_names
  for (i in seq_along(old_names)) if (old_names[i] %in% names(rename_map)) new_names[i] <- rename_map[[old_names[i]]]
  names(dimnames(arr)) <- new_names
  arr
}

#' @keywords internal
#' @noRd
reorder_array_by_sets <- function(arr, set_list) {
  if (is.null(dimnames(arr)) || is.null(set_list)) return(arr)
  dim_names <- names(dimnames(arr))
  for (i in seq_along(dim_names)) {
    dim_key <- toupper(substr(dim_names[i], 1, 4))
    if (dim_key %in% names(set_list)) {
      current_vals <- dimnames(arr)[[i]]
      desired_order <- set_list[[dim_key]]
      common_vals <- intersect(desired_order, current_vals)
      if (length(common_vals) > 0 && !identical(current_vals, common_vals)) {
        idx_list <- vector("list", length(dim(arr)))
        for (j in seq_along(idx_list)) idx_list[[j]] <- if (j == i) match(common_vals, current_vals) else seq_len(dim(arr)[j])
        arr <- do.call(`[`, c(list(arr), idx_list, drop = FALSE))
      }
    }
  }
  arr
}

#' @keywords internal
#' @noRd
extract_unique_sets <- function(arrays) {
  all_sets <- list()
  for (arr in arrays) {
    dn <- names(dimnames(arr))
    for (i in seq_along(dn)) {
      hdr <- toupper(substr(dn[i], 1, 4))
      if (hdr %in% names(all_sets)) all_sets[[hdr]] <- unique(c(all_sets[[hdr]], dimnames(arr)[[i]]))
      else all_sets[[hdr]] <- dimnames(arr)[[i]]
    }
  }
  lapply(all_sets, sort)
}

#' @keywords internal
#' @importFrom utils read.csv
#' @noRd
process_dim_order <- function(dim_order, lowercase) {
  if (is.null(dim_order)) return(NULL)
  if (is.character(dim_order) && length(dim_order) == 1) {
    if (grepl("\\.xlsx?$", dim_order, ignore.case = TRUE)) {
      if (!requireNamespace("openxlsx", quietly = TRUE)) stop("Package 'openxlsx' needed for Excel files. Install with: install.packages('openxlsx')")
      dim_order <- openxlsx::read.xlsx(dim_order, sheet = 1)
    } else if (grepl("\\.csv$", dim_order, ignore.case = TRUE)) {
      dim_order <- read.csv(dim_order, stringsAsFactors = FALSE)
    } else {
      stop("File must be .xlsx, .xls, or .csv")
    }
  }
  if (is.data.frame(dim_order)) {
    order_map <- lapply(names(dim_order), function(col) {
      vals <- dim_order[[col]]
      vals <- vals[!is.na(vals) & nchar(trimws(as.character(vals))) > 0]
      vals <- unique(as.character(vals))
      if (lowercase) tolower(vals) else vals
    })
    names(order_map) <- toupper(substr(names(dim_order), 1, 4))
    return(order_map[sapply(order_map, length) > 0])
  }
  if (is.list(dim_order)) {
    order_map <- lapply(names(dim_order), function(nm) {
      vals <- dim_order[[nm]]
      vals <- vals[!is.na(vals) & nchar(trimws(as.character(vals))) > 0]
      vals <- unique(as.character(vals))
      if (lowercase) tolower(vals) else vals
    })
    names(order_map) <- toupper(substr(names(dim_order), 1, 4))
    return(order_map[sapply(order_map, length) > 0])
  }
  stop("dim_order must be NULL, data frame, list, or file path")
}

#' @title Load and Process HAR Files with Header Selection
#'
#' @description Reads a GEMPACK HAR file and extracts structured data while maintaining compatibility 
#' with standard HAR formats. Provides flexibility in naming conventions and header selection.
#'
#' @details
#' - Uses `load_harplus()` internally for efficient HAR file reading.
#' - Allows optional conversion of variable names to lowercase (`lowercase = TRUE`).
#' - Supports coefficient-based naming (`coefAsname = TRUE`).
#' - Enables selective header extraction via `select_header = c("A", "E1")`.
#' - Returns structured data with explicit dimension names and sizes.
#'
#' @param file_path Character. The file path to the HAR file.
#' @param coefAsname Logical. If `TRUE`, replaces four-letter headers with coefficient names when available. Default is `FALSE`.
#' @param lowercase Logical. If `TRUE`, converts all variable names to lowercase. Default is `FALSE`.
#' @param select_header Character vector. Specific headers to read; if `NULL`, all headers are read. Example: `select_header = c("A", "E1")`.
#'
#' @return A structured list containing:
#' - `data`: Extracted HAR variable data stored as matrices, arrays, or vectors.
#' - `dimension_info`: A list with:
#'     - `dimension_string`: A textual representation of dimensions (e.g., "REG*COMM*YEAR").
#'     - `dimension_names`: The names of each dimension.
#'     - `dimension_sizes`: The size of each dimension.
#'
#' @author Pattawee Puangchit
#'   
#' @seealso \code{\link{load_sl4x}}, \code{\link{get_data_by_var}}, \code{\link{get_data_by_dims}}
#'
#' @export
#'
#' @examples
#' # Path to example files
#' har_path <- system.file("extdata", "TAR10-WEL.har", package = "HARplus")
#' 
#' # Basic loading
#' har_data <- load_harx(har_path)
#' 
#' # Load with coefficient names
#' har_data_coef <- load_harx(har_path, coefAsname = TRUE)
#' 
#' # Load with lowercase names
#' har_data_lower <- load_harx(har_path, lowercase = TRUE)
#' 
#' # Load specific headers
#' har_selected <- load_harx(har_path, select_header = c("A", "E1"))
#' 
#' # Load with multiple options
#' har_combined <- load_harx(har_path,
#'                          coefAsname = TRUE,
#'                          lowercase = TRUE,
#'                          select_header = c("A", "E1"))
#'
load_harx <- function(file_path, coefAsname = FALSE, lowercase = FALSE, select_header = NULL) {
  har_data <- load_harplus(file_path, 
                           coefAsname = coefAsname,
                           lowercase = lowercase,
                           select_header = select_header)
  
  dimension_info <- list()
  
  for (header_name in names(har_data)) {
    header_obj <- har_data[[header_name]]
    
    if (!is.null(dim(header_obj))) {
      dims <- dim(header_obj)
      dim_names <- dimnames(header_obj)
      
      if (!is.null(dim_names)) {
        dim_string <- paste(names(dim_names), collapse="*")
        dimension_info[[header_name]] <- list(
          dimension_string = dim_string,
          dimension_names = names(dim_names),
          dimension_sizes = dims,
          dimension_elements = dim_names
        )
      } else {
        generic_names <- paste0("DIM", seq_along(dims))
        dim_string <- paste(generic_names, collapse="*")
        dimension_info[[header_name]] <- list(
          dimension_string = dim_string,
          dimension_names = generic_names,
          dimension_sizes = dims,
          dimension_elements = lapply(dims, function(x) paste0("Element", 1:x))
        )
      }
    } else {
      dimension_info[[header_name]] <- list(
        dimension_string = "scalar",
        dimension_names = character(0),
        dimension_sizes = integer(0),
        dimension_elements = list()
      )
    }
  }
  
  enhanced_results <- list(
    data = har_data,
    dimension_info = dimension_info
  )
  
  class(enhanced_results) <- "enhanced_har"
  
  return(enhanced_results)
}

#' @title Get Variable Structure Summary from SL4 and HAR Objects
#'
#' @description Generates a summary of the variables within one or more SL4 or HAR objects, listing their
#' dimension sizes, structures, and optionally, column and observation counts.
#'
#' @details
#' - Extracts dimension structures for variables in one or more SL4 or HAR datasets.
#' - If `include_col_size = TRUE`, adds column and observation counts.
#' - Supports multiple datasets and returns results as a named list, with each dataset’s summary stored separately.
#' - Can summarize specific variables or `"ALL"`.
#'
#' @param variables Character vector. Variable names to summarize. Use `NULL` or `"ALL"` to summarize all variables.
#' @param ... One or more SL4 or HAR objects created using `load_sl4x()` or `load_harx()`.
#' @param include_col_size Logical. If `TRUE`, includes column and observation counts. Default is `FALSE`.
#'
#' @return A named list, where each element contains a data frame with:
#' - `Variable`: The variable name.
#' - `Dimensions`: The associated dimensions.
#' - `DimSize`: The number of dimensions.
#' - `DataShape`: The shape of the data (e.g., `10x20x30`).
#' - `No.Col`: (Optional) The number of columns.
#' - `No.Obs`: (Optional) The number of observations.
#'
#' @importFrom stats setNames
#'
#' @author Pattawee Puangchit
#'
#' @seealso \code{\link{get_dim_patterns}}, \code{\link{get_dim_elements}}
#'
#' @export
#'
#' @examples
#' # Import data sample:
#' sl4_data <- load_sl4x(system.file("extdata", "TAR10.sl4", package = "HARplus"))
#' sl4_data1 <- load_sl4x(system.file("extdata", "SUBT10.sl4", package = "HARplus"))
#'
#' # Get summary for all variables in a single dataset
#' get_var_structure(data_obj = sl4_data)
#'
#' # Get summary for specific variables
#' get_var_structure(c("gdp", "trade"), sl4_data)
#'
#' # Include column and observation counts
#' get_var_structure("ALL", sl4_data, include_col_size = TRUE)
#'
#' # Compare structures across multiple datasets
#' get_var_structure("ALL", sl4_data, sl4_data1)
#'
#' # Include column and observation counts across multiple datasets
#' get_var_structure("ALL", sl4_data, sl4_data1, include_col_size = TRUE)
#'
get_var_structure <- function(variables = NULL, ..., include_col_size = FALSE) {
  data_list <- list(...)
  if (length(data_list) == 0) {
    stop("At least one data object is required.")
  }

  dots <- match.call(expand.dots = FALSE)$...
  dataset_names <- if (!is.null(names(data_list))) {
    names(data_list)
  } else if (length(dots) == 1) {
    deparse(dots[[1]])
  } else {
    vapply(dots, deparse, character(1))
  }

  result <- lapply(seq_along(data_list), function(i) {
    data_obj <- data_list[[i]]

    if (is.null(variables) || identical(variables, "ALL")) {
      var_list <- names(data_obj$dimension_info)
    } else {
      var_list <- variables[variables %in% names(data_obj$dimension_info)]
    }

    var_list <- sort(var_list)

    if (length(var_list) == 0) return(data.frame())

    var_info <- lapply(var_list, function(var_name) {
      info <- get_dim_info(data_obj$dimension_info[[var_name]])
      df_row <- data.frame(
        Variable = var_name,
        Dimensions = info$dimension_string,
        DimSize = info$dim_size,
        DataShape = info$data_shape,
        stringsAsFactors = FALSE
      )
      if (include_col_size) {
        df_row$No.Col <- info$col_size
        df_row$No.Obs <- info$n_obs
      }
      df_row
    })

    do.call(rbind, var_info)
  })

  names(result) <- dataset_names
  return(result)
}


#' @title Compare Variable Structures Across SL4 and HAR Objects
#'
#' @description Compares variable structures across multiple SL4 and HAR datasets to ensure compatibility.
#' Identifies matching and mismatched variable structures, helping users diagnose inconsistencies.
#'
#' @details
#' - Verifies whether variables have consistent structures across multiple datasets.
#' - Ensures correct ordering of dimensions and checks for structural compatibility.
#' - If `keep_unique = TRUE`, returns a list of unique variable structures instead of performing a direct comparison.
#' - Useful for merging or aligning datasets before further processing.
#' - Helps detect differences in variable dimensions, which may arise due to model updates or dataset variations.
#'
#' @param variables Character vector. Variable names to compare. Use `NULL` or `"ALL"` to compare all variables.
#' @param ... Named SL4 or HAR objects to compare.
#' @param keep_unique Logical. If `TRUE`, returns unique variable structures across datasets instead of checking for compatibility. Default is `FALSE`.
#'
#' @return A list containing:
#' - `match`: A data frame listing variables with identical structures across datasets.
#' - `diff`: A data frame listing variables with mismatched structures, useful for debugging and alignment.
#' - If `keep_unique = TRUE`, instead of `match` and `diff`, returns a data frame with distinct variable structures across datasets.
#'
#' @importFrom stats setNames
#'
#' @author Pattawee Puangchit
#'
#' @seealso \code{\link{get_var_structure}}, \code{\link{get_dim_patterns}}, \code{\link{get_dim_elements}}
#'
#' @export
#'
#' @examples
#' # Import sample data:
#' har_data1 <- load_harx(system.file("extdata", "TAR10-WEL.har", package = "HARplus"))
#' har_data2 <- load_harx(system.file("extdata", "SUBT10-WEL.har", package = "HARplus"))
#'
#' # Compare structure for a single variable across multiple datasets
#' compare_var_structure("A", har_data1, har_data2)
#'
#' # Compare structure for multiple variables across multiple datasets
#' comparison_multiple <- compare_var_structure(c("A", "E1"), har_data1, har_data2)
#'
#' # Extract unique variable structures across multiple datasets
#' unique_vars <- compare_var_structure("ALL", har_data1, har_data2, keep_unique = TRUE)
#'
compare_var_structure <- function(variables = NULL, ..., keep_unique = FALSE) {
  inputs <- list(...)
  input_names <- if(!is.null(names(inputs))) names(inputs) else paste0("input", seq_along(inputs))
  if (length(inputs) < 2) stop("At least two data objects must be provided for comparison")

  get_var_info <- function(data_obj, var_name) {
    if (!var_name %in% names(data_obj$dimension_info)) return(NULL)
    get_dim_info(data_obj$dimension_info[[var_name]])
  }

  if (is.null(variables) || identical(variables, "ALL")) {
    variables <- unique(unlist(lapply(inputs, function(x) names(x$dimension_info))))
    variables <- sort(variables)
  }

  if (keep_unique) {
    match_df <- data.frame(
      Variable = character(),
      Dimensions = character(),
      DimSize = numeric(),
      DataShape = character(),
      stringsAsFactors = FALSE
    )

    diff_df <- data.frame(
      Variable = character(),
      stringsAsFactors = FALSE
    )
    for (name in input_names) {
      diff_df[[paste0(name, "_DimensionName")]] <- character()
    }

    for (var in variables) {
      var_dims <- list()
      for (i in seq_along(inputs)) {
        info <- get_var_info(inputs[[i]], var)
        if (!is.null(info)) {
          var_dims[[input_names[i]]] <- info
        }
      }

      if (length(unique(sapply(var_dims, function(x)
        paste(x$dimension_string, x$dim_size, x$data_shape)))) == 1) {
        first_info <- var_dims[[1]]
        match_df <- rbind(match_df, data.frame(
          Variable = var,
          Dimensions = first_info$dimension_string,
          DimSize = first_info$dim_size,
          DataShape = first_info$data_shape,
          stringsAsFactors = FALSE
        ))
      } else {
        new_row <- data.frame(Variable = var, stringsAsFactors = FALSE)
        for (name in input_names) {
          new_row[[paste0(name, "_DimensionName")]] <-
            if (!is.null(var_dims[[name]])) var_dims[[name]]$dimension_string else NA
        }
        diff_df <- rbind(diff_df, new_row)
      }
    }

    result <- list(match = match_df)
    if (nrow(diff_df) > 0) result$diff <- diff_df

  } else {
    match_df <- data.frame(
      Variable = character(),
      Dimensions = character(),
      DataShape = character(),
      stringsAsFactors = FALSE
    )
    for (name in input_names) {
      match_df[[paste0(name, "_ColSize")]] <- numeric()
    }

    diff_df <- data.frame(
      Variable = character(),
      stringsAsFactors = FALSE
    )
    for (name in input_names) {
      diff_df[[paste0(name, "_DimSize")]] <- character()
    }

    for (var in variables) {
      var_infos <- list()
      col_sizes <- numeric(length(inputs))

      for (i in seq_along(inputs)) {
        info <- get_var_info(inputs[[i]], var)
        if (!is.null(info)) {
          var_infos[[i]] <- info
          col_sizes[i] <- info$col_size
        }
      }

      all_match <- length(var_infos) == length(inputs) &&
        all(sapply(var_infos[-1], function(x)
          identical(x$dimension_string, var_infos[[1]]$dimension_string)))

      if (all_match) {
        new_row <- data.frame(
          Variable = var,
          Dimensions = var_infos[[1]]$dimension_string,
          DataShape = var_infos[[1]]$data_shape,
          stringsAsFactors = FALSE
        )
        for (i in seq_along(inputs)) {
          new_row[[paste0(input_names[i], "_ColSize")]] <- col_sizes[i]
        }
        match_df <- rbind(match_df, new_row)
      } else {
        new_row <- data.frame(Variable = var, stringsAsFactors = FALSE)
        for (i in seq_along(inputs)) {
          new_row[[paste0(input_names[i], "_DimSize")]] <-
            if (!is.null(var_infos[[i]])) var_infos[[i]]$dim_size else NA
        }
        diff_df <- rbind(diff_df, new_row)
      }
    }

    result <- list(match = match_df)
    if (nrow(diff_df) > 0) result$diff <- diff_df
  }

  return(result)
}


# Dimension Summary ------------------------------------------------------------

#' @title Get Dimension Patterns from SL4 and HAR Objects
#'
#' @description Extracts and lists unique dimension patterns (e.g., `REG*COMM`, `REG*REG*ACTS`) from one or more datasets.
#'
#' @details
#' - Extracts dimension structure details from the dataset.
#' - If multiple datasets are provided, combines their dimension information.
#' - If `keep_unique = TRUE`, returns only distinct dimension patterns.
#'
#' @param ... One or more structured SL4 or HAR objects containing dimension information.
#' @param keep_unique Logical. If `TRUE`, returns only unique dimension patterns. Default is `FALSE`.
#'
#' @return A data frame containing:
#' - `DimPattern`: The unique dimension patterns.
#'
#' @author Pattawee Puangchit
#'
#' @seealso \code{\link{get_dim_elements}}, \code{\link{get_var_structure}}
#'
#' @export
#'
#' @examples
#' # Import sample data:
#' sl4_data <- load_sl4x(system.file("extdata", "TAR10.sl4", package = "HARplus"))
#' sl4_data2 <- load_sl4x(system.file("extdata", "SUBT10.sl4", package = "HARplus"))
#'
#' # Extract dimension patterns
#' get_dim_patterns(sl4_data)
#'
#' # Extract only unique dimension patterns across datasets
#' get_dim_patterns(sl4_data, sl4_data2, keep_unique = TRUE)
#'
get_dim_patterns <- function(..., keep_unique = FALSE) {
  data_objs <- list(...)
  if (length(data_objs) == 0) stop("At least one data object is required.")

  patterns <- unlist(lapply(data_objs, function(obj) {
    sapply(obj$dimension_info, `[[`, "dimension_string")
  }))

  data.frame(
    DimPattern = if(keep_unique) unique(patterns) else patterns,
    stringsAsFactors = FALSE,
    row.names = NULL
  )
}


#' @title Get Dimension Elements from SL4 and HAR Objects
#'
#' @description Extracts and lists unique dimension elements (e.g., `REG`, `COMM`, `ACTS`) from one or more datasets.
#'
#' @param ... One or more structured SL4 or HAR objects containing dimension information.
#' @param keep_unique Logical. If `TRUE`, returns only unique dimension elements across inputs. Default is `FALSE`.
#'
#' @return A data frame containing unique dimension elements.
#'
#' @author Pattawee Puangchit
#'
#' @seealso \code{\link{get_dim_patterns}}, \code{\link{get_var_structure}}
#'
#' @export
#'
#' @examples
#' # Import sample data:
#' sl4_data1 <- load_sl4x(system.file("extdata", "TAR10.sl4", package = "HARplus"))
#' sl4_data2 <- load_sl4x(system.file("extdata", "SUBT10.sl4", package = "HARplus"))
#'
#'
#' # Extract dimension elements from a single dataset
#' get_dim_elements(sl4_data1)
#'
#' # Extract dimension elements from multiple datasets
#' get_dim_elements(sl4_data1, sl4_data2)
#'
#' # Extract unique dimension elements across datasets
#' get_dim_elements(sl4_data1, sl4_data2, keep_unique = TRUE)
#'
get_dim_elements <- function(..., keep_unique = FALSE) {
  data_objs <- list(...)
  if (length(data_objs) == 0) stop("At least one data object is required.")

  elements <- unlist(lapply(data_objs, function(obj) {
    unique(unlist(lapply(obj$dimension_info, `[[`, "dimension_names")))
  }))

  data.frame(
    DimName = if(keep_unique) unique(elements) else elements,
    stringsAsFactors = FALSE,
    row.names = NULL
  )
}

#' @title Create Initial Value Configuration
#'
#' @description
#' Defines the configuration for loading the initial dataset, including path, format,
#' variable header, and value column name. Used as input for \code{\link{shock_calculate}}
#' and \code{\link{shock_calculate_uniform}}.
#'
#' @details
#' - Supports `HAR` and `SL4` file formats
#' - Specifies the header name to extract from the dataset
#' - Allows custom naming for the value column (`Value` by default)
#'
#' @param path Path to the initial data file.
#' @param format File format of the initial dataset. Must be "har" or "sl4".
#' @param header Header name within the HAR or SL4 file to extract.
#' @param value_col Name of the column containing numeric values. Default is "Value".
#'
#' @return A list containing:
#'    - \code{path}: input file path
#'    - \code{format}: file format ("har" or "sl4")
#'    - \code{header}: target header name
#'    - \code{value_col}: column name for numeric values
#'
#' @author Pattawee Puangchit
#'
#' @seealso \code{\link{create_target_config}}, \code{\link{create_calc_config}},
#'   \code{\link{shock_calculate}}, \code{\link{shock_calculate_uniform}}
#'
#' @export
#'
#' @examples
#' # Example: Define Initial Configuration
#' initial <- create_initial_config(
#'   path   = "D:/Data/taxrates_2017.har",
#'   format = "har",
#'   header = "rTMS"
#' )
create_initial_config <- function(path, format, header, value_col = "Value") {
  if (!format %in% c("sl4", "har")) stop("format must be either 'sl4' or 'har'")
  list(path = path, format = tolower(format), header = header, value_col = value_col)
}

#' @title Create Target Value Configuration
#'
#' @description
#' Defines the configuration for loading the target dataset, which represents
#' post-adjustment or comparative rate values. Supports multiple file formats.
#'
#' @details
#' - Supports `HAR`, `SL4`, `CSV`, and `XLSX` file formats
#' - Can also represent a uniform numeric target value when no file path is provided
#' - Used in combination with \code{\link{create_initial_config}} for shock computation
#'
#' @param path Optional path to the target data file.
#' @param type Optional file type for the target dataset ("har", "sl4", "csv", or "xlsx").
#' @param header Optional header name within the HAR or SL4 file to extract.
#' @param value_col Column name containing numeric target values. Default is "Value".
#'
#' @return A list containing:
#'    - \code{path}: file path to target data
#'    - \code{type}: file format (lowercase)
#'    - \code{header}: header name in HAR/SL4 file
#'    - \code{value_col}: column name for target values
#'
#' @author Pattawee Puangchit
#'
#' @seealso \code{\link{create_initial_config}}, \code{\link{create_calc_config}},
#'   \code{\link{shock_calculate}}, \code{\link{shock_calculate_uniform}}
#'
#' @export
#'
#' @examples
#' # Example: Define Target Configuration
#' target <- create_target_config(
#'   path   = "D:/Data/taxrates_2019.har",
#'   type   = "har",
#'   header = "rTMS"
#' )
create_target_config <- function(path = NULL, type = NULL, header = NULL, value_col = "Value") {
  if (!is.null(type) && !type %in% c("sl4", "har", "csv", "xlsx")) {
    stop("type must be 'sl4', 'har', 'csv', 'xlsx', or NULL")
  }
  list(path = path, type = if(!is.null(type)) tolower(type) else NULL, header = header, value_col = value_col)
}

#' @title Create Calculation Configuration
#'
#' @description
#' Defines calculation settings for generating shock values, including variable mapping,
#' timeline sequence, and exclusion criteria. Used as input for both
#' \code{\link{shock_calculate}} and \code{\link{shock_calculate_uniform}}.
#'
#' @details
#' - Controls column alignment between initial and target datasets
#' - Supports numeric or range-based timeline definitions (e.g., "1-10")
#' - Excludes self-trade or specified region/sector pairs if configured
#' - Provides core metadata for shock calculation functions
#'
#' @param column_mapping Optional named vector mapping initial and target columns
#'   (e.g., \code{c(COMM = "COMM", REG = "REG", REG.1 = "REG.1")}).
#' @param timeline Numeric or character range defining simulation periods.
#'   - Example: "1-5" expands to 1:5.
#' @param exclude_self_trade Logical; if TRUE, removes intra-region records. Default is FALSE.
#' @param exclusion_values Optional named list of elements to exclude from calculation.
#'
#' @return A list containing:
#'    - \code{column_mapping}: variable mapping between datasets
#'    - \code{timeline}: expanded numeric sequence of simulation periods
#'    - \code{exclude_self_trade}: logical flag
#'    - \code{exclusion_values}: exclusion list by dimension
#'
#' @author Pattawee Puangchit
#'
#' @seealso \code{\link{create_initial_config}}, \code{\link{create_target_config}},
#'   \code{\link{shock_calculate}}, \code{\link{shock_calculate_uniform}}
#'
#' @export
#'
#' @examples
#' # Example: Define Calculation Configuration
#' calc <- create_calc_config(
#'   column_mapping      = c(COMM = "COMM", REG = "REG", REG.1 = "REG.1"),
#'   timeline            = "1-5",
#'   exclude_self_trade  = TRUE
#' )
create_calc_config <- function(column_mapping = NULL, timeline = 1, 
                               exclude_self_trade = FALSE, exclusion_values = NULL) {
  if (is.character(timeline) && grepl("-", timeline)) {
    range_parts <- as.numeric(strsplit(timeline, "-")[[1]])
    timeline_seq <- seq(range_parts[1], range_parts[2])
  } else {
    timeline_seq <- as.numeric(timeline)
  }
  list(column_mapping = column_mapping, timeline = timeline_seq, 
       exclude_self_trade = exclude_self_trade, exclusion_values = exclusion_values)
}

#' @title Calculate Shocks with Uniform Adjustment
#'
#' @description
#' Generates GEMPACK-style percentage shocks using a uniform proportional adjustment
#' applied to all values in the initial dataset. The function supports additive or
#' multiplicative adjustments, single or multi-period configurations, and direct HAR export.
#'
#' @details
#' - Applies uniform adjustment to all base rates across defined dimensions
#' - Supports additive ("+", "-") and proportional ("*", "/") adjustments
#' - Computes compounded shocks following the "power of tax" formulation
#' - Handles multiple time periods as defined by `timeline` in \code{calc_config}
#' - Excludes self-trade or specified region/sector pairs if configured
#' - Outputs results as multi-header HAR file (one per timeline period)
#' - Optionally generates new baseline rate file showing post-adjustment values
#'
#' @param initial_config A list created by \code{\link{create_initial_config}}, defining:
#'    - Input file path, format, and variable header
#'    - Column name for initial rate values ("Value_ini")
#' @param adjustment_value Numeric scalar specifying the uniform adjustment to apply.
#'    - Interpreted according to \code{calculation_method}
#'    - For example, 0.5 with method "*" halves the base rate
#' @param calculation_method Operator defining the adjustment method:
#'    - "*" multiply (default)
#'    - "/" divide
#'    - "+" add
#'    - "-" subtract
#' @param calc_config A list created by \code{\link{create_calc_config}}, specifying:
#'    - \code{timeline}: sequence of simulation periods (e.g., "1-10")
#'    - \code{exclude_self_trade}: logical, whether to omit intra-region pairs
#'    - \code{exclusion_values}: optional list defining excluded elements
#' @param output_path Path to the output HAR file where calculated shocks will be written.
#' @param long_desc Optional text for header description. Default is "Uniform shock adjustment".
#' @param dim_order Optional dimension ordering specification. Can be:
#'    - NULL (default): alphabetical A-Z ordering
#'    - a named list defining preferred order per dimension
#'    - a data frame or path to Excel/CSV with explicit order definitions
#' @param lowercase Logical; if TRUE, converts all dimension elements to lowercase. Default is FALSE.
#' @param new_baseline Logical; if TRUE, generates an additional HAR file with the new baseline rates
#'    (target values where available, initial values otherwise). The output file will have "_baseline"
#'    appended to the filename. Default is FALSE.
#'
#' @return Invisibly returns a list containing summary metadata:
#'    - \code{n_observations}: total records processed
#'    - \code{n_included}: records included in shock computation
#'    - \code{n_excluded}: records excluded by configuration
#'    - \code{output_path}: normalized path to the generated HAR file
#'    - \code{baseline_path}: normalized path to baseline file (if new_baseline = TRUE)
#'
#' @author Pattawee Puangchit
#'
#' @seealso \code{\link{shock_calculate}}, \code{\link{create_initial_config}},
#'   \code{\link{create_calc_config}}, \code{\link{save_har}}
#'
#' @export
#'
#' @examples
#' # Example 1: Uniform Shock (50% Reduction) with New Baseline
#' har_path <- system.file("extdata", "baserate.har", package = "HARplus")
#'
#' # Sorting Column
#' mapping <- list(
#'   REG = c("USA", "EU", "ROW")
#' )
#'
#' # Initial File
#' initial <- create_initial_config(
#'   path   = har_path,
#'   format = "har",
#'   header = "rTMS"
#' )
#'
#' # Calculation Setup
#' calc <- create_calc_config(
#'   timeline           = "1-10",
#'   exclude_self_trade = TRUE
#' )
#'
#' # Compute Uniform 50% Reduction (Value_tar = Value_ini * 0.5)
#' # Also generate new baseline file showing the adjusted rates
#' shock_calculate_uniform(
#'   initial_config     = initial,
#'   adjustment_value   = 0.5,
#'   calculation_method = "*",
#'   calc_config        = calc,
#'   output_path        = file.path(tempdir(), "output_uniform.har"),
#'   dim_order          = mapping,
#'   new_baseline       = TRUE
#' )
shock_calculate_uniform <- function(initial_config, adjustment_value, 
                                    calculation_method = "*", calc_config,
                                    output_path, long_desc = "Uniform shock adjustment",
                                    dim_order = NULL, lowercase = FALSE, new_baseline = FALSE) {
  
  if (!calculation_method %in% c("+", "-", "*", "/")) {
    stop("calculation_method must be one of: '+', '-', '*', '/'")
  }
  
  initial_df <- load_data_simple(initial_config$path, initial_config$format, initial_config$header)
  
  if (!initial_config$value_col %in% names(initial_df)) {
    stop(sprintf("Value column '%s' not found", initial_config$value_col))
  }
  
  meta_cols <- c("Value", "Variable", "Dimension", "Experiment", "Subtotal")
  dim_cols <- setdiff(names(initial_df), meta_cols)
  
  if (!is.null(calc_config$exclusion_values)) {
    for (col in names(calc_config$exclusion_values)) {
      if (!col %in% names(initial_df)) {
        stop(sprintf("Exclusion column '%s' not found", col))
      }
    }
  }
  
  keep_mask <- rep(TRUE, nrow(initial_df))
  
  if (calc_config$exclude_self_trade && length(dim_cols) > 1) {
    for (i in 1:(length(dim_cols) - 1)) {
      for (j in (i + 1):length(dim_cols)) {
        self_mask <- initial_df[[dim_cols[i]]] == initial_df[[dim_cols[j]]]
        self_mask[is.na(self_mask)] <- FALSE
        keep_mask <- keep_mask & !self_mask
      }
    }
  }
  
  if (!is.null(calc_config$exclusion_values)) {
    for (col in names(calc_config$exclusion_values)) {
      if (col %in% names(initial_df)) {
        ex_mask <- initial_df[[col]] %in% calc_config$exclusion_values[[col]]
        ex_mask[is.na(ex_mask)] <- FALSE
        keep_mask <- keep_mask & !ex_mask
      }
    }
  }
  
  timeline_labels <- c("ONEY", "TWOY", "THRY", "FOUR", "FIVE", "SIXY", "SEVN", 
                       "EIGT", "NINE", "TENY", "ELVN", "TWLV", "THTN", "FRTN", "FFTN",
                       "SXTN", "SVTN", "EGTN", "NNTN", "TWTY", "TONE", "TTWO", "TTRE",
                       "TFOR", "TFIV", "TSIX", "TSEV", "TEIG", "TNIN", "THTY", "TONE",
                       "TTWO", "TTRE", "TFOR", "TFIV", "TSIX", "TSEV", "TEIG", "TNIN",
                       "FRTY", "FONE", "FTWO", "FTRE", "FFOR", "FFIV", "FSIX", "FSEV",
                       "FEIG", "FNIN", "FFTY", "FONE", "FTWO", "FTRE", "FFOR", "FFIV",
                       "FSIX", "FSEV", "FEIG", "FNIN", "SXTY", "SONE", "STWO", "STRE",
                       "SFOR", "SFIV", "SSIX", "SSEV", "SEIG", "SNIN", "SVTY", "VONE",
                       "VTWO", "VTRE", "VFOR", "VFIV", "VSIX", "VSEV", "VEIG", "VNIN",
                       "EGTY", "EONE", "ETWO", "ETRE", "EFOR", "EFIV", "ESIX", "ESEV",
                       "EEIG", "ENIN", "NNTY", "NONE", "NTWO", "NTRE", "NFOR", "NFIV",
                       "NSIX", "NSEV", "NEIG", "NNIN", "HUND")
  output_headers <- timeline_labels[1:length(calc_config$timeline)]
  
  shock_list <- list()
  
  for (i in seq_along(calc_config$timeline)) {
    period <- calc_config$timeline[i]
    shock_df <- initial_df
    
    Value_ini <- shock_df[[initial_config$value_col]]
    
    # Calculate target values
    Value_tar <- switch(calculation_method,
                        "+" = Value_ini + adjustment_value,
                        "-" = Value_ini - adjustment_value,
                        "*" = Value_ini * adjustment_value,
                        "/" = Value_ini / adjustment_value
    )
    
    # Calculate compound shock using power-of-tax formula
    denominator <- 1 + Value_ini/100
    shock_df$Value <- ifelse(abs(denominator) < 1e-10, 0,
                             100 * (((1 + Value_tar/100) / denominator)^(1/period) - 1))
    shock_df$Value[is.na(shock_df$Value)] <- 0
    shock_df$Value[is.infinite(shock_df$Value)] <- 0
    shock_df$Value[!keep_mask] <- 0
    
    keep_cols <- c(dim_cols, "Value")
    if ("Subtotal" %in% names(shock_df)) keep_cols <- c(keep_cols, "Subtotal")
    
    shock_list[[output_headers[i]]] <- shock_df[, keep_cols, drop = FALSE]
  }
  
  dimension_list <- lapply(shock_list, function(x) setdiff(names(x), c("Value", "Subtotal")))
  names(dimension_list) <- output_headers
  
  dim_rename_list <- lapply(output_headers, function(header) {
    dims <- dimension_list[[header]]
    setNames(gsub("\\.\\d+$", "", dims), dims)
  })
  names(dim_rename_list) <- output_headers
  
  unique_dim_names <- unique(unlist(lapply(dim_rename_list, function(x) unique(as.character(x)))))
  
  processed_dim_order <- process_dim_order(dim_order, lowercase)
  if (!is.null(processed_dim_order)) {
    expanded_dim_order <- list()
    for (base_name in unique_dim_names) {
      if (base_name %in% names(processed_dim_order)) {
        expanded_dim_order[[base_name]] <- processed_dim_order[[base_name]]
      }
    }
    processed_dim_order <- if (length(expanded_dim_order) > 0) expanded_dim_order else NULL
  }
  
  long_desc_vec <- setNames(
    paste0(long_desc, " (Year ", calc_config$timeline, ")"),
    output_headers
  )
  
  save_har(
    data_list = shock_list,
    file_path = output_path,
    dimensions = dimension_list,
    value_cols = setNames(rep("Value", length(shock_list)), output_headers),
    long_desc = long_desc_vec,
    dim_rename = dim_rename_list,
    export_sets = TRUE,
    lowercase = lowercase,
    dim_order = processed_dim_order
  )
  
  # Generate new baseline file if requested
  if (new_baseline) {
    baseline_path <- sub("\\.har$", "_baseline.har", output_path, ignore.case = TRUE)
    if (baseline_path == output_path) {
      baseline_path <- paste0(tools::file_path_sans_ext(output_path), "_baseline.har")
    }
    
    # Create single baseline dataframe (not time-dependent)
    baseline_df <- initial_df
    
    Value_ini <- baseline_df[[initial_config$value_col]]
    
    # Calculate new baseline rates (target values)
    Value_tar <- switch(calculation_method,
                        "+" = Value_ini + adjustment_value,
                        "-" = Value_ini - adjustment_value,
                        "*" = Value_ini * adjustment_value,
                        "/" = Value_ini / adjustment_value
    )
    
    # For new baseline, use target value where included, initial value where excluded
    baseline_df$Value <- ifelse(keep_mask, Value_tar, Value_ini)
    baseline_df$Value[is.na(baseline_df$Value)] <- Value_ini[is.na(baseline_df$Value)]
    baseline_df$Value[is.infinite(baseline_df$Value)] <- Value_ini[is.infinite(baseline_df$Value)]
    
    keep_cols <- c(dim_cols, "Value")
    if ("Subtotal" %in% names(baseline_df)) keep_cols <- c(keep_cols, "Subtotal")
    
    # Single header using initial header name
    baseline_header <- toupper(substr(initial_config$header, 1, 4))
    baseline_list <- list()
    baseline_list[[baseline_header]] <- baseline_df[, keep_cols, drop = FALSE]
    
    baseline_dimension_list <- list()
    baseline_dimension_list[[baseline_header]] <- setdiff(names(baseline_list[[baseline_header]]), c("Value", "Subtotal"))
    
    baseline_dim_rename <- list()
    dims <- baseline_dimension_list[[baseline_header]]
    baseline_dim_rename[[baseline_header]] <- setNames(gsub("\\.\\d+$", "", dims), dims)
    
    save_har(
      data_list = baseline_list,
      file_path = baseline_path,
      dimensions = baseline_dimension_list,
      value_cols = setNames("Value", baseline_header),
      long_desc = setNames("New baseline rates", baseline_header),
      dim_rename = baseline_dim_rename,
      export_sets = TRUE,
      lowercase = lowercase,
      dim_order = processed_dim_order
    )
    
    message(sprintf("\nGenerated new baseline file: %s", normalizePath(baseline_path)))
  }
  
  invisible(list(
    n_observations = nrow(initial_df),
    n_included = sum(keep_mask),
    n_excluded = sum(!keep_mask),
    output_path = normalizePath(output_path),
    baseline_path = if(new_baseline) normalizePath(baseline_path) else NULL
  ))
}

#' @title Calculate Shocks from Initial and Target Values
#'
#' @description
#' Computes compounded GEMPACK-style percentage shocks between initial and target values,
#' producing multi-period shock series for dynamic simulation models. The function
#' automatically aligns dimensions across datasets and exports results in HAR format.
#'
#' @details
#' - Computes percentage shocks using compounded "power of tax" formula
#' - Supports multiple periods defined via `timeline` configuration
#' - Compatible with HAR, SL4, CSV, or XLSX input formats
#' - Excludes self-trade or specified region-sector pairs when configured
#' - Exports results as multi-header HAR file (one header per timeline period)
#' - Optionally generates new baseline rate file showing post-adjustment values
#'
#' @param initial_config A list created by \code{\link{create_initial_config}}, defining:
#'    - Input path, file format, and variable header for the initial dataset
#'    - Column name of the initial value field ("Value_ini")
#' @param target_config A list created by \code{\link{create_target_config}}, defining:
#'    - Path, format, and variable header for the target dataset
#'    - Target value column ("Value_tar") or numeric target for uniform shock
#' @param calc_config A list created by \code{\link{create_calc_config}}, specifying:
#'    - \code{column_mapping}: mapping between initial and target columns
#'    - \code{timeline}: sequence of years or periods (e.g., "1-5")
#'    - \code{exclude_self_trade}: logical, whether to drop self-pairs
#'    - \code{exclusion_values}: list of region/sector values to omit
#' @param output_path Path to the output HAR file where calculated shocks will be written.
#' @param long_desc Optional text for header description. Default is "Calculated shock values".
#' @param dim_order Optional dimension ordering specification. Can be:
#'    - NULL (default): alphabetical A-Z ordering
#'    - a named list defining order for each dimension (e.g., REG, COMM)
#'    - a data frame or path to Excel/CSV file containing order definitions
#' @param lowercase Logical; if TRUE, converts dimension elements to lowercase. Default is FALSE.
#' @param new_baseline Logical; if TRUE, generates an additional HAR file with the new baseline rates
#'    (target values where available, initial values otherwise). The output file will have "_baseline"
#'    appended to the filename. Default is FALSE.
#'
#' @return Invisibly returns a list containing summary metadata:
#'    - \code{n_observations}: total records processed
#'    - \code{n_included}: records included in shock computation
#'    - \code{n_excluded}: records excluded by configuration
#'    - \code{output_path}: normalized path to output HAR file
#'    - \code{baseline_path}: normalized path to baseline file (if new_baseline = TRUE)
#'
#' @author Pattawee Puangchit
#'
#' @seealso \code{\link{shock_calculate_uniform}}, \code{\link{create_initial_config}},
#'   \code{\link{create_target_config}}, \code{\link{create_calc_config}}, \code{\link{save_har}}
#'
#' @export
#'
#' @examples
#' # Example 1: Target-Based Shock Calculation with New Baseline
#' har_path <- system.file("extdata", "baserate.har", package = "HARplus")
#'
#' # Sorting Column
#' mapping <- list(
#'   REG = c("USA", "EU", "ROW")
#' )
#'
#' # Initial File
#' initial <- create_initial_config(
#'   path   = har_path,
#'   format = "har",
#'   header = "rTMS"
#' )
#'
#' # Target File
#' target <- create_target_config(
#'   path   = har_path,
#'   type   = "har",
#'   header = "rTMS"
#' )
#'
#' # Calculation Setup with Column Mapping
#' calc <- create_calc_config(
#'   column_mapping      = c(TRAD_COMM = "TRAD_COMM", REG = "REG", REG.1 = "REG.1"),
#'   timeline            = "1-5",
#'   exclude_self_trade  = TRUE
#' )
#'
#' # Compute Shock Based on Initial and Target Values
#' # Also generate new baseline file
#' shock_calculate(
#'   initial_config = initial,
#'   target_config  = target,
#'   calc_config    = calc,
#'   output_path    = file.path(tempdir(), "output_target.har"),
#'   dim_order      = mapping,
#'   new_baseline   = TRUE
#' )
shock_calculate <- function(initial_config, target_config, calc_config,
                            output_path, long_desc = "Calculated shock values",
                            dim_order = NULL, lowercase = FALSE, new_baseline = FALSE) {
  
  if (is.null(calc_config$column_mapping)) {
    stop("column_mapping is required for target file shocks")
  }
  
  initial_df <- load_data_simple(initial_config$path, initial_config$format, initial_config$header)
  
  if (!initial_config$value_col %in% names(initial_df)) {
    stop(sprintf("Value column '%s' not found in initial", initial_config$value_col))
  }
  
  initial_cols <- names(calc_config$column_mapping)
  target_cols <- unname(calc_config$column_mapping)
  
  missing_initial <- setdiff(initial_cols, names(initial_df))
  if (length(missing_initial) > 0) {
    stop(sprintf("Initial columns not found: %s", paste(missing_initial, collapse = ", ")))
  }
  
  initial_df <- initial_df[, c(initial_cols, initial_config$value_col), drop = FALSE]
  names(initial_df)[names(initial_df) == initial_config$value_col] <- "Value_ini"
  
  if (is.null(target_config$type)) {
    if (!is.numeric(target_config$value_col)) {
      stop("For uniform target, value_col must be numeric")
    }
    initial_df$Value_tar <- target_config$value_col
    merged_df <- initial_df
  } else {
    target_df <- load_data_simple(target_config$path, target_config$type, target_config$header)
    
    if (!target_config$value_col %in% names(target_df)) {
      stop(sprintf("Value column '%s' not found in target", target_config$value_col))
    }
    
    missing_target <- setdiff(target_cols, names(target_df))
    if (length(missing_target) > 0) {
      stop(sprintf("Target columns not found: %s", paste(missing_target, collapse = ", ")))
    }
    
    target_df <- target_df[, c(target_cols, target_config$value_col), drop = FALSE]
    names(target_df)[names(target_df) == target_config$value_col] <- "Value_tar"
    
    names(target_df)[match(target_cols, names(target_df))] <- initial_cols
    
    merged_df <- merge(initial_df, target_df, by = initial_cols, all.x = TRUE)
  }
  
  keep_mask <- rep(TRUE, nrow(merged_df))
  
  if (calc_config$exclude_self_trade && length(initial_cols) > 1) {
    for (i in 1:(length(initial_cols) - 1)) {
      for (j in (i + 1):length(initial_cols)) {
        self_mask <- merged_df[[initial_cols[i]]] == merged_df[[initial_cols[j]]]
        self_mask[is.na(self_mask)] <- FALSE
        keep_mask <- keep_mask & !self_mask
      }
    }
  }
  
  if (!is.null(calc_config$exclusion_values)) {
    for (col in names(calc_config$exclusion_values)) {
      if (col %in% names(merged_df)) {
        ex_mask <- merged_df[[col]] %in% calc_config$exclusion_values[[col]]
        ex_mask[is.na(ex_mask)] <- FALSE
        keep_mask <- keep_mask & !ex_mask
      }
    }
  }
  
  timeline_labels <- c("ONEY", "TWOY", "THRY", "FOUR", "FIVE", "SIXY", "SEVN",
                       "EIGT", "NINE", "TENY", "ELVN", "TWLV", "THTN", "FRTN", "FFTN",
                       "SXTN", "SVTN", "EGTN", "NNTN", "TWTY", "TONE", "TTWO", "TTRE",
                       "TFOR", "TFIV", "TSIX", "TSEV", "TEIG", "TNIN", "THTY", "TONE",
                       "TTWO", "TTRE", "TFOR", "TFIV", "TSIX", "TSEV", "TEIG", "TNIN",
                       "FRTY", "FONE", "FTWO", "FTRE", "FFOR", "FFIV", "FSIX", "FSEV",
                       "FEIG", "FNIN", "FFTY", "FONE", "FTWO", "FTRE", "FFOR", "FFIV",
                       "FSIX", "FSEV", "FEIG", "FNIN", "SXTY", "SONE", "STWO", "STRE",
                       "SFOR", "SFIV", "SSIX", "SSEV", "SEIG", "SNIN", "SVTY", "VONE",
                       "VTWO", "VTRE", "VFOR", "VFIV", "VSIX", "VSEV", "VEIG", "VNIN",
                       "EGTY", "EONE", "ETWO", "ETRE", "EFOR", "EFIV", "ESIX", "ESEV",
                       "EEIG", "ENIN", "NNTY", "NONE", "NTWO", "NTRE", "NFOR", "NFIV",
                       "NSIX", "NSEV", "NEIG", "NNIN", "HUND")
  output_headers <- timeline_labels[1:length(calc_config$timeline)]
  
  shock_list <- list()
  
  for (i in seq_along(calc_config$timeline)) {
    period <- calc_config$timeline[i]
    shock_df <- merged_df
    
    denominator <- 1 + shock_df$Value_ini/100
    shock_df$Value <- ifelse(abs(denominator) < 1e-10, 0,
                             100 * (((1 + shock_df$Value_tar/100) / denominator)^(1/period) - 1))
    shock_df$Value[is.na(shock_df$Value)] <- 0
    shock_df$Value[is.infinite(shock_df$Value)] <- 0
    shock_df$Value[!keep_mask] <- 0
    
    keep_cols <- c(initial_cols, "Value")
    shock_list[[output_headers[i]]] <- shock_df[, keep_cols, drop = FALSE]
  }
  
  dimension_list <- lapply(shock_list, function(x) setdiff(names(x), c("Value", "Subtotal")))
  names(dimension_list) <- output_headers
  
  dim_rename_list <- lapply(output_headers, function(header) {
    dims <- dimension_list[[header]]
    setNames(gsub("\\.\\d+$", "", dims), dims)
  })
  names(dim_rename_list) <- output_headers
  
  unique_dim_names <- unique(unlist(lapply(dim_rename_list, function(x) unique(as.character(x)))))
  
  processed_dim_order <- process_dim_order(dim_order, lowercase)
  if (!is.null(processed_dim_order)) {
    expanded_dim_order <- list()
    for (base_name in unique_dim_names) {
      if (base_name %in% names(processed_dim_order)) {
        expanded_dim_order[[base_name]] <- processed_dim_order[[base_name]]
      }
    }
    processed_dim_order <- if (length(expanded_dim_order) > 0) expanded_dim_order else NULL
  }
  
  long_desc_vec <- setNames(
    paste0(long_desc, " (Year ", calc_config$timeline, ")"),
    output_headers
  )
  
  save_har(
    data_list = shock_list,
    file_path = output_path,
    dimensions = dimension_list,
    value_cols = setNames(rep("Value", length(shock_list)), output_headers),
    long_desc = long_desc_vec,
    dim_rename = dim_rename_list,
    export_sets = TRUE,
    lowercase = lowercase,
    dim_order = processed_dim_order
  )
  
  # Generate new baseline file if requested
  if (new_baseline) {
    baseline_path <- sub("\\.har$", "_baseline.har", output_path, ignore.case = TRUE)
    if (baseline_path == output_path) {
      baseline_path <- paste0(tools::file_path_sans_ext(output_path), "_baseline.har")
    }
    
    # Create single baseline dataframe (not time-dependent)
    baseline_df <- merged_df
    
    # For new baseline, use target value where available and included, initial value otherwise
    baseline_df$Value <- ifelse(keep_mask & !is.na(baseline_df$Value_tar), 
                                baseline_df$Value_tar, 
                                baseline_df$Value_ini)
    baseline_df$Value[is.na(baseline_df$Value)] <- baseline_df$Value_ini[is.na(baseline_df$Value)]
    baseline_df$Value[is.infinite(baseline_df$Value)] <- baseline_df$Value_ini[is.infinite(baseline_df$Value)]
    
    keep_cols <- c(initial_cols, "Value")
    
    # Single header using initial header name
    baseline_header <- toupper(substr(initial_config$header, 1, 4))
    baseline_list <- list()
    baseline_list[[baseline_header]] <- baseline_df[, keep_cols, drop = FALSE]
    
    baseline_dimension_list <- list()
    baseline_dimension_list[[baseline_header]] <- setdiff(names(baseline_list[[baseline_header]]), c("Value", "Subtotal"))
    
    baseline_dim_rename <- list()
    dims <- baseline_dimension_list[[baseline_header]]
    baseline_dim_rename[[baseline_header]] <- setNames(gsub("\\.\\d+$", "", dims), dims)
    
    save_har(
      data_list = baseline_list,
      file_path = baseline_path,
      dimensions = baseline_dimension_list,
      value_cols = setNames("Value", baseline_header),
      long_desc = setNames("New baseline rates", baseline_header),
      dim_rename = baseline_dim_rename,
      export_sets = TRUE,
      lowercase = lowercase,
      dim_order = processed_dim_order
    )
    
    message(sprintf("\nGenerated new baseline file: %s", normalizePath(baseline_path)))
  }
  
  invisible(list(
    n_observations = nrow(merged_df),
    n_included = sum(keep_mask),
    n_excluded = sum(!keep_mask),
    output_path = normalizePath(output_path),
    baseline_path = if(new_baseline) normalizePath(baseline_path) else NULL
  ))
}

#' @keywords internal
#' @noRd
load_data_simple <- function(path, format, header = NULL) {
  if (!file.exists(path)) stop(sprintf("ERROR: File not found: '%s'", path))
  
  if (format == "sl4") {
    if (is.null(header) || header == "") stop("ERROR: Header required for SL4")
    data_obj <- load_sl4x(path, lowercase = FALSE)
    if (!header %in% names(data_obj$data)) {
      stop(sprintf("ERROR: Header '%s' not found\nAvailable: %s",
                   header, paste(names(data_obj$data), collapse = ", ")))
    }
    result <- get_data_by_var(header, data_obj)
    return(result[[1]][[header]])
  } else if (format == "har") {
    if (is.null(header) || header == "") stop("ERROR: Header required for HAR")
    data_obj <- load_harx(path, lowercase = FALSE)
    if (!header %in% names(data_obj$data)) {
      stop(sprintf("ERROR: Header '%s' not found\nAvailable: %s",
                   header, paste(names(data_obj$data), collapse = ", ")))
    }
    result <- get_data_by_var(header, data_obj)
    return(result[[1]][[header]])
  } else if (format == "csv") {
    return(read.csv(path, stringsAsFactors = FALSE))
  } else if (format == "xlsx") {
    if (is.null(header)) stop("ERROR: Sheet required for XLSX")
    if (!requireNamespace("openxlsx", quietly = TRUE)) stop("openxlsx package required")
    return(openxlsx::read.xlsx(path, sheet = header))
  }
  stop("Unsupported format")
}

#' @keywords internal
#' @noRd
process_dim_order <- function(dim_order, lowercase = FALSE) {
  if (is.null(dim_order)) return(NULL)
  
  if (is.data.frame(dim_order)) {
    order_map <- lapply(names(dim_order), function(col) {
      vals <- dim_order[[col]]
      vals <- vals[!is.na(vals) & nchar(trimws(as.character(vals))) > 0]
      vals <- unique(trimws(as.character(vals)))
      if (lowercase) tolower(vals) else vals
    })
    names(order_map) <- names(dim_order)
    return(order_map[sapply(order_map, length) > 0])
  }
  
  if (is.list(dim_order)) {
    order_map <- lapply(dim_order, function(vals) {
      vals <- vals[!is.na(vals) & nchar(trimws(as.character(vals))) > 0]
      vals <- unique(trimws(as.character(vals)))
      if (lowercase) tolower(vals) else vals
    })
    return(order_map[sapply(order_map, length) > 0])
  }
  
  return(NULL)
}

---
title: "Enhanced R Package for GEMPACK .HAR and .SL4 File Processing"
author: "Pattawee Puangchit"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{HARplus: Enhanced R Package for GEMPACK .HAR and .SL4 File Processing}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message = FALSE,
  eval = requireNamespace("HARplus", quietly = TRUE)
)
```

# Overview of HARplus

The `HARplus` package enhances GEMPACK users' experience by streamlining `.har` and `.sl4` file processing. It efficiently extracts, organizes, and manages dimension structures, ensuring consistency, optimized memory usage, and simplified data manipulation. The package enables fast data merging, aggregation, and transformation while maintaining a consistent interface across GEMPACK file types. Users can easily pivot data into familiar formats and export results in various formats.

A key feature of `HARplus` is its **flexible subtotal level handling**, allowing users to selectively retain `"TOTAL"` values, decomposed components, or both. This ensures precise data extraction for various economic modeling needs without unnecessary redundancy.

# Introduction

This vignette covers key functions of `HARplus`, highlighting its ability to handle multiple inputs for efficient data processing. Designed for economic modelers and GEMPACK practitioners, it simplifies working with multiple datasets and enhances analytical workflows.

Sample data used in this vignette is obtained from the GTAPv7 model and utilizes publicly available data from the GTAP 9 database. For more details, refer to the [GTAP Database Archive](https://www.gtap.agecon.purdue.edu/databases/archives.asp).

# Acknowledgement

The development of `HARplus` builds upon the foundational work of the `HARr` package. I sincerely acknowledge and appreciate the contributions of Maros Ivanic, whose `HARr` package served as the baseline for this development, advancing efficient data handling for GEMPACK users.

# GTAPViz: An Extension of HARplus for Visualization

I have developed another package specifically for visualization, particularly for GTAP users: **[GTAPViz](https://www.pattawee-pp.com/GTAPViz/)**

---

# Installation and Loading Example Data  
Before proceeding, ensure that `HARplus` is installed and loaded:

```{r load package}
library(HARplus)
```

# Loading Data `.HAR` and `.SL4` 
First, load the example data using the following commands `<load_harx>` and `<load_sl4x>`:
```{r load data}
# Paths to the .har files
har_path1 <- system.file("extdata", "TAR10-WEL.har", package = "HARplus")
har_path2 <- system.file("extdata", "SUBT10-WEL.har", package = "HARplus")

# Paths to the .sl4 files
sl4_path1 <- system.file("extdata", "TAR10.sl4", package = "HARplus")
sl4_path2 <- system.file("extdata", "SUBT10.sl4", package = "HARplus")

# Load the .har files using load_harx()
har_data1 <- load_harx(har_path1)
har_data2 <- load_harx(har_path2)

# Load the .sl4 files using load_sl4x()
sl4_data1 <- load_sl4x(sl4_path1)
sl4_data2 <- load_sl4x(sl4_path2)
```


# Extracting Data

This package allows two main ways to enhance the GEMPACK user experience and improve user-friendliness when extracting data:  
1. By selecting the variable/header name using `<get_data_by_var>`.  
2. By using dimension patterns (e.g., `REG*COMM`) with `<get_data_by_dims>`.  

Both functions have similar options:  
- Extract a single variable, multiple variables, or all variables (NULL or "ALL") from the datasets.  
- Extract data from a single dataset or multiple datasets.  
- Rename the experiment (i.e., dataset) name, which is displayed in the **Experiment** column (this column is automatically added to handle multiple dataset merges). By default, the name is the dataset name.  
- Reporting data level (all, only total, only subtotals).
- Rename dimension names, which will be used as column names.  
- Merge data across multiple datasets within the same patterns.  
  - For `<get_data_by_var>`, variables with the same name (e.g., `"qo"`) from different experiments (`exp1` and `exp2`) will be merged into a single final dataframe, with the input source identified in the **Experiment** column.  
  - Similarly, for `<get_data_by_dims>`, data with matching dimension patterns (e.g., `REG*COMM`) will be merged.  

Let's start with the simplest example as follows:

```{r get_data_by_var}
# Extract data for a single variable
data_qo <- get_data_by_var("qo", sl4_data1)
print(head(data_qo[["sl4_data1"]][["qo"]], 4))

# Extract multiple variables from multiple datasets
data_multiple <- get_data_by_var(c("qo", "qgdp"), sl4_data1, sl4_data2)
print(head(data_multiple[["sl4_data1"]][["qo"]], 4))
print(head(data_multiple[["sl4_data2"]][["qo"]], 4))

# Extract all variables separately from multiple datasets
data_list <- get_data_by_var(NULL, sl4_data1, sl4_data2)
print(names(data_list))

# Extract all variables and merge the same variable from multiple datasets
data_all <- get_data_by_var(NULL, sl4_data1, sl4_data2, merge_data = TRUE)
print(names(data_all))

# Return all value levels
data_all <- get_data_by_var("qo", sl4_data1, sl4_data2, subtotal_level = TRUE)
print(head(data_all[["sl4_data1"]][["qo"]], 4))

# Return only TOTAL, drop subtotal
data_total <- get_data_by_var("qo", sl4_data1, sl4_data2, subtotal_level = FALSE)
print(head(data_total[["sl4_data1"]][["qo"]], 4))

# Return only subtotal, drop TOTAL (result is empty if there in subtotal)
data_decomp <- get_data_by_var("qo", sl4_data1, sl4_data2, subtotal_level = "decomposed")
print(head(data_decomp[["sl4_data2"]][["qo"]], 4))

# Rename specific columns
data_col_renamed <- get_data_by_var("qo", sl4_data1, 
                             rename_cols = c(REG = "Region", COMM = "Commodity"))
str(data_col_renamed)

# Rename experiment names
data_exp_renamed <- get_data_by_var("qo", sl4_data1, sl4_data2, 
                             experiment_names = c("EXP1", "EXP2"))
print(names(data_exp_renamed))

# Merge variable data across multiple datasets with custom experiment names
data_merged <- get_data_by_var(, sl4_data1, sl4_data2,
                            experiment_names = c("EXP1", "EXP2"), 
                            merge_data = TRUE,
                            rename_cols = c(REG = "Region", COMM = "Commodity"))
print(head(data_merged$merged[[1]], 4))

```

Now, let's delve into `<get_data_by_dims>`. This command offers an additional feature compared to `<get_data_by_var>`: it allows for `patter_mix`. For instance, REG*COMM and COMM*REG can be treated as equivalent when merging data (this only applies if merge_data = TRUE). You can experiment with the following commands:

```{r get_data_by_dims}
# Merge data by dimensions (e.g., REG*COMM != COMM*REG)
data_no_mix <- get_data_by_dims(NULL, sl4_data1, sl4_data2, 
                                merge_data = TRUE, 
                                pattern_mix = FALSE)

# Merge data while allowing interchangeable dimensions (e.g., REG*COMM = COMM*REG)
data_pattern_mixed <- get_data_by_dims(NULL, sl4_data1, sl4_data2, 
                                       merge_data = TRUE, 
                                       pattern_mix = TRUE)
```
This flexibility is particularly useful when working with datasets where dimension order does not affect the interpretation of the data.


# Grouping Data By Dimension

The `<group_data_by_dims>` function is a powerful tool that **categorizes extracted data into meaningful dimension-based groups**. This is a key feature of this package, as it allows GEMPACK users (particularly those working with GTAP model results) to merge data into a structured and useful dataframe.  

For example, if users want to retrieve all variables defined by `REG`, they can simply set `REG` as a priority while also assigning a new column name for `REG`, such as `REG = "Region"`. The function will then **merge all datasets that contain `REG` as a dimension element into a single dataframe**, ensuring that all relevant data is consolidated.

Unlike `<get_data_by_dims>`, which focuses on extracting data, `<group_data_by_dims>` **organizes and merges data dynamically** based on the structure and priority defined by the user.

It is particularly useful because:
- It **groups extracted data by dimension levels** (`1D`, `2D`, `3D`, …).
- Users can **define priority-based merging** (e.g., prioritizing `REG` first, then `COMM`). The function first attempts to merge all datasets containing `REG`, then moves on to merge datasets containing `COMM`. If a dataset contains `REG*COMM`, it will be included in the `REG` output.
- It **automatically renames dimensions** (`auto_rename = TRUE`) to improve dataset compatibility while preserving all variable information.
- It **transforms data into a structured long format**, simplifying further analysis.
- It **supports merging grouped data across multiple datasets**.
- It **filters subtotal levels**, allowing users to retain `"TOTAL"` values or decomposed components as needed.
- If certain datasets cannot be merged, the function generates a **detailed report identifying the root of the issue**, enabling users to manually manipulate the data if necessary.
- This function primarily focuses on **1D and 2D data**, ensuring that **higher-dimension data (>2D) is merged only with datasets that share the same pattern**.


## **Understanding Priority List in `group_data_by_dims`**

The **priority list** in `<group_data_by_dims>` allows users to control **how dimensions are grouped and merged**.  
By defining priorities, the function ensures that **datasets containing high-priority dimensions are merged as much as possible**, but **only if they share the same structure**.  

### **1. Single Priority Example (`REG` only)**
If we prioritize only `REG`, the function will **attempt to merge all datasets containing `REG` as much as possible**, while ensuring that datasets with different structures remain separate.

```{r group_data_single_priority}
# Define single priority (Only Region-based grouping)
priority_list <- list("Region" = c("REG"))

# Grouping data with a single priority
grouped_data_single <- group_data_by_dims("ALL", sl4_data1, sl4_data2,
                                          priority = priority_list, auto_rename = TRUE)

# Print structure
print(names(grouped_data_single))
print(names(grouped_data_single[["1D"]]))
print(names(grouped_data_single[["2D"]]))
```
What happens here?
- The function tries to merge all datasets containing REG, but only if they share the same data structure.
- If some datasets have different structures, they will remain separate.
- The merging happens separately in each data dimension (e.g., 1D, 2D, etc.).


### **2. Multiple Priorities Example (`COMM` before `REG`)**

If we prioritize **`COMM` first and `REG` second**, datasets containing `COMM` will be **merged first**, followed by `REG`, **as much as possible** while ensuring datasets with different structures remain separate.
```{r group_data_multi_priority}
# Define multiple priority levels: Sector first, then Region
priority_list <- list(
  "Sector" = c("COMM", "ACTS"),
  "Region" = c("REG")
)

# Grouping data with multiple priorities
grouped_data_multiple <- group_data_by_dims("ALL", sl4_data1, 
                                            priority = priority_list, 
                                            auto_rename = TRUE)

# Print structure
print(names(grouped_data_multiple))
print(names(grouped_data_multiple[["1D"]]))
print(names(grouped_data_multiple[["2D"]]))
```
What happens here?  
- The function attempts to merge all datasets containing `COMM` and `ACTS` first, both of which are merged into the **Sector** column before attempting to merge other datasets with `REG`, as long as they share the same structure.  
- Therefore, if datasets contain `REG*COMM` and `REG*ACTS`, they will be merged into the **Sector** dataframe, **not the REG dataframe**.
- If datasets cannot be merged due to structural differences, they are kept separate in their respective dimension groups.


## **Understanding `auto_rename` in `group_data_by_dims`**

The `auto_rename` option in `<group_data_by_dims>` plays a crucial role in ensuring **successful and smooth merging** of datasets.  
When enabled, it **automatically renames lower-priority dimensions** to `"Dim1"`, `"Dim2"`, etc., which allows datasets with slightly different dimension names to be merged, while still preserving the original dimension structure.  

Without `auto_rename`, datasets with similar but non-identical dimension names **may fail to merge**, leading to separate outputs instead of a unified dataset.

### **Example: Grouping Without `auto_rename`**
By default (`auto_rename = FALSE`), the function **keeps all original dimension names**.  
This means datasets that contain **similar but not identical dimensions** (e.g., `REG*ENDW` and `REG*END`) **cannot be merged**.

```{r group_data_no_auto_rename}
# Define priority: First by Sector (COMM, ACTS), then by Region (REG)
priority_list <- list(
  "Sector" = c("COMM", "ACTS"),
  "Region" = c("REG")
)

# Grouping data without auto_rename
grouped_data_no_rename <- group_data_by_dims("ALL", sl4_data1, 
                                             priority = priority_list, 
                                             auto_rename = FALSE)

# Print structure
print(names(grouped_data_no_rename))
print(names(grouped_data_no_rename[["1D"]]))
print(names(grouped_data_no_rename[["2D"]]))
```
Without `auto_rename`, the number of mergeable dataframes will be **less** compared to when `auto_rename = TRUE`, as it **does not allow merging across different dimension names**, even if their structures are otherwise compatible.

# Reshaping and Renaming Data
## Pivoting Data
The `<pivot_data>` function transforms long-format data from SL4 or HAR objects into a wide format, making it more suitable for analysis and visualization. This transformation is particularly useful when working with GEMPACK outputs that need to be reshaped for reporting or further processing.

```{r pivot}
# Extract data to pivot
data_multiple <- get_data_by_var(c("qo", "pca"), sl4_data1)

# Pivot a single column
pivoted_single <- pivot_data(data_multiple, 
                             pivot_cols = "REG")

# Pivot multiple columns
pivoted_multi <- pivot_data(data_multiple, 
                            pivot_cols = c("COMM", "REG"))

```

## Pivot Data Hierarchy

While `pivot_data` provides basic pivoting functionality, `pivot_data_hierarchy` offers enhanced capabilities for creating hierarchical pivot tables similar to those found in spreadsheet applications. This function is particularly useful when you need to maintain dimensional hierarchies in your output or create Excel-ready pivot tables.

Key differences from regular pivoting:

- Maintains hierarchical relationships between pivot columns
- Supports direct Excel export with properly formatted headers
- Preserves dimension order as specified in pivot_cols
- Creates nested column headers reflecting the hierarchy
- **Must be exported to XLSX only using the command option `<export = TRUE, file_path = "./path/to/output">`. This pivot table cannot be exported with the <export> command.**

```{r pivot_data_hierarchy, eval=FALSE}
# Create hierarchical pivot without export
pivot_hier <- pivot_data_hierarchy(data_multiple, 
                                  pivot_cols = c("REG", "COMM"))

# Create and export to Excel in one step
pivot_export <- pivot_data_hierarchy(data_multiple, 
                                   pivot_cols = c("REG", "COMM"),
                                   export = TRUE,
                                   file_path = file.path(tempdir(), "pivot_output.xlsx"))

```
The hierarchy in the resulting pivot table follows the order specified in `pivot_cols`. For example, when using `c("REG", "COMM")`, the output will show:

- First level: Region (REG)  
- Second level: Commodity (COMM) nested under each region
- Values arranged according to this hierarchy

When exported to Excel, the hierarchical structure is automatically formatted with:

- Properly aligned multi-level column headers
- Bold formatting for header levels  
- Clean sheet names for multiple variables

## Rename Dimensions

The `rename_dims` function provides flexible renaming capabilities for dimensions in SL4 or HAR objects. You can rename either dimension names, list names, or both.

```{r rename}
# Define a renaming map
mapping_df <- data.frame(
 old = c("REG", "COMM"),
 new = c("Region", "Commodity")
)

# Rename dimensions only
renamed_dims <- rename_dims(sl4_data1, mapping_df)

# Rename both dimensions and list names
renamed_both <- rename_dims(sl4_data1, mapping_df, rename_list_names = TRUE)
```
The mapping dataframe must have two columns: the first for current names and the second for new names. The function preserves data structure while updating dimension labels according to the specified mapping.


# Saving Data to GEMPACK `.HAR` Format

This function is particularly useful when:
- Creating custom HAR files for simulation inputs or aggregated datasets  
- Converting R-based results into GEMPACK-readable format  
- Exporting modified or merged results from GTAP-style models for use in RunDynam or RunGTAP

## Key Features

- Supports both **1C (string)** and **RE (real)** headers  
- Automatically generates 1C dimension sets (e.g., `REG`, `COMM`, `ENDW`)  
- Accepts data frames or arrays with flexible column naming  
- Writes associated set headers when `export_sets = TRUE`  
- Maintains full GEMPACK binary structure with no size limitation  
- Supports up to seven dimensions and approximately 2 million elements per chunk
- Allows **dimension renaming** and supports **duplicate dimension names** (e.g., `COMMxREGxREG`) during export  

```{r save_har_single, eval=FALSE}
# Load example HAR data
har_path <- system.file("extdata", "TAR10-WEL.har", package = "HARplus")
har_data <- load_harx(har_path)

# Extract one matrix for demonstration
welfare_data <- get_data_by_var("C17", har_data)
welfare_matrix <- welfare_data[["har_data"]][["C17"]]

# Save as a single HAR file
save_har(
  data_list   = list(WELF = welfare_matrix),
  file_path   = file.path(tempdir(), "output_single.har"),
  dimensions  = list(WELF = c("REG", "COLUMN")),
  value_cols  = c(WELF = "Value"),
  long_desc   = c(WELF = "Welfare Decomposition"),
  dim_rename  = list(RTMS = c(COMM = "COMM", SREG = "REG", DREG = "REG")),
  export_sets = TRUE,
  dim_order   = NULL,
  lowercase   = FALSE
)
```


# Exporting Data to CSV/STATA/TEXT/RDS/XLSX

The `export_data` function allows you to export SL4 and HAR data into various formats. Supported formats include: `"csv"`, `"xlsx"`, `"stata"`, `"txt"`, and `"rds"`. You can export single data frames or multi-variable results while preserving their structure.

```{r export data, eval=FALSE}
# Extract data
data_multiple <- get_data_by_var(c("qo", "pca"), sl4_data1)

# Export 
export_data(data_multiple, file.path(tempdir(), "output_directory"), 
           format = c("csv", "xlsx", "stata", "txt", "rds"),
           create_subfolder = TRUE,
           multi_sheet_xlsx = TRUE)
```

# Exploring Data Structure

Since this package is designed to handle multiple inputs with a similar structure, such as simulation outputs from the GTAP model with different shocks or experiments, the first important step is to understand the data structure. This process can also be useful even with a single input, as it helps in analyzing the data shape and dimension size of each variable.  

There are a couple of commands available to illustrate the data structure, all of which can be applied to both `.har` and `.sl4` files in the same manner.  

## Variable Structure  

To get a summary of variable names, dimension counts, dimension patterns, and optionally the column and observation counts for one or multiple variables from a single or multiple datasets (return as separate lists):  

```{r get_var_structure}
# (1) Getting all variables from the input file
vars_har_sum <- get_var_structure("ALL", har_data1)
vars_sl4_sum <- get_var_structure(, har_data1)

# (2) Getting selected variables
var_sl4_sum <- get_var_structure(c("pds","pfd","pms"), sl4_data1)
print(head(var_sl4_sum[["sl4_data1"]], 4))


# (3) Including column size and number of observation in the summary
var_sl4_sum <- get_var_structure(c("pds","pfd","pms"), sl4_data1, sl4_data2, 
                                 include_col_size = TRUE)
print(head(var_sl4_sum[["sl4_data1"]], 4))
```

Understanding the data structure is crucial for aggregating data across multiple experiments (inputs). Even when using the same variables, different experiments may have varying column sizes or data structures due to factors such as subtotal effects and other experimental settings. These discrepancies can lead to errors if the merging process relies solely on variable names without accounting for structural differences.

To compare data structures across multiple experiments, use the following command:

```{r compare_var_structure}
# (1) Comparing all variable structures across experiments 
vars_comparison <- compare_var_structure(
  variables = "ALL", sl4_data1, sl4_data2
)
print(vars_comparison$match[1:2, ])

# (2) Comparing selected variable structures across experiments 
var_comparison <- compare_var_structure(
  variables = c("pds", "pms"), sl4_data1, sl4_data2
)
print(var_comparison$match[1:2, ])
```
This function returns a list containing:
- **match**: A data frame of variables with identical dimension names and structures across inputs.
- **diff** (if any): A data frame listing variables with structural mismatches. Note: If this list appears, you may need to focus on handling these variables. It is designed to report potential issue-causing variables when merging datasets.

Additionally, the function includes the `<keep_unique>` option (default: `FALSE`) which allows users to extract variables with unique names and structures across all inputs. This is particularly useful when inputs contain different sets of variables that need to be combined into a final dataset.

```{r unique_compare_var_structure}
# (3) Extracting unique variable structures
unique_vars <- compare_var_structure(, 
                                     sl4_data1, sl4_data2,
  keep_unique = TRUE
)
print(unique_vars$match[1:10, ])
```
This function returns a list containing:
- **match**: A data frame displaying distinct variable structures found across inputs.
- **diff** (if any): A data frame detailing how structures differ between inputs.

To sum up,
- `keep_unique = FALSE` → Checks whether variables match across inputs.
- `keep_unique = TRUE` → Extracts all unique variable structures, regardless of whether they match, while reports any variables that do not align.

## Dimension Structure  

The following commands can be used to retrieve unique dimension names as patterns and their corresponding elements:
```{r get_dim_patterns}
# (1) Extracting dimension patterns (e.g., REG*COMM*ACTS)
dims_strg_har <- get_dim_patterns(har_data1, har_data2)
print(dims_strg_har[1:4, ])

# (2) Extracting unique dimension patterns (e.g., REG*COMM*ACTS)
dims_strg_har <- get_dim_patterns(har_data1, har_data2,
                                  keep_unique =TRUE)
print(dims_strg_har[1:4, ])

# (2) Extracting dimension elements e.g., REG, COMM, ACTS
dims_strg_har_uniq <- get_dim_elements(har_data1, har_data2, 
                                       keep_unique =TRUE)
print(dims_strg_har_uniq[1:4, ])

```
### Patterns vs. Elements 
Patterns represent structured dimension names (e.g., `"REG*COMM*ACTS"`), while elements extract individual dimension elements (e.g., `"REG"`, `"COMM"`, `"ACTS"`).  

Enhanced Processing of GEMPACK .HAR and .SL4 Files
================
Pattawee Puangchit
2025-02-19

# Overview of HARplus

The `HARplus` package enhances GEMPACK users’ experience by streamlining
`.har` and `.sl4` file processing. It efficiently extracts, organizes,
and manages dimension structures, ensuring consistency, optimized memory
usage, and simplified data manipulation. The package enables fast data
merging, aggregation, and transformation while maintaining a consistent
interface across GEMPACK file types. Users can easily pivot data into
familiar formats and export results in various formats.

A key feature of `HARplus` is its **flexible subtotal level handling**,
allowing users to selectively retain `"TOTAL"` values, decomposed
components, or both. This ensures precise data extraction for various
economic modeling needs without unnecessary redundancy.

# Introduction

This vignette covers key functions of `HARplus`, highlighting its
ability to handle multiple inputs for efficient data processing.
Designed for economic modelers and GEMPACK practitioners, it simplifies
working with multiple datasets and enhances analytical workflows.

Sample data used in this vignette is obtained from the GTAPv7 model and
utilizes publicly available data from the GTAP 9 database. For more
details, refer to the [GTAP Database
Archive](https://www.gtap.agecon.purdue.edu/databases/archives.asp).

# Acknowledgement

The development of `HARplus` builds upon the foundational work of the
`HARr` package. I sincerely acknowledge and appreciate the contributions
of Maros Ivanic, whose `HARr` package served as the baseline for this
development, advancing efficient data handling for GEMPACK users.

# Installation and Loading Example Data

Before proceeding, ensure that `HARplus` is installed and loaded:

``` r
library(HARplus)
```

# Loading Data `.HAR` and `.SL4`

First, load the example data using the following commands `<load_harx>`
and `<load_sl4x>`:

``` r
# Paths to the .har files
har_path1 <- system.file("extdata", "TAR10-WEL.har", package = "HARplus")
har_path2 <- system.file("extdata", "SUBT10-WEL.har", package = "HARplus")

# Paths to the .sl4 files
sl4_path1 <- system.file("extdata", "TAR10.sl4", package = "HARplus")
sl4_path2 <- system.file("extdata", "SUBT10.sl4", package = "HARplus")

# Load the .har files using load_harx()
har_data1 <- load_harx(har_path1)
har_data2 <- load_harx(har_path2)

# Load the .sl4 files using load_sl4x()
sl4_data1 <- load_sl4x(sl4_path1)
sl4_data2 <- load_sl4x(sl4_path2)
```

# Extracting Data

This package allows two main ways to enhance the GEMPACK user experience
and improve user-friendliness when extracting data:  
1. By selecting the variable/header name using `<get_data_by_var>`.  
2. By using dimension patterns (e.g., `REG*COMM`) with
`<get_data_by_dims>`.

Both functions have similar options:  
- Extract a single variable, multiple variables, or all variables (NULL
or “ALL”) from the datasets.  
- Extract data from a single dataset or multiple datasets.  
- Rename the experiment (i.e., dataset) name, which is displayed in the
**Experiment** column (this column is automatically added to handle
multiple dataset merges). By default, the name is the dataset name.  
- Reporting data level (all, only total, only subtotals). - Rename
dimension names, which will be used as column names.  
- Merge data across multiple datasets within the same patterns.  
- For `<get_data_by_var>`, variables with the same name (e.g., `"qo"`)
from different experiments (`exp1` and `exp2`) will be merged into a
single final dataframe, with the input source identified in the
**Experiment** column.  
- Similarly, for `<get_data_by_dims>`, data with matching dimension
patterns (e.g., `REG*COMM`) will be merged.

Let’s start with the simplest example as follows:

``` r
# Extract data for a single variable
data_qo <- get_data_by_var("qo", sl4_data1)
print(head(data_qo[["sl4_data1"]][["qo"]], 4))
#>          ACTS     REG Subtotal     Value Variable Dimension Experiment
#> 1 GrainsCrops Oceania    TOTAL -5.283822       qo  ACTS*REG  sl4_data1
#> 2    MeatLstk Oceania    TOTAL -8.473134       qo  ACTS*REG  sl4_data1
#> 3  Extraction Oceania    TOTAL -1.617209       qo  ACTS*REG  sl4_data1
#> 4    ProcFood Oceania    TOTAL -3.570224       qo  ACTS*REG  sl4_data1

# Extract multiple variables from multiple datasets
data_multiple <- get_data_by_var(c("qo", "qgdp"), sl4_data1, sl4_data2)
print(head(data_multiple[["sl4_data1"]][["qo"]], 4))
#>          ACTS     REG Subtotal     Value Variable Dimension Experiment
#> 1 GrainsCrops Oceania    TOTAL -5.283822       qo  ACTS*REG  sl4_data1
#> 2    MeatLstk Oceania    TOTAL -8.473134       qo  ACTS*REG  sl4_data1
#> 3  Extraction Oceania    TOTAL -1.617209       qo  ACTS*REG  sl4_data1
#> 4    ProcFood Oceania    TOTAL -3.570224       qo  ACTS*REG  sl4_data1
print(head(data_multiple[["sl4_data2"]][["qo"]], 4))
#>          ACTS     REG Subtotal     Value Variable Dimension Experiment
#> 1 GrainsCrops Oceania    TOTAL -5.283822       qo  ACTS*REG  sl4_data2
#> 2    MeatLstk Oceania    TOTAL -8.473134       qo  ACTS*REG  sl4_data2
#> 3  Extraction Oceania    TOTAL -1.617209       qo  ACTS*REG  sl4_data2
#> 4    ProcFood Oceania    TOTAL -3.570224       qo  ACTS*REG  sl4_data2

# Extract all variables separately from multiple datasets
data_list <- get_data_by_var(NULL, sl4_data1, sl4_data2)
print(names(data_list))
#> [1] "sl4_data1" "sl4_data2"

# Extract all variables and merge the same variable from multiple datasets
data_all <- get_data_by_var(NULL, sl4_data1, sl4_data2, merge_data = TRUE)
print(names(data_all))
#> [1] "merged"

# Return all value levels
data_all <- get_data_by_var("qo", sl4_data1, sl4_data2, subtotal_level = TRUE)
print(head(data_all[["sl4_data1"]][["qo"]], 4))
#>          ACTS     REG Subtotal     Value Variable Dimension Experiment
#> 1 GrainsCrops Oceania    TOTAL -5.283822       qo  ACTS*REG  sl4_data1
#> 2    MeatLstk Oceania    TOTAL -8.473134       qo  ACTS*REG  sl4_data1
#> 3  Extraction Oceania    TOTAL -1.617209       qo  ACTS*REG  sl4_data1
#> 4    ProcFood Oceania    TOTAL -3.570224       qo  ACTS*REG  sl4_data1

# Return only TOTAL, drop subtotal
data_total <- get_data_by_var("qo", sl4_data1, sl4_data2, subtotal_level = FALSE)
print(head(data_total[["sl4_data1"]][["qo"]], 4))
#>          ACTS     REG Subtotal     Value Variable Dimension Experiment
#> 1 GrainsCrops Oceania    TOTAL -5.283822       qo  ACTS*REG  sl4_data1
#> 2    MeatLstk Oceania    TOTAL -8.473134       qo  ACTS*REG  sl4_data1
#> 3  Extraction Oceania    TOTAL -1.617209       qo  ACTS*REG  sl4_data1
#> 4    ProcFood Oceania    TOTAL -3.570224       qo  ACTS*REG  sl4_data1

# Return only subtotal, drop TOTAL (result is empty if there in subtotal)
data_decomp <- get_data_by_var("qo", sl4_data1, sl4_data2, subtotal_level = "decomposed")
print(head(data_decomp[["sl4_data2"]][["qo"]], 4))
#>            ACTS     REG   Subtotal Value Variable Dimension Experiment
#> 101 GrainsCrops Oceania to changes     0       qo  ACTS*REG  sl4_data2
#> 102    MeatLstk Oceania to changes     0       qo  ACTS*REG  sl4_data2
#> 103  Extraction Oceania to changes     0       qo  ACTS*REG  sl4_data2
#> 104    ProcFood Oceania to changes     0       qo  ACTS*REG  sl4_data2

# Rename specific columns
data_col_renamed <- get_data_by_var("qo", sl4_data1, 
                             rename_cols = c(REG = "Region", COMM = "Commodity"))
str(data_col_renamed)
#> List of 1
#>  $ sl4_data1:List of 1
#>   ..$ qo:'data.frame':   100 obs. of  7 variables:
#>   .. ..$ ACTS      : chr [1:100] "GrainsCrops" "MeatLstk" "Extraction" "ProcFood" ...
#>   .. ..$ Region    : chr [1:100] "Oceania" "Oceania" "Oceania" "Oceania" ...
#>   .. ..$ Subtotal  : chr [1:100] "TOTAL" "TOTAL" "TOTAL" "TOTAL" ...
#>   .. ..$ Value     : num [1:100] -5.28 -8.47 -1.62 -3.57 8.39 ...
#>   .. ..$ Variable  : chr [1:100] "qo" "qo" "qo" "qo" ...
#>   .. ..$ Dimension : chr [1:100] "ACTS*REG" "ACTS*REG" "ACTS*REG" "ACTS*REG" ...
#>   .. ..$ Experiment: chr [1:100] "sl4_data1" "sl4_data1" "sl4_data1" "sl4_data1" ...

# Rename experiment names
data_exp_renamed <- get_data_by_var("qo", sl4_data1, sl4_data2, 
                             experiment_names = c("EXP1", "EXP2"))
print(names(data_exp_renamed))
#> [1] "EXP1" "EXP2"

# Merge variable data across multiple datasets with custom experiment names
data_merged <- get_data_by_var(, sl4_data1, sl4_data2,
                            experiment_names = c("EXP1", "EXP2"), 
                            merge_data = TRUE,
                            rename_cols = c(REG = "Region", COMM = "Commodity"))
print(head(data_merged$merged[[1]], 4))
#>     Commodity  Region Subtotal    Value Variable Dimension Experiment
#> 1 GrainsCrops Oceania    TOTAL 10.34259      pds  COMM*REG       EXP1
#> 2    MeatLstk Oceania    TOTAL 11.86026      pds  COMM*REG       EXP1
#> 3  Extraction Oceania    TOTAL 11.49640      pds  COMM*REG       EXP1
#> 4    ProcFood Oceania    TOTAL 13.40175      pds  COMM*REG       EXP1
```

Now, let’s delve into `<get_data_by_dims>`. This command offers an
additional feature compared to `<get_data_by_var>`: it allows for
`patter_mix`. For instance, REG*COMM and COMM*REG can be treated as
equivalent when merging data (this only applies if merge_data = TRUE).
You can experiment with the following commands:

``` r
# Merge data by dimensions (e.g., REG*COMM != COMM*REG)
data_no_mix <- get_data_by_dims(NULL, sl4_data1, sl4_data2, 
                                merge_data = TRUE, 
                                pattern_mix = FALSE)

# Merge data while allowing interchangeable dimensions (e.g., REG*COMM = COMM*REG)
data_pattern_mixed <- get_data_by_dims(NULL, sl4_data1, sl4_data2, 
                                       merge_data = TRUE, 
                                       pattern_mix = TRUE)
```

This flexibility is particularly useful when working with datasets where
dimension order does not affect the interpretation of the data.

# Grouping Data By Dimension

The `<group_data_by_dims>` function is a powerful tool that
**categorizes extracted data into meaningful dimension-based groups**.
This is a key feature of this package, as it allows GEMPACK users
(particularly those working with GTAP model results) to merge data into
a structured and useful dataframe.

For example, if users want to retrieve all variables defined by `REG`,
they can simply set `REG` as a priority while also assigning a new
column name for `REG`, such as `REG = "Region"`. The function will then
**merge all datasets that contain `REG` as a dimension element into a
single dataframe**, ensuring that all relevant data is consolidated.

Unlike `<get_data_by_dims>`, which focuses on extracting data,
`<group_data_by_dims>` **organizes and merges data dynamically** based
on the structure and priority defined by the user.

It is particularly useful because: - It **groups extracted data by
dimension levels** (`1D`, `2D`, `3D`, …). - Users can **define
priority-based merging** (e.g., prioritizing `REG` first, then `COMM`).
The function first attempts to merge all datasets containing `REG`, then
moves on to merge datasets containing `COMM`. If a dataset contains
`REG*COMM`, it will be included in the `REG` output. - It
**automatically renames dimensions** (`auto_rename = TRUE`) to improve
dataset compatibility while preserving all variable information. - It
**transforms data into a structured long format**, simplifying further
analysis. - It **supports merging grouped data across multiple
datasets**. - It **filters subtotal levels**, allowing users to retain
`"TOTAL"` values or decomposed components as needed. - If certain
datasets cannot be merged, the function generates a **detailed report
identifying the root of the issue**, enabling users to manually
manipulate the data if necessary. - This function primarily focuses on
**1D and 2D data**, ensuring that **higher-dimension data (\>2D) is
merged only with datasets that share the same pattern**.

## **Understanding Priority List in `group_data_by_dims`**

The **priority list** in `<group_data_by_dims>` allows users to control
**how dimensions are grouped and merged**.  
By defining priorities, the function ensures that **datasets containing
high-priority dimensions are merged as much as possible**, but **only if
they share the same structure**.

### **1. Single Priority Example (`REG` only)**

If we prioritize only `REG`, the function will **attempt to merge all
datasets containing `REG` as much as possible**, while ensuring that
datasets with different structures remain separate.

``` r
# Define single priority (Only Region-based grouping)
priority_list <- list("Region" = c("REG"))

# Grouping data with a single priority
grouped_data_single <- group_data_by_dims("ALL", sl4_data1, sl4_data2,
                                          priority = priority_list, auto_rename = TRUE)

# Print structure
print(names(grouped_data_single))
#> [1] "1D" "2D" "3D" "4D"
print(names(grouped_data_single[["1D"]]))
#> [1] "Region" "Other"
print(names(grouped_data_single[["2D"]]))
#> [1] "Region"
```

What happens here? - The function tries to merge all datasets containing
REG, but only if they share the same data structure. - If some datasets
have different structures, they will remain separate. - The merging
happens separately in each data dimension (e.g., 1D, 2D, etc.).

### **2. Multiple Priorities Example (`COMM` before `REG`)**

If we prioritize **`COMM` first and `REG` second**, datasets containing
`COMM` will be **merged first**, followed by `REG`, **as much as
possible** while ensuring datasets with different structures remain
separate.

``` r
# Define multiple priority levels: Sector first, then Region
priority_list <- list(
  "Sector" = c("COMM", "ACTS"),
  "Region" = c("REG")
)

# Grouping data with multiple priorities
grouped_data_multiple <- group_data_by_dims("ALL", sl4_data1, 
                                            priority = priority_list, 
                                            auto_rename = TRUE)

# Print structure
print(names(grouped_data_multiple))
#> [1] "1D" "2D" "3D" "4D"
print(names(grouped_data_multiple[["1D"]]))
#> [1] "Sector" "Region" "Other"
print(names(grouped_data_multiple[["2D"]]))
#> [1] "Sector" "Region"
```

What happens here?  
- The function attempts to merge all datasets containing `COMM` and
`ACTS` first, both of which are merged into the **Sector** column before
attempting to merge other datasets with `REG`, as long as they share the
same structure.  
- Therefore, if datasets contain `REG*COMM` and `REG*ACTS`, they will be
merged into the **Sector** dataframe, **not the REG dataframe**. - If
datasets cannot be merged due to structural differences, they are kept
separate in their respective dimension groups.

## **Understanding `auto_rename` in `group_data_by_dims`**

The `auto_rename` option in `<group_data_by_dims>` plays a crucial role
in ensuring **successful and smooth merging** of datasets.  
When enabled, it **automatically renames lower-priority dimensions** to
`"Dim1"`, `"Dim2"`, etc., which allows datasets with slightly different
dimension names to be merged, while still preserving the original
dimension structure.

Without `auto_rename`, datasets with similar but non-identical dimension
names **may fail to merge**, leading to separate outputs instead of a
unified dataset.

### **Example: Grouping Without `auto_rename`**

By default (`auto_rename = FALSE`), the function **keeps all original
dimension names**.  
This means datasets that contain **similar but not identical
dimensions** (e.g., `REG*ENDW` and `REG*END`) **cannot be merged**.

``` r
# Define priority: First by Sector (COMM, ACTS), then by Region (REG)
priority_list <- list(
  "Sector" = c("COMM", "ACTS"),
  "Region" = c("REG")
)

# Grouping data without auto_rename
grouped_data_no_rename <- group_data_by_dims("ALL", sl4_data1, 
                                             priority = priority_list, 
                                             auto_rename = FALSE)

# Print structure
print(names(grouped_data_no_rename))
#> [1] "1D"     "2D"     "3D"     "4D"     "report"
print(names(grouped_data_no_rename[["1D"]]))
#> [1] "Sector"   "Region"   "unmerged"
print(names(grouped_data_no_rename[["2D"]]))
#> [1] "Sector"   "unmerged"
```

Without `auto_rename`, the number of mergeable dataframes will be
**less** compared to when `auto_rename = TRUE`, as it **does not allow
merging across different dimension names**, even if their structures are
otherwise compatible.

# Reshaping and Renaming Data

## Pivoting Data

The `<pivot_data>` function transforms long-format data from SL4 or HAR
objects into a wide format, making it more suitable for analysis and
visualization. This transformation is particularly useful when working
with GEMPACK outputs that need to be reshaped for reporting or further
processing.

``` r
# Extract data to pivot
data_multiple <- get_data_by_var(c("qo", "pca"), sl4_data1)

# Pivot a single column
pivoted_single <- pivot_data(data_multiple, 
                             pivot_cols = "REG")

# Pivot multiple columns
pivoted_multi <- pivot_data(data_multiple, 
                            pivot_cols = c("COMM", "REG"))
```

## Pivot Data Hierarchy

While `pivot_data` provides basic pivoting functionality,
`pivot_data_hierarchy` offers enhanced capabilities for creating
hierarchical pivot tables similar to those found in spreadsheet
applications. This function is particularly useful when you need to
maintain dimensional hierarchies in your output or create Excel-ready
pivot tables.

Key differences from regular pivoting:

- Maintains hierarchical relationships between pivot columns
- Supports direct Excel export with properly formatted headers
- Preserves dimension order as specified in pivot_cols
- Creates nested column headers reflecting the hierarchy
- **Must be exported to XLSX only using the command option
  `<export = TRUE, file_path = "./path/to/output">`. This pivot table
  cannot be exported with the <export> command.**

``` r
# Create hierarchical pivot without export
pivot_hier <- pivot_data_hierarchy(data_multiple, 
                                  pivot_cols = c("REG", "COMM"))

# Create and export to Excel in one step
pivot_export <- pivot_data_hierarchy(data_multiple, 
                                   pivot_cols = c("REG", "COMM"),
                                   export = TRUE,
                                   file_path = file.path(tempdir(), "pivot_output.xlsx"))
```

The hierarchy in the resulting pivot table follows the order specified
in `pivot_cols`. For example, when using `c("REG", "COMM")`, the output
will show:

- First level: Region (REG)  
- Second level: Commodity (COMM) nested under each region
- Values arranged according to this hierarchy

When exported to Excel, the hierarchical structure is automatically
formatted with:

- Properly aligned multi-level column headers
- Bold formatting for header levels  
- Clean sheet names for multiple variables

## Rename Dimensions

The `rename_dims` function provides flexible renaming capabilities for
dimensions in SL4 or HAR objects. You can rename either dimension names,
list names, or both.

``` r
# Define a renaming map
mapping_df <- data.frame(
 old = c("REG", "COMM"),
 new = c("Region", "Commodity")
)

# Rename dimensions only
renamed_dims <- rename_dims(sl4_data1, mapping_df)

# Rename both dimensions and list names
renamed_both <- rename_dims(sl4_data1, mapping_df, rename_list_names = TRUE)
```

The mapping dataframe must have two columns: the first for current names
and the second for new names. The function preserves data structure
while updating dimension labels according to the specified mapping.

# Exporting Data to CSV/STATA/TEXT/RDS/XLSX

The `export_data` function allows you to export SL4 and HAR data into
various formats. Supported formats include: `"csv"`, `"xlsx"`,
`"stata"`, `"txt"`, and `"rds"`. You can export single data frames or
multi-variable results while preserving their structure.

``` r
# Extract data
data_multiple <- get_data_by_var(c("qo", "pca"), sl4_data1)

# Export 
export_data(data_multiple, file.path(tempdir(), "output_directory"), 
           format = c("csv", "xlsx", "stata", "txt", "rds"),
           create_subfolder = TRUE,
           multi_sheet_xlsx = TRUE)
```

# Exploring Data Structure

Since this package is designed to handle multiple inputs with a similar
structure, such as simulation outputs from the GTAP model with different
shocks or experiments, the first important step is to understand the
data structure. This process can also be useful even with a single
input, as it helps in analyzing the data shape and dimension size of
each variable.

There are a couple of commands available to illustrate the data
structure, all of which can be applied to both `.har` and `.sl4` files
in the same manner.

## Variable Structure

To get a summary of variable names, dimension counts, dimension
patterns, and optionally the column and observation counts for one or
multiple variables from a single or multiple datasets (return as
separate lists):

``` r
# (1) Getting all variables from the input file
vars_har_sum <- get_var_structure("ALL", har_data1)
vars_sl4_sum <- get_var_structure(, har_data1)

# (2) Getting selected variables
var_sl4_sum <- get_var_structure(c("pds","pfd","pms"), sl4_data1)
print(head(var_sl4_sum[["sl4_data1"]], 4))
#>   Variable    Dimensions DimSize DataShape
#> 1      pds      COMM*REG       2     10x10
#> 2      pfd COMM*ACTS*REG       3  10x10x10
#> 3      pms      COMM*REG       2     10x10


# (3) Including column size and number of observation in the summary
var_sl4_sum <- get_var_structure(c("pds","pfd","pms"), sl4_data1, sl4_data2, 
                                 include_col_size = TRUE)
print(head(var_sl4_sum[["sl4_data1"]], 4))
#>   Variable    Dimensions DimSize DataShape No.Col No.Obs
#> 1      pds      COMM*REG       2     10x10     10     10
#> 2      pfd COMM*ACTS*REG       3  10x10x10    100     10
#> 3      pms      COMM*REG       2     10x10     10     10
```

Understanding the data structure is crucial for aggregating data across
multiple experiments (inputs). Even when using the same variables,
different experiments may have varying column sizes or data structures
due to factors such as subtotal effects and other experimental settings.
These discrepancies can lead to errors if the merging process relies
solely on variable names without accounting for structural differences.

To compare data structures across multiple experiments, use the
following command:

``` r
# (1) Comparing all variable structures across experiments 
vars_comparison <- compare_var_structure(
  variables = "ALL", sl4_data1, sl4_data2
)
print(vars_comparison$match[1:2, ])
#>   Variable    Dimensions DataShape input1_ColSize input2_ColSize
#> 1      afa COMM*ACTS*REG  10x10x10            100            100
#> 2    afall COMM*ACTS*REG  10x10x10            100            100

# (2) Comparing selected variable structures across experiments 
var_comparison <- compare_var_structure(
  variables = c("pds", "pms"), sl4_data1, sl4_data2
)
print(var_comparison$match[1:2, ])
#>   Variable Dimensions DataShape input1_ColSize input2_ColSize
#> 1      pds   COMM*REG     10x10             10             10
#> 2      pms   COMM*REG     10x10             10             10
```

This function returns a list containing: - **match**: A data frame of
variables with identical dimension names and structures across inputs. -
**diff** (if any): A data frame listing variables with structural
mismatches. Note: If this list appears, you may need to focus on
handling these variables. It is designed to report potential
issue-causing variables when merging datasets.

Additionally, the function includes the `<keep_unique>` option (default:
`FALSE`) which allows users to extract variables with unique names and
structures across all inputs. This is particularly useful when inputs
contain different sets of variables that need to be combined into a
final dataset.

``` r
# (3) Extracting unique variable structures
unique_vars <- compare_var_structure(, 
                                     sl4_data1, sl4_data2,
  keep_unique = TRUE
)
print(unique_vars$match[1:10, ])
#>    Variable    Dimensions DimSize DataShape
#> 1       afa COMM*ACTS*REG       3  10x10x10
#> 2     afall COMM*ACTS*REG       3  10x10x10
#> 3     afcom          COMM       1        10
#> 4       afe ENDW*ACTS*REG       3   5x10x10
#> 5    afeall ENDW*ACTS*REG       3   5x10x10
#> 6    afecom          ENDW       1         5
#> 7    afereg           REG       1        10
#> 8    afesec          ACTS       1        10
#> 9     afreg           REG       1        10
#> 10    afsec          ACTS       1        10
```

This function returns a list containing: - **match**: A data frame
displaying distinct variable structures found across inputs. - **diff**
(if any): A data frame detailing how structures differ between inputs.

To sum up, - `keep_unique = FALSE` → Checks whether variables match
across inputs. - `keep_unique = TRUE` → Extracts all unique variable
structures, regardless of whether they match, while reports any
variables that do not align.

## Dimension Structure

The following commands can be used to retrieve unique dimension names as
patterns and their corresponding elements:

``` r
# (1) Extracting dimension patterns (e.g., REG*COMM*ACTS)
dims_strg_har <- get_dim_patterns(har_data1, har_data2)
print(dims_strg_har[1:4, ])
#> [1] "scalar"     "scalar"     "scalar"     "REG*COLUMN"

# (2) Extracting unique dimension patterns (e.g., REG*COMM*ACTS)
dims_strg_har <- get_dim_patterns(har_data1, har_data2,
                                  keep_unique =TRUE)
print(dims_strg_har[1:4, ])
#> [1] "scalar"       "REG*COLUMN"   "ALLOCEFF*REG" "REG*CTAX"

# (2) Extracting dimension elements e.g., REG, COMM, ACTS
dims_strg_har_uniq <- get_dim_elements(har_data1, har_data2, 
                                       keep_unique =TRUE)
print(dims_strg_har_uniq[1:4, ])
#> [1] "REG"      "COLUMN"   "ALLOCEFF" "CTAX"
```

### Patterns vs. Elements

Patterns represent structured dimension names (e.g., `"REG*COMM*ACTS"`),
while elements extract individual dimension elements (e.g., `"REG"`,
`"COMM"`, `"ACTS"`).

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_data.R
\name{group_data_by_dims}
\alias{group_data_by_dims}
\title{Group Data by Dimension Patterns in SL4 or HAR Objects}
\usage{
group_data_by_dims(
  patterns = NULL,
  ...,
  priority,
  rename_cols = NULL,
  experiment_names = NULL,
  subtotal_level = FALSE,
  auto_rename = FALSE
)
}
\arguments{
\item{patterns}{Character vector. Dimension patterns to extract. Use \code{"ALL"} or \code{NULL} to extract all available patterns.}

\item{...}{One or more SL4 or HAR objects loaded using \code{load_sl4x()} or \code{load_harx()}.}

\item{priority}{Named list. Specifies priority dimension elements (\code{c("group_name" = c("dim1", "dim2"))}).}

\item{rename_cols}{Named vector. Column name replacements (\code{c("old_name" = "new_name")}).}

\item{experiment_names}{Character vector. Names assigned to each dataset. If \code{NULL}, names are inferred.}

\item{subtotal_level}{Character or logical. Determines which decomposition levels to retain:
\itemize{
\item \code{"total"}: Keeps only \code{"TOTAL"} values.
\item \code{"decomposed"}: Keeps only decomposed values (excludes \code{"TOTAL"}).
\item \code{"all"}: Keeps all rows.
\item \code{TRUE}: Equivalent to \code{"all"}, retaining both \code{"TOTAL"} and decomposed values.
\item \code{FALSE}: Equivalent to \code{"total"}, keeping only \code{"TOTAL"} values.
}}

\item{auto_rename}{Logical. If \code{TRUE}, automatically renames dimensions for consistency. Default is \code{FALSE}.}
}
\value{
A structured list of grouped data:
\itemize{
\item A named list where each element corresponds to a dimension size group (e.g., "2D", "3D").
\item Each group contains dimension-grouped data based on priority rules.
\item If unmerged data exists, includes a report attribute detailing merge issues.
}
}
\description{
Groups extracted SL4 or HAR data based on specified dimension structures and priority rules.
Supports automatic renaming, merging, subtotal filtering, and structured metadata handling.
}
\details{
\itemize{
\item Groups extracted variables based on dimension elements.
\item Applies predefined priority rules to structure the data.
\item Allows automatic renaming of dimensions (\code{auto_rename = TRUE}).
\item Supports merging of grouped data across multiple experiments.
\item Handles subtotal filtering (\code{subtotal_level}), controlling whether \code{"TOTAL"} or decomposed values are retained.
}
}
\examples{
# Import sample data
sl4_data1 <- load_sl4x(system.file("extdata", "TAR10.sl4", package = "HARplus"))
sl4_data2 <- load_sl4x(system.file("extdata", "SUBT10.sl4", package = "HARplus"))

# Case 1: Multiple priority levels (Sector then Region) with auto_rename
priority_list <- list(
  "Sector" = c("COMM", "ACTS"),
  "Region" = c("REG")
)
grouped_data_multiple <- group_data_by_dims(
  patterns = "ALL",
  sl4_data1,
  priority = priority_list,
  auto_rename = TRUE
)

# Case 2: Single priority (Region only) with auto_rename
priority_list <- list("Region" = c("REG"))
grouped_data_single <- group_data_by_dims(
  patterns = "ALL",
  sl4_data1, sl4_data2,
  priority = priority_list,
  auto_rename = TRUE
)

# Case 3: Multiple priorities without auto_rename
priority_list <- list(
  "Sector" = c("COMM", "ACTS"),
  "Region" = c("REG")
)
grouped_data_no_rename <- group_data_by_dims(
  patterns = "ALL",
  sl4_data1,
  priority = priority_list,
  auto_rename = FALSE
)
}
\seealso{
\code{\link{get_data_by_dims}}, \code{\link{get_data_by_var}}, \code{\link{load_sl4x}}, \code{\link{load_harx}}
}
\author{
Pattawee Puangchit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_data.R
\name{get_data_by_var}
\alias{get_data_by_var}
\title{Extract Variable Data from SL4 or HAR Objects}
\usage{
get_data_by_var(
  var_names = NULL,
  ...,
  experiment_names = NULL,
  subtotal_level = FALSE,
  rename_cols = NULL,
  merge_data = FALSE
)
}
\arguments{
\item{var_names}{Character vector. Variable names to extract. Use \code{"ALL"} or \code{NULL} to extract all available variables.}

\item{...}{One or more SL4 or HAR data objects loaded using \code{load_sl4x()} or \code{load_harx()}.}

\item{experiment_names}{Character vector. Names assigned to each dataset. If \code{NULL}, names are inferred.}

\item{subtotal_level}{Character or logical. Determines which decomposition levels to retain:
\itemize{
\item \code{"total"}: Keeps only \code{"TOTAL"} values.
\item \code{"decomposed"}: Keeps only decomposed values (excludes \code{"TOTAL"}).
\item \code{"all"}: Keeps all rows.
\item \code{TRUE}: Equivalent to \code{"all"}, retaining both \code{"TOTAL"} and decomposed values.
\item \code{FALSE}: Equivalent to \code{"total"}, keeping only \code{"TOTAL"} values.
}}

\item{rename_cols}{Named vector. Column name replacements (\code{c("old_name" = "new_name")}).}

\item{merge_data}{Logical. If \code{TRUE}, attempts to merge data across multiple experiments. Default is \code{FALSE}.}
}
\value{
A list of structured data:
\itemize{
\item If \code{merge_data = FALSE}, returns a named list where each element corresponds to an experiment.
\item If \code{merge_data = TRUE}, returns a named list of all merged data
}
}
\description{
Extracts structured data for one or more variables from SL4 or HAR objects,
transforming array-like data into a tidy format.
}
\details{
\itemize{
\item Retrieves specific variables, multiple variables, or all available variables from SL4 or HAR datasets.
\item Supports merging data from multiple experiments (\code{merge_data = TRUE}).
\item Allows renaming of column names (\code{rename_cols}).
\item Handles subtotal filtering (\code{subtotal_level}), controlling whether \code{"TOTAL"} or decomposed values are retained.
}
}
\examples{
# Import sample data:
sl4_data <- load_sl4x(system.file("extdata", "TAR10.sl4", package = "HARplus"))
sl4_data1 <- load_sl4x(system.file("extdata", "SUBT10.sl4", package = "HARplus"))

# Extract a single variable
data_qo <- get_data_by_var("qo", sl4_data)

# Extract multiple variables
data_multiple <- get_data_by_var(c("qo", "qgdp"), sl4_data)

# Extract all variables separately from multiple datasets
data_all <- get_data_by_var(NULL, sl4_data, sl4_data1, merge_data = FALSE)

# Merge variable data across multiple datasets
data_merged <- get_data_by_var(NULL, sl4_data, sl4_data1, merge_data = TRUE)

# Retain only "TOTAL" values, removing decomposed components (subtotal_level = "total" or FALSE)
data_total_only <- get_data_by_var("qo", sl4_data, subtotal_level = "total")
data_total_only_alt <- get_data_by_var("qo", sl4_data, subtotal_level = FALSE)

# Retain only decomposed components, removing "TOTAL" (subtotal_level = "decomposed")
data_decomposed_only <- get_data_by_var("qo", sl4_data, subtotal_level = "decomposed")

# Retain all value levels (subtotal_level = "all" or TRUE)
data_all_decomp <- get_data_by_var("qo", sl4_data, subtotal_level = "all")
data_all_decomp_alt <- get_data_by_var("qo", sl4_data, subtotal_level = TRUE)

# Rename specific columns
data_renamed <- get_data_by_var("qo", sl4_data, rename_cols = c(REG = "Region", COMM = "Commodity"))

# Merge data across multiple datasets with custom experiment names
data_merged_experiments <- get_data_by_var("qo", sl4_data, sl4_data1,
experiment_names = c("EXP1", "EXP2"),
merge_data = TRUE)

}
\seealso{
\code{\link{get_data_by_dims}}, , \code{\link{group_data_by_dims}}, \code{\link{load_sl4x}}, \code{\link{load_harx}}
}
\author{
Pattawee Puangchit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_summary.R
\name{compare_var_structure}
\alias{compare_var_structure}
\title{Compare Variable Structures Across SL4 and HAR Objects}
\usage{
compare_var_structure(variables = NULL, ..., keep_unique = FALSE)
}
\arguments{
\item{variables}{Character vector. Variable names to compare. Use \code{NULL} or \code{"ALL"} to compare all variables.}

\item{...}{Named SL4 or HAR objects to compare.}

\item{keep_unique}{Logical. If \code{TRUE}, returns unique variable structures across datasets instead of checking for compatibility. Default is \code{FALSE}.}
}
\value{
A list containing:
\itemize{
\item \code{match}: A data frame listing variables with identical structures across datasets.
\item \code{diff}: A data frame listing variables with mismatched structures, useful for debugging and alignment.
\item If \code{keep_unique = TRUE}, instead of \code{match} and \code{diff}, returns a data frame with distinct variable structures across datasets.
}
}
\description{
Compares variable structures across multiple SL4 and HAR datasets to ensure compatibility.
Identifies matching and mismatched variable structures, helping users diagnose inconsistencies.
}
\details{
\itemize{
\item Verifies whether variables have consistent structures across multiple datasets.
\item Ensures correct ordering of dimensions and checks for structural compatibility.
\item If \code{keep_unique = TRUE}, returns a list of unique variable structures instead of performing a direct comparison.
\item Useful for merging or aligning datasets before further processing.
\item Helps detect differences in variable dimensions, which may arise due to model updates or dataset variations.
}
}
\examples{
# Import sample data:
har_data1 <- load_harx(system.file("extdata", "TAR10-WEL.har", package = "HARplus"))
har_data2 <- load_harx(system.file("extdata", "SUBT10-WEL.har", package = "HARplus"))

# Compare structure for a single variable across multiple datasets
compare_var_structure("A", har_data1, har_data2)

# Compare structure for multiple variables across multiple datasets
comparison_multiple <- compare_var_structure(c("A", "E1"), har_data1, har_data2)

# Extract unique variable structures across multiple datasets
unique_vars <- compare_var_structure("ALL", har_data1, har_data2, keep_unique = TRUE)

}
\seealso{
\code{\link{get_var_structure}}, \code{\link{get_dim_patterns}}, \code{\link{get_dim_elements}}
}
\author{
Pattawee Puangchit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/save_har.R
\name{save_har}
\alias{save_har}
\title{Save Data to GEMPACK HAR Format}
\usage{
save_har(
  data_list,
  file_path,
  dimensions = NULL,
  value_cols = NULL,
  header_type = NULL,
  mappings = NULL,
  long_desc = NULL,
  coefficients = NULL,
  export_sets = TRUE,
  lowercase = TRUE,
  dim_order = NULL,
  dim_rename = NULL,
  force_sparse = NULL,
  max_chunk = 2e+06
)
}
\arguments{
\item{data_list}{Named list of objects to write. List names are used as HAR
header names after conversion to uppercase and truncation to four characters.}

\item{file_path}{Character string giving the output HAR file path.}

\item{dimensions}{Optional named list. For data-frame inputs, each element
gives the columns used as array dimensions.}

\item{value_cols}{Optional named list or named character vector. For
data-frame inputs, each element gives the numeric value column. Defaults to
\code{"Value"} when omitted.}

\item{header_type}{Optional named list or named character vector giving
explicit header roles. Accepted values are \code{"auto"}, \code{"set"},
\code{"mapping"}, \code{"real"}, \code{"sparse"}, and \code{"integer"}.}

\item{mappings}{Optional named list. Each element must be
\code{c(source_set, destination_set)} for the corresponding mapping header.}

\item{long_desc}{Optional named list or named character vector of long header
descriptions.}

\item{coefficients}{Optional named list or named character vector of
coefficient names for numeric headers.}

\item{export_sets}{Logical. If \code{TRUE}, dimension sets from numeric
arrays are written as character headers unless already supplied. Default is
\code{TRUE}.}

\item{lowercase}{Logical. If \code{TRUE}, character elements and dimension
values are converted to lowercase during data-frame and ordering
processing. Default is \code{TRUE}.}

\item{dim_order}{Optional dimension-ordering specification. Can be
\code{NULL}, a data frame, a named list, or a path to a CSV or Excel file.}

\item{dim_rename}{Optional named list for renaming array dimensions in the
HAR output.}

\item{force_sparse}{Optional character vector of headers to write in sparse
numeric format.}

\item{max_chunk}{Integer. Maximum number of elements per dense numeric data
chunk. Default is \code{2e6}.}
}
\value{
Invisibly returns a list containing the output path, written headers,
and counts of set, data, and mapping headers.
}
\description{
Writes a named list of R objects to a GEMPACK-compatible HAR file, including
character sets, mapping vectors, integer matrices, numeric arrays, sparse
numeric arrays, and data frames reshaped to arrays.
}
\examples{
# Example 1: Save one numeric data frame
REG <- c("USA", "EU", "ROW")
COLUMN <- c("alloc_A1", "tot_E1")
WELF <- expand.grid(REG = REG, COLUMN = COLUMN, stringsAsFactors = FALSE)
WELF$Value <- seq_len(nrow(WELF))

save_har(
  data_list = list(WELF = WELF),
  file_path = file.path(tempdir(), "output_single.har"),
  dimensions = list(WELF = c("REG", "COLUMN")),
  value_cols = list(WELF = "Value"),
  long_desc = list(WELF = "Welfare Decomposition"),
  coefficients = list(WELF = "WELF"),
  export_sets = TRUE,
  lowercase = FALSE
)

# Example 2: Save multiple numeric data frames
DECOM <- expand.grid(REG = REG, ALLOCEFF = c("A1", "A2"), stringsAsFactors = FALSE)
DECOM$Value <- seq_len(nrow(DECOM))

save_har(
  data_list = list(WELF = WELF, DECOM = DECOM),
  file_path = file.path(tempdir(), "output_multi.har"),
  dimensions = list(
    WELF = c("REG", "COLUMN"),
    DECOM = c("REG", "ALLOCEFF")
  ),
  value_cols = list(
    WELF = "Value",
    DECOM = "Value"
  ),
  long_desc = list(
    WELF = "Welfare Decomposition",
    DECOM = "Allocative efficiency effect"
  ),
  coefficients = list(
    WELF = "WELF",
    DECOM = "DECOM"
  ),
  export_sets = TRUE,
  lowercase = FALSE
)

# Example 3: Save a mapping vector
SC <- c("AF", "AP", "BA")
GSEC <- c("OCR", "V_F", "GRO")
MASC <- c(AF = "OCR", AP = "V_F", BA = "GRO")

save_har(
  data_list = list(SC = SC, GSEC = GSEC, MASC = MASC),
  file_path = file.path(tempdir(), "mapping.har"),
  mappings = list(MASC = c("SC", "GSEC")),
  long_desc = list(
    SC = "Set SC",
    GSEC = "Set GSEC",
    MASC = "Mapping SC to GSEC"
  ),
  export_sets = FALSE,
  lowercase = FALSE
)

# Example 4: Save mixed headers
CODE <- matrix(as.integer(c(1, 2, 3, 4)), nrow = 2)
TAX <- expand.grid(SC = SC, REG = REG, stringsAsFactors = FALSE)
TAX$Value <- c(0, 0, 0, 1.2, 0, 0, 0, 0, 2.5)

save_har(
  data_list = list(
    WELF = WELF,
    REG = REG,
    SC = SC,
    GSEC = GSEC,
    MASC = MASC,
    TAX = TAX,
    CODE = CODE
  ),
  file_path = file.path(tempdir(), "output_mixed.har"),
  dimensions = list(
    WELF = c("REG", "COLUMN"),
    TAX = c("SC", "REG")
  ),
  value_cols = list(
    WELF = "Value",
    TAX = "Value"
  ),
  mappings = list(
    MASC = c("SC", "GSEC")
  ),
  long_desc = list(
    WELF = "Welfare Decomposition",
    REG = "Set REG",
    SC = "Set SC",
    GSEC = "Set GSEC",
    MASC = "Mapping SC to GSEC",
    TAX = "Sparse tax example",
    CODE = "Integer matrix example"
  ),
  coefficients = list(
    WELF = "WELF",
    TAX = "TAX"
  ),
  force_sparse = "TAX",
  export_sets = FALSE,
  lowercase = FALSE
)

# Example 5: Apply custom dimension ordering
dim_order <- list(
  REG = c("ROW", "USA", "EU"),
  COLUMN = c("tot_E1", "alloc_A1")
)

save_har(
  data_list = list(WELF = WELF),
  file_path = file.path(tempdir(), "output_sorted.har"),
  dimensions = list(WELF = c("REG", "COLUMN")),
  value_cols = list(WELF = "Value"),
  long_desc = list(WELF = "Welfare Decomposition"),
  coefficients = list(WELF = "WELF"),
  export_sets = TRUE,
  dim_order = dim_order,
  lowercase = FALSE
)
}
\seealso{
\code{\link{load_harx}}, \code{\link{load_sl4x}}
}
\author{
Pattawee Puangchit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_manipulation_export.R
\name{rename_dims}
\alias{rename_dims}
\title{Rename Dimensions in SL4 or HAR Objects}
\usage{
rename_dims(data_obj, mapping_df, rename_list_names = FALSE)
}
\arguments{
\item{data_obj}{A structured SL4 or HAR object.}

\item{mapping_df}{A two-column data frame where the first column (\code{old}) contains the current names,
and the second column (\code{new}) contains the new names.}

\item{rename_list_names}{Logical. If \code{TRUE}, renames list element names. Default is \code{FALSE}.}
}
\value{
The modified SL4 or HAR object with updated dimension names and, optionally, updated list names.
}
\description{
Renames dimension and list names in structured SL4 or HAR objects.
}
\details{
\itemize{
\item Replaces old dimension names with new ones as specified in \code{mapping_df}.
\item If \code{rename_list_names = TRUE}, renames list element names as well.
\item Ensures consistency across SL4 and HAR datasets.
}
}
\examples{
# Import sample data:
sl4_data <- load_sl4x(system.file("extdata", "TAR10.sl4", package = "HARplus"))

# Define a renaming map
mapping_df <- data.frame(
  old = c("REG", "COMM"),
  new = c("Region", "Commodity")
)

# Rename columns in the dataset
rename_dims(sl4_data, mapping_df)

# Rename both columns and list names
rename_dims(sl4_data, mapping_df, rename_list_names = TRUE)

}
\seealso{
\code{\link{get_data_by_var}}, \code{\link{get_data_by_dims}}
}
\author{
Pattawee Puangchit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_summary.R
\name{get_dim_patterns}
\alias{get_dim_patterns}
\title{Get Dimension Patterns from SL4 and HAR Objects}
\usage{
get_dim_patterns(..., keep_unique = FALSE)
}
\arguments{
\item{...}{One or more structured SL4 or HAR objects containing dimension information.}

\item{keep_unique}{Logical. If \code{TRUE}, returns only unique dimension patterns. Default is \code{FALSE}.}
}
\value{
A data frame containing:
\itemize{
\item \code{DimPattern}: The unique dimension patterns.
}
}
\description{
Extracts and lists unique dimension patterns (e.g., \code{REG*COMM}, \code{REG*REG*ACTS}) from one or more datasets.
}
\details{
\itemize{
\item Extracts dimension structure details from the dataset.
\item If multiple datasets are provided, combines their dimension information.
\item If \code{keep_unique = TRUE}, returns only distinct dimension patterns.
}
}
\examples{
# Import sample data:
sl4_data <- load_sl4x(system.file("extdata", "TAR10.sl4", package = "HARplus"))
sl4_data2 <- load_sl4x(system.file("extdata", "SUBT10.sl4", package = "HARplus"))

# Extract dimension patterns
get_dim_patterns(sl4_data)

# Extract only unique dimension patterns across datasets
get_dim_patterns(sl4_data, sl4_data2, keep_unique = TRUE)

}
\seealso{
\code{\link{get_dim_elements}}, \code{\link{get_var_structure}}
}
\author{
Pattawee Puangchit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_manipulation_export.R
\name{pivot_data_hierarchy}
\alias{pivot_data_hierarchy}
\title{Create Hierarchical Pivot Table from SL4 or HAR Objects}
\usage{
pivot_data_hierarchy(
  data_obj,
  pivot_cols,
  name_repair = "unique",
  export = FALSE,
  file_path = NULL,
  xlsx_filename = NULL
)
}
\arguments{
\item{data_obj}{A list or data frame. The SL4 or HAR data to pivot.}

\item{pivot_cols}{Character vector. Column names to use as pivot keys in order of hierarchy.}

\item{name_repair}{Character. Method for handling duplicate column names
(\code{"unique"}, \code{"minimal"}, \code{"universal"}). Default is \code{"unique"}.}

\item{export}{Logical. If \code{TRUE}, exports result to Excel. Default is \code{FALSE}.}

\item{file_path}{Character. Required if export = TRUE. The path for Excel export.}

\item{xlsx_filename}{Character. The name for the Excel file when using multi_sheet_xlsx. If NULL, uses the name of the dataset. Default is \code{NULL}.}
}
\value{
A pivoted data object with hierarchical structure:
\itemize{
\item If input is a data frame: Returns a hierarchical pivot table.
\item If input is a list: Returns a nested list of hierarchical pivot tables.
\item If export = TRUE: Invisibly returns the pivoted object after Excel export.
}
}
\description{
Creates hierarchical pivot tables from structured SL4 or HAR data, with optional Excel export.
Supports both single data frames and nested lists, preserving dimension hierarchies.
}
\details{
\itemize{
\item Transforms data into hierarchical pivot format with nested column headers.
\item Supports multiple pivot columns in specified order (e.g., REG > COMM).
\item Handles both single data frames and nested list structures.
\item Optional direct export to Excel with formatted hierarchical headers.
\item Uses efficient data processing with tidyr and openxlsx.
}
}
\examples{
\donttest{
# Import sample data:
sl4_data <- load_sl4x(system.file("extdata", "TAR10.sl4", package = "HARplus"))

# Extract data
data_multiple <- get_data_by_var(c("qo", "pca"), sl4_data)

# Create hierarchical pivot without export
pivot_hier <- pivot_data_hierarchy(data_multiple, 
                                   pivot_cols = c("REG", "COMM"))

# Create and export to Excel in one step
pivot_export <- pivot_data_hierarchy(data_multiple, 
                                     pivot_cols = c("REG", "COMM"),
                                     export = TRUE,
                                     file_path = file.path(tempdir(), "pivot_output.xlsx"))
}

}
\seealso{
\code{\link{pivot_data}}
}
\author{
Pattawee Puangchit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shock_cal.R
\name{create_target_config}
\alias{create_target_config}
\title{Create Target Value Configuration}
\usage{
create_target_config(
  path = NULL,
  type = NULL,
  header = NULL,
  value_col = "Value"
)
}
\arguments{
\item{path}{Optional path to the target data file.}

\item{type}{Optional file type for the target dataset ("har", "sl4", "csv", or "xlsx").}

\item{header}{Optional header name within the HAR or SL4 file to extract.}

\item{value_col}{Column name containing numeric target values. Default is "Value".}
}
\value{
A list containing:
\itemize{
\item \code{path}: file path to target data
\item \code{type}: file format (lowercase)
\item \code{header}: header name in HAR/SL4 file
\item \code{value_col}: column name for target values
}
}
\description{
Defines the configuration for loading the target dataset, which represents
post-adjustment or comparative rate values. Supports multiple file formats.
}
\details{
\itemize{
\item Supports \code{HAR}, \code{SL4}, \code{CSV}, and \code{XLSX} file formats
\item Can also represent a uniform numeric target value when no file path is provided
\item Used in combination with \code{\link{create_initial_config}} for shock computation
}
}
\examples{
# Example: Define Target Configuration
target <- create_target_config(
  path   = "D:/Data/taxrates_2019.har",
  type   = "har",
  header = "rTMS"
)
}
\seealso{
\code{\link{create_initial_config}}, \code{\link{create_calc_config}},
\code{\link{shock_calculate}}, \code{\link{shock_calculate_uniform}}
}
\author{
Pattawee Puangchit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/load_HARplus.R
\name{load_harplus}
\alias{load_harplus}
\title{Load and Process GEMPACK HAR Files (Internal)}
\usage{
load_harplus(con, coefAsname = FALSE, lowercase = TRUE, select_header = NULL)
}
\arguments{
\item{con}{Character or connection. The file path to the HAR file or an open binary connection.}

\item{coefAsname}{Logical. If \code{TRUE}, replaces four-letter headers with coefficient names when available. Default is \code{FALSE}.}

\item{lowercase}{Logical. If \code{TRUE}, converts all string values to lowercase. Default is \code{TRUE}.}

\item{select_header}{Character vector. Specific headers to extract; if \code{NULL}, reads all headers.}
}
\value{
A structured list where:
\itemize{
\item Each element corresponds to a header in the HAR file.
\item Names are either header names or coefficient names (if \code{coefAsname = TRUE}).
\item Data maintains its original dimensions and attributes.
}
}
\description{
Reads a GEMPACK HAR file and efficiently extracts structured data while maintaining
compatibility with standard HAR formats. This implementation builds upon the foundational
work of the \strong{HARr} package, reorganizing the process for improved execution speed,
memory management, and handling of sparse data structures.
}
\details{
\itemize{
\item \strong{Efficient File Reading:} Reads large HAR files in chunks for better performance.
\item \strong{Optimized Memory Usage:} Reduces unnecessary allocations and improves cleanup.
\item \strong{Streamlined Header Processing:} Ensures accurate extraction of dimension metadata.
\item \strong{Supports Sparse Data Structures:} Handles \code{RESPSE} and \code{REFULL} headers efficiently.
}

\strong{Supported HAR Header Types:}
\itemize{
\item \verb{1CFULL}: Character headers
\item \verb{2IFULL}: Integer headers
\item \verb{2RFULL}: Real headers
\item \code{REFULL}: Real headers with extended metadata
\item \code{RESPSE}: Sparse real headers
}
}
\seealso{
\code{\link{load_sl4x}}, \code{\link{load_harx}}
}
\author{
Pattawee Puangchit
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_summary.R
\name{get_var_structure}
\alias{get_var_structure}
\title{Get Variable Structure Summary from SL4 and HAR Objects}
\usage{
get_var_structure(variables = NULL, ..., include_col_size = FALSE)
}
\arguments{
\item{variables}{Character vector. Variable names to summarize. Use \code{NULL} or \code{"ALL"} to summarize all variables.}

\item{...}{One or more SL4 or HAR objects created using \code{load_sl4x()} or \code{load_harx()}.}

\item{include_col_size}{Logical. If \code{TRUE}, includes column and observation counts. Default is \code{FALSE}.}
}
\value{
A named list, where each element contains a data frame with:
\itemize{
\item \code{Variable}: The variable name.
\item \code{Dimensions}: The associated dimensions.
\item \code{DimSize}: The number of dimensions.
\item \code{DataShape}: The shape of the data (e.g., \verb{10x20x30}).
\item \code{No.Col}: (Optional) The number of columns.
\item \code{No.Obs}: (Optional) The number of observations.
}
}
\description{
Generates a summary of the variables within one or more SL4 or HAR objects, listing their
dimension sizes, structures, and optionally, column and observation counts.
}
\details{
\itemize{
\item Extracts dimension structures for variables in one or more SL4 or HAR datasets.
\item If \code{include_col_size = TRUE}, adds column and observation counts.
\item Supports multiple datasets and returns results as a named list, with each dataset’s summary stored separately.
\item Can summarize specific variables or \code{"ALL"}.
}
}
\examples{
# Import data sample:
sl4_data <- load_sl4x(system.file("extdata", "TAR10.sl4", package = "HARplus"))
sl4_data1 <- load_sl4x(system.file("extdata", "SUBT10.sl4", package = "HARplus"))

# Get summary for all variables in a single dataset
get_var_structure(data_obj = sl4_data)

# Get summary for specific variables
get_var_structure(c("gdp", "trade"), sl4_data)

# Include column and observation counts
get_var_structure("ALL", sl4_data, include_col_size = TRUE)

# Compare structures across multiple datasets
get_var_structure("ALL", sl4_data, sl4_data1)

# Include column and observation counts across multiple datasets
get_var_structure("ALL", sl4_data, sl4_data1, include_col_size = TRUE)

}
\seealso{
\code{\link{get_dim_patterns}}, \code{\link{get_dim_elements}}
}
\author{
Pattawee Puangchit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shock_cal.R
\name{shock_calculate}
\alias{shock_calculate}
\title{Calculate Shocks from Initial and Target Values}
\usage{
shock_calculate(
  initial_config,
  target_config,
  calc_config,
  output_path,
  long_desc = "Calculated shock values",
  dim_order = NULL,
  lowercase = FALSE,
  new_baseline = FALSE
)
}
\arguments{
\item{initial_config}{A list created by \code{\link{create_initial_config}}, defining:
\itemize{
\item Input path, file format, and variable header for the initial dataset
\item Column name of the initial value field ("Value_ini")
}}

\item{target_config}{A list created by \code{\link{create_target_config}}, defining:
\itemize{
\item Path, format, and variable header for the target dataset
\item Target value column ("Value_tar") or numeric target for uniform shock
}}

\item{calc_config}{A list created by \code{\link{create_calc_config}}, specifying:
\itemize{
\item \code{column_mapping}: mapping between initial and target columns
\item \code{timeline}: sequence of years or periods (e.g., "1-5")
\item \code{exclude_self_trade}: logical, whether to drop self-pairs
\item \code{exclusion_values}: list of region/sector values to omit
}}

\item{output_path}{Path to the output HAR file where calculated shocks will be written.}

\item{long_desc}{Optional text for header description. Default is "Calculated shock values".}

\item{dim_order}{Optional dimension ordering specification. Can be:
\itemize{
\item NULL (default): alphabetical A-Z ordering
\item a named list defining order for each dimension (e.g., REG, COMM)
\item a data frame or path to Excel/CSV file containing order definitions
}}

\item{lowercase}{Logical; if TRUE, converts dimension elements to lowercase. Default is FALSE.}

\item{new_baseline}{Logical; if TRUE, generates an additional HAR file with the new baseline rates
(target values where available, initial values otherwise). The output file will have "_baseline"
appended to the filename. Default is FALSE.}
}
\value{
Invisibly returns a list containing summary metadata:
\itemize{
\item \code{n_observations}: total records processed
\item \code{n_included}: records included in shock computation
\item \code{n_excluded}: records excluded by configuration
\item \code{output_path}: normalized path to output HAR file
\item \code{baseline_path}: normalized path to baseline file (if new_baseline = TRUE)
}
}
\description{
Computes compounded GEMPACK-style percentage shocks between initial and target values,
producing multi-period shock series for dynamic simulation models. The function
automatically aligns dimensions across datasets and exports results in HAR format.
}
\details{
\itemize{
\item Computes percentage shocks using compounded "power of tax" formula
\item Supports multiple periods defined via \code{timeline} configuration
\item Compatible with HAR, SL4, CSV, or XLSX input formats
\item Excludes self-trade or specified region-sector pairs when configured
\item Exports results as multi-header HAR file (one header per timeline period)
\item Optionally generates new baseline rate file showing post-adjustment values
}
}
\examples{
# Example 1: Target-Based Shock Calculation with New Baseline
har_path <- system.file("extdata", "baserate.har", package = "HARplus")

# Sorting Column
mapping <- list(
  REG = c("USA", "EU", "ROW")
)

# Initial File
initial <- create_initial_config(
  path   = har_path,
  format = "har",
  header = "rTMS"
)

# Target File
target <- create_target_config(
  path   = har_path,
  type   = "har",
  header = "rTMS"
)

# Calculation Setup with Column Mapping
calc <- create_calc_config(
  column_mapping      = c(TRAD_COMM = "TRAD_COMM", REG = "REG", REG.1 = "REG.1"),
  timeline            = "1-5",
  exclude_self_trade  = TRUE
)

# Compute Shock Based on Initial and Target Values
# Also generate new baseline file
shock_calculate(
  initial_config = initial,
  target_config  = target,
  calc_config    = calc,
  output_path    = file.path(tempdir(), "output_target.har"),
  dim_order      = mapping,
  new_baseline   = TRUE
)
}
\seealso{
\code{\link{shock_calculate_uniform}}, \code{\link{create_initial_config}},
\code{\link{create_target_config}}, \code{\link{create_calc_config}}, \code{\link{save_har}}
}
\author{
Pattawee Puangchit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_manipulation_export.R
\name{pivot_data}
\alias{pivot_data}
\title{Pivot Data from SL4 or HAR Objects}
\usage{
pivot_data(data_obj, pivot_cols, name_repair = "unique")
}
\arguments{
\item{data_obj}{A list or data frame. The SL4 or HAR data to pivot.}

\item{pivot_cols}{Character vector. Column names to use as pivot keys.}

\item{name_repair}{Character. Method for handling duplicate column names (\code{"unique"}, \code{"minimal"}, \code{"universal"}). Default is \code{"unique"}.}
}
\value{
A transformed data object where the specified \code{pivot_cols} are pivoted into wide format.
}
\description{
Transforms long-format SL4 or HAR data into wide format by pivoting selected columns.
Supports both single data frames and nested lists.
}
\details{
\itemize{
\item Uses \code{tidyr::pivot_wider()} internally to reshape data.
\item Allows multiple columns to be pivoted simultaneously.
\item Recursively processes nested lists, ensuring all data frames are transformed.
}
}
\examples{
# Import sample data:
sl4_data <- load_sl4x(system.file("extdata", "TAR10.sl4", package = "HARplus"))

# Extract multiple variables
data_multiple <- get_data_by_var(c("qo", "qxs"), sl4_data)

# Pivot a single column
pivoted_data <- pivot_data(data_multiple, pivot_cols = "REG")

# Pivot multiple columns
pivoted_data_multi <- pivot_data(data_multiple, pivot_cols = c("REG", "COMM"))

}
\seealso{
\code{\link{get_data_by_var}}, \code{\link{get_data_by_dims}}
}
\author{
Pattawee Puangchit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{pattern_match}
\alias{pattern_match}
\title{Match Patterns with Optional Mixing (Internal)}
\usage{
pattern_match(pattern1, pattern2, mix_patterns = FALSE)
}
\arguments{
\item{pattern1}{A character string representing the first pattern.}

\item{pattern2}{A character string representing the second pattern.}

\item{mix_patterns}{Logical; if \code{TRUE}, allows dimension order to be ignored during comparison.}
}
\value{
Logical; \code{TRUE} if the patterns match, \code{FALSE} otherwise.
}
\description{
Compares two patterns to determine if they match, with an option to allow flexible dimension order.
}
\details{
\itemize{
\item Performs case-insensitive pattern matching.
\item If \code{mix_patterns = TRUE}, allows patterns to match even if dimensions are in different order.
}
}
\seealso{
\code{\link{get_original_pattern}}, \code{\link{process_pattern}}, \code{\link{get_data_by_dims}}
}
\author{
Pattawee Puangchit
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_original_pattern}
\alias{get_original_pattern}
\title{Retrieve the Original Dimension Pattern (Internal)}
\usage{
get_original_pattern(pattern, data_obj, mix_patterns = FALSE)
}
\arguments{
\item{pattern}{Character. The pattern to search for in dimension structures.}

\item{data_obj}{An SL4 or HAR object containing dimension information.}

\item{mix_patterns}{Logical. If \code{TRUE}, allows dimension order to vary when matching patterns.}
}
\value{
The original dimension pattern name as a character string, or \code{NULL} if no match is found.
}
\description{
A helper function that finds the original dimension pattern name in an SL4 or HAR dataset
that matches a given pattern. Used internally in \code{get_data_by_dims()}.
}
\details{
\itemize{
\item Performs a case-insensitive comparison to identify matching dimension patterns.
\item Supports flexible pattern matching when \code{mix_patterns = TRUE}, allowing dimension order to vary.
\item Returns the standardized dimension pattern name as stored in the dataset.
}
}
\seealso{
\code{\link{pattern_match}}, \code{\link{process_pattern}}, \code{\link{get_data_by_dims}}
}
\author{
Pattawee Puangchit
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_manipulation_export.R
\name{export_data}
\alias{export_data}
\title{Export Data to Various Formats (CSV/STATA/TEXT/RDS/XLSX)}
\usage{
export_data(
  data,
  output_path,
  format = "csv",
  prefix = "",
  create_subfolder = FALSE,
  multi_sheet_xlsx = FALSE,
  xlsx_filename = NULL,
  report_output = FALSE
)
}
\arguments{
\item{data}{A list or data frame. The SL4 or HAR data to export.}

\item{output_path}{Character. The base output directory or file path.}

\item{format}{Character. The export format (\code{"csv"}, \code{"stata"}, \code{"txt"}, \code{"rds"}, \code{"xlsx"}). Default is \code{"csv"}.}

\item{prefix}{Character. An optional prefix added to exported file names. Default is \code{""} (empty).}

\item{create_subfolder}{Logical. If \code{TRUE}, creates a subfolder for each format. Default is \code{FALSE}.}

\item{multi_sheet_xlsx}{Logical. If TRUE, exports lists as multi-sheet XLSX files.}

\item{xlsx_filename}{An optional filename for the XLSX file (used when \code{multi_sheet_xlsx = TRUE}).}

\item{report_output}{Logical. If TRUE, generates an export report.}
}
\value{
A list containing the file paths of the exported data.
}
\description{
Exports structured SL4 or HAR data to multiple file formats, including CSV, Stata, TXT, RDS, and XLSX.
Supports nested lists, automatic subfolder creation, and multi-sheet Excel exports.
}
\details{
\itemize{
\item Supports exporting data in \code{"csv"}, \code{"stata"}, \code{"txt"}, \code{"rds"}, and \code{"xlsx"} formats.
\item Handles nested lists and exports each data frame individually.
\item Optionally creates subfolders for each format (\code{create_subfolder = TRUE}).
\item Customizes file names using \code{prefix}.
\item When multi_sheet_xlsx = TRUE, all exported data is stored in a \strong{single Excel workbook}, with each dataset as a separate sheet.
}

\itemize{
\item If exporting to Stata (\code{"stata"} format), column names containing \code{.} will be replaced with \verb{_} to ensure compatibility.
\item If \code{multi_sheet_xlsx = TRUE}, list elements are exported as separate sheets in a single XLSX file.
\item The function creates necessary directories if they do not exist.
}
}
\examples{
\donttest{
# Import sample data:
sl4_data <- load_sl4x(system.file("extdata", "TAR10.sl4", package = "HARplus"))

# Extract data
data_multiple <- get_data_by_var(c("qo", "pca"), sl4_data)

# Export
export_data(data_multiple, file.path(tempdir(), "output_directory"), 
           format = c("csv", "xlsx", "stata", "txt", "rds"),
           create_subfolder = TRUE,
           multi_sheet_xlsx = TRUE)
}

}
\seealso{
\code{\link{pivot_data}}, \code{\link{get_data_by_var}}, \code{\link{get_data_by_dims}}
}
\author{
Pattawee Puangchit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{process_export_report}
\alias{process_export_report}
\title{Process and Export Report (Internal)}
\usage{
process_export_report(data, output_path, prefix = "", data_name = "data")
}
\arguments{
\item{data}{A structured SL4 or HAR object or a nested list of data frames.}

\item{output_path}{A character string specifying the output directory or file path.}

\item{prefix}{A character string to prepend to the exported filenames.}
}
\description{
Generates a summary report of variables and their corresponding output files
from structured SL4 or HAR data. The report is saved as an Excel file.
}
\details{
\itemize{
\item Extracts unique variable names and their associated export filenames.
\item Handles nested list structures by recursively traversing them.
\item The output report is saved as an Excel file named \code{"Report_<data_name>.xlsx"}.
}
}
\seealso{
\code{\link{export_data}}
}
\author{
Pattawee Puangchit
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_dim_info}
\alias{get_dim_info}
\title{Extract and Organize Dimension Metadata (Internal)}
\usage{
get_dim_info(dim_info)
}
\arguments{
\item{dim_info}{A list containing dimension metadata, including:
\itemize{
\item \code{dimension_string}: A textual representation of dimensions (e.g., \code{"REG*COMM*YEAR"}).
\item \code{dimension_names}: A character vector of dimension names.
\item \code{dimension_sizes}: A numeric vector indicating the size of each dimension.
}}
}
\value{
A structured list containing:
\itemize{
\item \code{dimension_string}: The original dimension string.
\item \code{dim_size}: The number of dimensions.
\item \code{data_shape}: A formatted string representing the data shape (e.g., \code{"10x20x30"}).
\item \code{col_size}: The product of all dimension sizes except the first, representing column count.
\item \code{n_obs}: The first dimension size, typically representing the number of observations.
}
}
\description{
A helper function that extracts and structures dimension-related metadata
from a given dimension structure. Used internally in \code{get_var_structure()}
and \code{compare_var_structure()}.
}
\details{
\itemize{
\item Retrieves structured metadata for variables in SL4 and HAR datasets.
\item Computes data shape and ensures consistency in dimension structures.
\item Helps determine observation counts and column sizes for variable summaries.
}
}
\seealso{
\code{\link{get_var_structure}}, \code{\link{compare_var_structure}}
}
\author{
Pattawee Puangchit
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shock_cal.R
\name{create_initial_config}
\alias{create_initial_config}
\title{Create Initial Value Configuration}
\usage{
create_initial_config(path, format, header, value_col = "Value")
}
\arguments{
\item{path}{Path to the initial data file.}

\item{format}{File format of the initial dataset. Must be "har" or "sl4".}

\item{header}{Header name within the HAR or SL4 file to extract.}

\item{value_col}{Name of the column containing numeric values. Default is "Value".}
}
\value{
A list containing:
\itemize{
\item \code{path}: input file path
\item \code{format}: file format ("har" or "sl4")
\item \code{header}: target header name
\item \code{value_col}: column name for numeric values
}
}
\description{
Defines the configuration for loading the initial dataset, including path, format,
variable header, and value column name. Used as input for \code{\link{shock_calculate}}
and \code{\link{shock_calculate_uniform}}.
}
\details{
\itemize{
\item Supports \code{HAR} and \code{SL4} file formats
\item Specifies the header name to extract from the dataset
\item Allows custom naming for the value column (\code{Value} by default)
}
}
\examples{
# Example: Define Initial Configuration
initial <- create_initial_config(
  path   = "D:/Data/taxrates_2017.har",
  format = "har",
  header = "rTMS"
)
}
\seealso{
\code{\link{create_target_config}}, \code{\link{create_calc_config}},
\code{\link{shock_calculate}}, \code{\link{shock_calculate_uniform}}
}
\author{
Pattawee Puangchit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shock_cal.R
\name{create_calc_config}
\alias{create_calc_config}
\title{Create Calculation Configuration}
\usage{
create_calc_config(
  column_mapping = NULL,
  timeline = 1,
  exclude_self_trade = FALSE,
  exclusion_values = NULL
)
}
\arguments{
\item{column_mapping}{Optional named vector mapping initial and target columns
(e.g., \code{c(COMM = "COMM", REG = "REG", REG.1 = "REG.1")}).}

\item{timeline}{Numeric or character range defining simulation periods.
\itemize{
\item Example: "1-5" expands to 1:5.
}}

\item{exclude_self_trade}{Logical; if TRUE, removes intra-region records. Default is FALSE.}

\item{exclusion_values}{Optional named list of elements to exclude from calculation.}
}
\value{
A list containing:
\itemize{
\item \code{column_mapping}: variable mapping between datasets
\item \code{timeline}: expanded numeric sequence of simulation periods
\item \code{exclude_self_trade}: logical flag
\item \code{exclusion_values}: exclusion list by dimension
}
}
\description{
Defines calculation settings for generating shock values, including variable mapping,
timeline sequence, and exclusion criteria. Used as input for both
\code{\link{shock_calculate}} and \code{\link{shock_calculate_uniform}}.
}
\details{
\itemize{
\item Controls column alignment between initial and target datasets
\item Supports numeric or range-based timeline definitions (e.g., "1-10")
\item Excludes self-trade or specified region/sector pairs if configured
\item Provides core metadata for shock calculation functions
}
}
\examples{
# Example: Define Calculation Configuration
calc <- create_calc_config(
  column_mapping      = c(COMM = "COMM", REG = "REG", REG.1 = "REG.1"),
  timeline            = "1-5",
  exclude_self_trade  = TRUE
)
}
\seealso{
\code{\link{create_initial_config}}, \code{\link{create_target_config}},
\code{\link{shock_calculate}}, \code{\link{shock_calculate_uniform}}
}
\author{
Pattawee Puangchit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_data.R
\name{get_data_by_dims}
\alias{get_data_by_dims}
\title{Extract Data by Dimension Patterns from SL4 or HAR Objects}
\usage{
get_data_by_dims(
  patterns = NULL,
  ...,
  experiment_names = NULL,
  subtotal_level = FALSE,
  rename_cols = NULL,
  merge_data = FALSE,
  pattern_mix = FALSE
)
}
\arguments{
\item{patterns}{Character vector. Dimension patterns to extract. Use \code{"ALL"} or \code{NULL} to extract all available patterns.}

\item{...}{One or more SL4 or HAR data objects loaded using \code{load_sl4x()} or \code{load_harx()}.}

\item{experiment_names}{Character vector. Names assigned to each dataset. If \code{NULL}, names are inferred.}

\item{subtotal_level}{Character or logical. Determines which decomposition levels to retain:
\itemize{
\item \code{"total"}: Keeps only \code{"TOTAL"} values.
\item \code{"decomposed"}: Keeps only decomposed values (excludes \code{"TOTAL"}).
\item \code{"all"}: Keeps all rows.
\item \code{TRUE}: Equivalent to \code{"all"}, retaining both \code{"TOTAL"} and decomposed values.
\item \code{FALSE}: Equivalent to \code{"total"}, keeping only \code{"TOTAL"} values.
}}

\item{rename_cols}{Named vector. Column name replacements (\code{c("old_name" = "new_name")}).}

\item{merge_data}{Logical. If \code{TRUE}, attempts to merge data across multiple experiments. Default is \code{FALSE}.}

\item{pattern_mix}{Logical. If \code{TRUE}, allows flexible pattern matching, ignoring dimension order. Default is \code{FALSE}.}
}
\value{
A structured list of extracted data:
\itemize{
\item If \code{merge_data = FALSE}, returns a named list where each element corresponds to an experiment.
\item If \code{merge_data = TRUE}, returns a named list of all merged data
}
}
\description{
Retrieves structured data from SL4 or HAR objects based on specified dimension patterns.
Supports multiple experiments and merging datasets while maintaining structured dimension metadata.
}
\details{
\itemize{
\item Extracts variables matching specified dimension patterns.
\item Allows for flexible pattern matching (\code{pattern_mix = TRUE}).
\item Supports merging data across multiple experiments (\code{merge_data = TRUE}).
\item Provides column renaming functionality (\code{rename_cols}).
\item Handles subtotal filtering (\code{subtotal_level}), controlling whether \code{"TOTAL"} or decomposed values are retained.
}
}
\examples{
# Import sample data:
sl4_data <- load_sl4x(
  system.file("extdata", "TAR10.sl4", package = "HARplus")
)
sl4_data1 <- load_sl4x(
  system.file("extdata", "SUBT10.sl4", package = "HARplus")
)

# Extract data for a single dimension pattern
data_single_pattern <- get_data_by_dims(
  "comm*reg",
  sl4_data
)

# Extract multiple dimension patterns
data_multiple_patterns <- get_data_by_dims(
  c("comm*reg", "REG*ACTS"),
  sl4_data
)

# Extract all dimension patterns separately from multiple datasets
data_all_patterns <- get_data_by_dims(
  NULL,
  sl4_data, sl4_data1,
  merge_data = FALSE
)

# Merge data for identical patterns across multiple datasets
data_merged_patterns <- get_data_by_dims(
  NULL,
  sl4_data, sl4_data1,
  merge_data = TRUE
)

# Merge data while allowing interchangeable dimensions (e.g., A*B = B*A)
data_pattern_mixed <- get_data_by_dims(
  NULL,
  sl4_data, sl4_data1,
  merge_data = TRUE,
  pattern_mix = TRUE
)

# Retain only "TOTAL" values
data_total_only <- get_data_by_dims(
  "comm*reg",
  sl4_data,
  subtotal_level = "total"
)
data_total_only_alt <- get_data_by_dims(
  "comm*reg",
  sl4_data,
  subtotal_level = FALSE
)

# Retain only decomposed components
data_decomposed_only <- get_data_by_dims(
  "comm*reg",
  sl4_data,
  subtotal_level = "decomposed"
)

# Retain all value levels
data_all_decomp <- get_data_by_dims(
  "comm*reg",
  sl4_data,
  subtotal_level = "all"
)
data_all_decomp_alt <- get_data_by_dims(
  "comm*reg",
  sl4_data,
  subtotal_level = TRUE
)

# Rename specific columns
data_renamed <- get_data_by_dims(
  "comm*reg",
  sl4_data,
  rename_cols = c(REG = "Region", COMM = "Commodity")
)

# Merge data with custom experiment names
data_merged_experiments <- get_data_by_dims(
  "comm*reg",
  sl4_data, sl4_data1,
  experiment_names = c("EXP1", "EXP2"),
  merge_data = TRUE
)

}
\seealso{
\code{\link{get_data_by_var}}, \code{\link{group_data_by_dims}}
}
\author{
Pattawee Puangchit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_manipulation_export.R
\name{export_hierarchy_to_excel}
\alias{export_hierarchy_to_excel}
\title{Export Hierarchical Pivot Table to Excel (Internal)}
\usage{
export_hierarchy_to_excel(pivot_df, file_path, xlsx_filename = NULL)
}
\arguments{
\item{pivot_df}{A hierarchical pivot object created by pivot_data_hierarchy().}

\item{file_path}{Character. The file path for Excel export.}

\item{xlsx_filename}{Character. The name for the Excel file when using multi_sheet_xlsx. If NULL, uses the name of the dataset. Default is \code{NULL}.}
}
\value{
Invisibly returns NULL.
}
\description{
Internal function to export hierarchical pivot tables to Excel with formatted headers.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/load_harx.R
\name{load_harx}
\alias{load_harx}
\title{Load and Process HAR Files with Header Selection}
\usage{
load_harx(
  file_path,
  coefAsname = FALSE,
  lowercase = FALSE,
  select_header = NULL
)
}
\arguments{
\item{file_path}{Character. The file path to the HAR file.}

\item{coefAsname}{Logical. If \code{TRUE}, replaces four-letter headers with coefficient names when available. Default is \code{FALSE}.}

\item{lowercase}{Logical. If \code{TRUE}, converts all variable names to lowercase. Default is \code{FALSE}.}

\item{select_header}{Character vector. Specific headers to read; if \code{NULL}, all headers are read. Example: \code{select_header = c("A", "E1")}.}
}
\value{
A structured list containing:
\itemize{
\item \code{data}: Extracted HAR variable data stored as matrices, arrays, or vectors.
\item \code{dimension_info}: A list with:
\itemize{
\item \code{dimension_string}: A textual representation of dimensions (e.g., "REG\emph{COMM}YEAR").
\item \code{dimension_names}: The names of each dimension.
\item \code{dimension_sizes}: The size of each dimension.
}
}
}
\description{
Reads a GEMPACK HAR file and extracts structured data while maintaining compatibility
with standard HAR formats. Provides flexibility in naming conventions and header selection.
}
\details{
\itemize{
\item Uses \code{load_harplus()} internally for efficient HAR file reading.
\item Allows optional conversion of variable names to lowercase (\code{lowercase = TRUE}).
\item Supports coefficient-based naming (\code{coefAsname = TRUE}).
\item Enables selective header extraction via \code{select_header = c("A", "E1")}.
\item Returns structured data with explicit dimension names and sizes.
}
}
\examples{
# Path to example files
har_path <- system.file("extdata", "TAR10-WEL.har", package = "HARplus")

# Basic loading
har_data <- load_harx(har_path)

# Load with coefficient names
har_data_coef <- load_harx(har_path, coefAsname = TRUE)

# Load with lowercase names
har_data_lower <- load_harx(har_path, lowercase = TRUE)

# Load specific headers
har_selected <- load_harx(har_path, select_header = c("A", "E1"))

# Load with multiple options
har_combined <- load_harx(har_path,
                         coefAsname = TRUE,
                         lowercase = TRUE,
                         select_header = c("A", "E1"))

}
\seealso{
\code{\link{load_sl4x}}, \code{\link{get_data_by_var}}, \code{\link{get_data_by_dims}}
}
\author{
Pattawee Puangchit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{rename_col}
\alias{rename_col}
\title{Rename Columns in a Data Frame (Internal)}
\usage{
rename_col(df, rename_cols)
}
\arguments{
\item{df}{A data frame containing columns to be renamed.}

\item{rename_cols}{A named vector where names are existing column names,
and values are the corresponding new names.}
}
\value{
A modified data frame with renamed columns.
}
\description{
A helper function that renames columns in a data frame based on a specified mapping.
Used internally in \code{get_var_structure()}, \code{get_data_by_dims()}, and \code{rename_dims()}.
}
\details{
\itemize{
\item Replaces column names according to the provided \code{rename_cols} mapping.
\item Ensures no duplicate column names by appending numerical suffixes when necessary.
\item Helps standardize column names across SL4 and HAR datasets.
}
}
\seealso{
\code{\link{get_var_structure}}, \code{\link{get_data_by_dims}}, \code{\link{rename_dims}}
}
\author{
Pattawee Puangchit
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/load_sl4x.R
\name{load_sl4x}
\alias{load_sl4x}
\title{Load and Process SL4 Files with Enhanced Options}
\usage{
load_sl4x(file_path, lowercase = FALSE, select_header = NULL)
}
\arguments{
\item{file_path}{Character. The full path to the SL4 file to be read.}

\item{lowercase}{Logical. If \code{TRUE}, converts all variable names to lowercase. Default is \code{FALSE}.}

\item{select_header}{Character vector. Specific headers to extract; if \code{NULL}, all headers are read.}
}
\value{
A structured list containing:
\itemize{
\item \code{data}: Extracted SL4 variable data, stored as arrays or matrices.
\item \code{dimension_info}: A list with:
\itemize{
\item \code{dimension_string}: A textual representation of dimensions (e.g., "REG\emph{COMM}YEAR").
\item \code{dimension_names}: The names of each dimension.
\item \code{dimension_sizes}: The size of each dimension.
}
}
}
\description{
Reads an SL4 file and processes its structured data into an enhanced SL4 object.
Extracts structured variable information, dimensions, and handles subtotal columns.
}
\details{
\itemize{
\item Uses \code{load_harplus()} internally for optimized SL4 file reading.
\item Extracts variable names, dimension structures, and metadata.
\item Converts variable names to lowercase if \code{lowercase = TRUE}.
\item Allows the selection of specific headers using \code{select_header}.
\item Returns structured data with explicit dimension names and sizes.
}
}
\examples{
# Path to example files
sl4_path <- system.file("extdata", "TAR10.sl4", package = "HARplus")

# Basic loading
sl4_data <- load_sl4x(sl4_path)

# Load with lowercase names
sl4_data_lower <- load_sl4x(sl4_path, lowercase = TRUE)

# Load specific headers
sl4_selected <- load_sl4x(sl4_path, select_header = c("qo", "qgdp"))

}
\seealso{
\code{\link{load_harx}}, \code{\link{get_data_by_var}}, \code{\link{get_data_by_dims}}
}
\author{
Pattawee Puangchit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_summary.R
\name{get_dim_elements}
\alias{get_dim_elements}
\title{Get Dimension Elements from SL4 and HAR Objects}
\usage{
get_dim_elements(..., keep_unique = FALSE)
}
\arguments{
\item{...}{One or more structured SL4 or HAR objects containing dimension information.}

\item{keep_unique}{Logical. If \code{TRUE}, returns only unique dimension elements across inputs. Default is \code{FALSE}.}
}
\value{
A data frame containing unique dimension elements.
}
\description{
Extracts and lists unique dimension elements (e.g., \code{REG}, \code{COMM}, \code{ACTS}) from one or more datasets.
}
\examples{
# Import sample data:
sl4_data1 <- load_sl4x(system.file("extdata", "TAR10.sl4", package = "HARplus"))
sl4_data2 <- load_sl4x(system.file("extdata", "SUBT10.sl4", package = "HARplus"))


# Extract dimension elements from a single dataset
get_dim_elements(sl4_data1)

# Extract dimension elements from multiple datasets
get_dim_elements(sl4_data1, sl4_data2)

# Extract unique dimension elements across datasets
get_dim_elements(sl4_data1, sl4_data2, keep_unique = TRUE)

}
\seealso{
\code{\link{get_dim_patterns}}, \code{\link{get_var_structure}}
}
\author{
Pattawee Puangchit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shock_cal.R
\name{shock_calculate_uniform}
\alias{shock_calculate_uniform}
\title{Calculate Shocks with Uniform Adjustment}
\usage{
shock_calculate_uniform(
  initial_config,
  adjustment_value,
  calculation_method = "*",
  calc_config,
  output_path,
  long_desc = "Uniform shock adjustment",
  dim_order = NULL,
  lowercase = FALSE,
  new_baseline = FALSE
)
}
\arguments{
\item{initial_config}{A list created by \code{\link{create_initial_config}}, defining:
\itemize{
\item Input file path, format, and variable header
\item Column name for initial rate values ("Value_ini")
}}

\item{adjustment_value}{Numeric scalar specifying the uniform adjustment to apply.
\itemize{
\item Interpreted according to \code{calculation_method}
\item For example, 0.5 with method "*" halves the base rate
}}

\item{calculation_method}{Operator defining the adjustment method:
\itemize{
\item "*" multiply (default)
\item "/" divide
\item "+" add
\item "-" subtract
}}

\item{calc_config}{A list created by \code{\link{create_calc_config}}, specifying:
\itemize{
\item \code{timeline}: sequence of simulation periods (e.g., "1-10")
\item \code{exclude_self_trade}: logical, whether to omit intra-region pairs
\item \code{exclusion_values}: optional list defining excluded elements
}}

\item{output_path}{Path to the output HAR file where calculated shocks will be written.}

\item{long_desc}{Optional text for header description. Default is "Uniform shock adjustment".}

\item{dim_order}{Optional dimension ordering specification. Can be:
\itemize{
\item NULL (default): alphabetical A-Z ordering
\item a named list defining preferred order per dimension
\item a data frame or path to Excel/CSV with explicit order definitions
}}

\item{lowercase}{Logical; if TRUE, converts all dimension elements to lowercase. Default is FALSE.}

\item{new_baseline}{Logical; if TRUE, generates an additional HAR file with the new baseline rates
(target values where available, initial values otherwise). The output file will have "_baseline"
appended to the filename. Default is FALSE.}
}
\value{
Invisibly returns a list containing summary metadata:
\itemize{
\item \code{n_observations}: total records processed
\item \code{n_included}: records included in shock computation
\item \code{n_excluded}: records excluded by configuration
\item \code{output_path}: normalized path to the generated HAR file
\item \code{baseline_path}: normalized path to baseline file (if new_baseline = TRUE)
}
}
\description{
Generates GEMPACK-style percentage shocks using a uniform proportional adjustment
applied to all values in the initial dataset. The function supports additive or
multiplicative adjustments, single or multi-period configurations, and direct HAR export.
}
\details{
\itemize{
\item Applies uniform adjustment to all base rates across defined dimensions
\item Supports additive ("+", "-") and proportional ("*", "/") adjustments
\item Computes compounded shocks following the "power of tax" formulation
\item Handles multiple time periods as defined by \code{timeline} in \code{calc_config}
\item Excludes self-trade or specified region/sector pairs if configured
\item Outputs results as multi-header HAR file (one per timeline period)
\item Optionally generates new baseline rate file showing post-adjustment values
}
}
\examples{
# Example 1: Uniform Shock (50\% Reduction) with New Baseline
har_path <- system.file("extdata", "baserate.har", package = "HARplus")

# Sorting Column
mapping <- list(
  REG = c("USA", "EU", "ROW")
)

# Initial File
initial <- create_initial_config(
  path   = har_path,
  format = "har",
  header = "rTMS"
)

# Calculation Setup
calc <- create_calc_config(
  timeline           = "1-10",
  exclude_self_trade = TRUE
)

# Compute Uniform 50\% Reduction (Value_tar = Value_ini * 0.5)
# Also generate new baseline file showing the adjusted rates
shock_calculate_uniform(
  initial_config     = initial,
  adjustment_value   = 0.5,
  calculation_method = "*",
  calc_config        = calc,
  output_path        = file.path(tempdir(), "output_uniform.har"),
  dim_order          = mapping,
  new_baseline       = TRUE
)
}
\seealso{
\code{\link{shock_calculate}}, \code{\link{create_initial_config}},
\code{\link{create_calc_config}}, \code{\link{save_har}}
}
\author{
Pattawee Puangchit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{process_decomp_level}
\alias{process_decomp_level}
\title{Process Decomposition Levels in Data Frames (Internal)}
\usage{
process_decomp_level(df, subtotal_level)
}
\arguments{
\item{df}{A data frame containing a \code{"Subtotal"} column.}

\item{subtotal_level}{Character or logical. Determines which values to retain:
\itemize{
\item \code{"total"}: Keeps only \code{"TOTAL"} values.
\item \code{"decomposed"}: Keeps only decomposed values (excludes \code{"TOTAL"}).
\item \code{"all"}: Keeps all rows.
\item \code{TRUE}: Equivalent to \code{"all"} (keeps both \code{"TOTAL"} and decomposed values).
\item \code{FALSE}: Equivalent to \code{"total"} (keeps only \code{"TOTAL"} values, removing decomposed components).
}}
}
\value{
A filtered data frame based on the specified decomposition level.
}
\description{
A helper function that filters data based on decomposition levels in the \code{"Subtotal"} column.
Used internally in \code{get_data_by_var()}, \code{get_data_by_dims()}, and \code{group_data_by_dims()}.
}
\details{
\itemize{
\item If \code{subtotal_level = "total"}, \strong{keeps only \code{"TOTAL"} values}, removing all decomposed components.
\item If \code{subtotal_level = "decomposed"}, \strong{keeps only decomposed components}, removing \code{"TOTAL"}.
\item If \code{subtotal_level = "all"}, \strong{keeps both \code{"TOTAL"} and decomposed values} (no filtering).
\item If \code{subtotal_level = TRUE} (logical), it is \strong{equivalent to \code{"all"}}, meaning all values are kept.
\item If \code{subtotal_level = FALSE} (logical), it is \strong{equivalent to \code{"total"}}, meaning only \code{"TOTAL"} values are kept, and decomposed components are removed.
\item Filtering is applied \strong{only when both \code{"TOTAL"} and decomposed values exist}.
}
}
\seealso{
\code{\link{get_data_by_var}}, \code{\link{get_data_by_dims}}, \code{\link{group_data_by_dims}}
}
\author{
Pattawee Puangchit
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{process_pattern}
\alias{process_pattern}
\title{Extract and Process Pattern-Matched Variables (Internal)}
\usage{
process_pattern(pattern, data_obj, exp_name, pattern_mix = FALSE)
}
\arguments{
\item{pattern}{Character. The pattern to match against dimension structures.}

\item{data_obj}{An SL4 or HAR object containing dimension information and data.}

\item{exp_name}{Character. The experiment name assigned to the extracted data.}

\item{pattern_mix}{Logical. If \code{TRUE}, allows pattern matching to ignore dimension order.}
}
\value{
A data frame containing processed data for the matching pattern, or \code{NULL} if no matches are found.
}
\description{
A helper function that extracts and processes variables matching a specified pattern
within an SL4 or HAR data object. Used internally in \code{get_data_by_dims()}.
}
\details{
\itemize{
\item Searches for variables whose dimension structures match the given pattern.
\item Supports flexible pattern matching when \code{pattern_mix = TRUE}, allowing dimension order to vary.
\item Extracted data is converted into a tidy format, preserving dimension structures.
\item Standardizes \code{"Subtotal"} column naming for consistency across datasets.
\item Ensures only non-empty extracted variables are retained.
}
}
\seealso{
\code{\link{get_original_pattern}}, \code{\link{pattern_match}}, \code{\link{get_data_by_dims}}
}
\author{
Pattawee Puangchit
}
\keyword{internal}
